
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <allegro.h>
#include "melee.h"
REGISTER_FILE
#include "frame.h"
#include "libs.h"

extern FILE* debug_file;
#define LIST_INCREMENT 10

int BACKGROUND_COLOR = 0;

int item_cmp(const void* p1, const void* p2)
{
	return( ( ((DirtyItem*) p1)->y) -  ( ((DirtyItem*) p2)->y) );
}

void erase_pixel(DirtyItem *item, BITMAP *frame)
{
  putpixel(frame, item->x, item->y, BACKGROUND_COLOR);
}

void draw_pixel(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
  putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
}

void erase_box(DirtyItem *item, BITMAP *frame)
{
    rectfill(frame, item->x, item->y, item->x + item->a - 1, item->y + item->b - 1, BACKGROUND_COLOR);
}

void draw_box(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
  if((item->a == 1) && (item->b == 1))
    putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
  else

    blit(frame, child, 
				item->x-1, //compensating for a bug somewhere...
				item->y-1, 
				item->x-1, 
				item->y-1, 
				item->a+2, 
				item->b+2);
}

void erase_line(DirtyItem *item, BITMAP *frame)
{
  if((item->x == item->a) && (item->y == item->b))
    putpixel(frame, item->x, item->y, BACKGROUND_COLOR);
  else
    line(frame, item->x, item->y, item->a, item->b, BACKGROUND_COLOR);
}

BITMAP *line_frame;

void line_pixel(BITMAP *line_child, int x, int y, int d)
{
  putpixel(line_child, x, y, getpixel(line_frame, x, y));
}

void draw_line(DirtyItem *item, BITMAP *frame, BITMAP *child)
{
  if((item->x == item->a) && (item->y == item->b))
    putpixel(child, item->x, item->y, getpixel(frame, item->x, item->y));
  else {
    line_frame = frame;
    do_line(child, item->x, item->y, item->a, item->b, 0, line_pixel);
  }
}

Frame::Frame(int x, int y, int w, int h, int max_items) {
	background_color = pallete_color[0];
  acquire_screen();
  child = create_sub_bitmap(screen, x, y, w, h);
  frame = create_bitmap(w, h);
  release_screen();
	clear_to_color(frame, background_color);

  list_size = max_items;

  item_count = 0;
  item       = new DirtyItem[list_size];


  old_item_count = 0;

  old_item       = new DirtyItem[list_size];
}

Frame::~Frame() {
	destroy_bitmap(frame);

  delete item;
  item_count = 0;

  delete old_item;
  old_item_count = 0;
}

void Frame::enlarge_list(int increment) {
		
	DirtyItem *temp;

	temp = item;
	item = new DirtyItem[list_size + increment];
	memcpy(item, temp, item_count * sizeof(DirtyItem));
	delete temp;

	temp = old_item;
	old_item = new DirtyItem[list_size + increment];
	memcpy(old_item, temp, old_item_count * sizeof(DirtyItem));
	delete temp;

	list_size += increment;
	return;
	}

void Frame::add_to_list(int x, int y, int a, int b,
  void (*erase_item)(DirtyItem *item, BITMAP *frame),
  void (*draw_item)(DirtyItem *item, BITMAP *frame, BITMAP *child))
{
  if(item_count == list_size)
    enlarge_list(LIST_INCREMENT);

  item[item_count].x = x;
  item[item_count].y = y;
  item[item_count].a = a;
  item[item_count].b = b;

  item[item_count].erase_item = erase_item;
  item[item_count].draw_item  = draw_item;

  item_count++;
}

void Frame::add_to_old_list(int x, int y, int a, int b,
  void (*erase_item)(DirtyItem *item, BITMAP *frame),
  void (*draw_item)(DirtyItem *item, BITMAP *frame, BITMAP *child))
{
  if(old_item_count == list_size)
    enlarge_list(LIST_INCREMENT);

  old_item[old_item_count].x = x;
  old_item[old_item_count].y = y;
  old_item[old_item_count].a = a;
  old_item[old_item_count].b = b;

  old_item[old_item_count].erase_item = erase_item;
  old_item[old_item_count].draw_item  = draw_item;

  old_item_count++;
}

 void Frame::add_pixel(int x, int y, int a, int b)
{
  add_to_list(x, y, a, b, erase_pixel, draw_pixel);
}

void Frame::add_box(int x, int y, int a, int b)
{
  add_to_list(x, y, a, b, erase_box, draw_box);
}

 void Frame::add_line(int x, int y, int a, int b)
{
  add_to_list(x, y, a, b, erase_line, draw_line);
}

 void Frame::add_old_pixel(int x, int y, int a, int b)
{
  add_to_old_list(x, y, a, b, erase_pixel, draw_pixel);
}

 void Frame::add_old_box(int x, int y, int a, int b)
{
  add_to_old_list(x, 
									y, 
									a, 
									b, erase_box, draw_box);
}

 void Frame::add_old_line(int x, int y, int a, int b)
{
  add_to_old_list(x, y, a, b, erase_line, draw_line);
}

void Frame::erase()
{
  int c;
	BACKGROUND_COLOR = background_color;

  for(c = 0; c < item_count; c++) {
    item[c].erase_item(&item[c], frame);
  }

  memcpy(old_item, item, item_count * sizeof(DirtyItem));
  old_item_count = item_count;
  item_count = 0;
}

void Frame::draw()
{
  int c;
	BACKGROUND_COLOR = background_color;

  for(c = 0; c < item_count; c++)
    add_to_old_list(item[c].x, item[c].y, item[c].a, item[c].b,
      item[c].erase_item, item[c].draw_item);

  if(!gfx_driver->linear)
    qsort(old_item, old_item_count, sizeof(DirtyItem), item_cmp);

  for(c = 0; c < old_item_count; c++)
    old_item[c].draw_item(&old_item[c], frame, child);
}
