#ifndef _MELEE_H
#define _MELEE_H

#include <stdlib.h>
#include <math.h>
#ifndef PI
	#define PI 3.14159265358979323
#endif
#ifndef MAXINT
	#define MAXINT 0x7FFFFFFF
#endif

/*
	error displays an error message, and prompts the user to
	"Abort", "Retry", or "Debug"
	on "Abort", it throws 0, which will be silently caught at the closest catch (int)
	on "Retry" it returns
	on "Debug" it throws -1, which will be rethrown all the way out. or something roughly equivalent
*/
void error(const char *format, ...) ;
// like error, but no "Abort" or "Retry", just "Continue"
void tw_error(const char *format, ...) ;
int tw_alert(char *message, char *b1, char *b2 = 0, char *b3 = 0) ;
extern char dialog_string[20][128];
extern int __error_flag;



//Allego
struct FONT;
struct SAMPLE;
struct BITMAP;
struct DATAFILE;
//sorta Allegro
struct JGMOD;

typedef SAMPLE Sound;
typedef JGMOD Music;

class Logger;

extern DATAFILE *scppal;

#define MELEE_ASTEROID          0
#define MELEE_ASTEROIDEXPLOSION 64
#define MELEE_BOOM              84
#define MELEE_BOOMSHIP          88
#define MELEE_HOTSPOT           89
#define MELEE_KABOOM            101
#define MELEE_MUSIC             121
#define MELEE_PANEL             122
#define MELEE_PLANET            125
#define MELEE_SPARK             129
#define MELEE_STAR              135

#define GLOBAL_MAXSPEED 1.0

#define SCPPAL_PALLETE 0
#define SCPPAL_FONT    4

extern double X_MAX;
extern double Y_MAX;

#define SPACE_LAYERS 7

#define LAYER_HOTSPOTS   0
#define LAYER_CBODIES    1
#define LAYER_LINES      2
#define LAYER_SHOTS      3
#define LAYER_SHIPS      4
#define LAYER_SPECIAL    5
#define LAYER_EXPLOSIONS 6

#define ALL_LAYERS ( (1<<SPACE_LAYERS) - 1)
#define OBJECT_LAYERS ( (1<<SPACE_LAYERS) - 1 - (1<<LAYER_LINES) )
#define LINE_LAYERS ( (1<<LAYER_LINES) )

#define MAX_SPACE_ITEMS 1024

#define MAX_FLEET_SIZE 80
#define MAX_SHIP_ID    6
#define EMPTY_SLOT     -1

extern char *key_string[];
int key_num (const char *name);
int get_key ();

#define CAPTAIN_WIDTH   55
#define CAPTAIN_HEIGHT  30

#define bit(a) (1 << (a))

#define BOOM_SAMPLES 4


#define ASTEROID_FRAMES          64
#define ASTEROIDEXPLOSION_FRAMES 20
#define HOTSPOT_FRAMES           12
#define KABOOM_FRAMES            20
#define SPARK_FRAMES             6
#define PLANET_FRAMES            3
#define PANEL_FRAMES             3

#define HOT_COLORS 12

#define PHASE_MAX   12
#define PHASE_DELAY 50

extern int hot_color[HOT_COLORS];

//melee/mframe.h
class Physics;
class SpaceSprite;
class SpaceLocation;
class SpaceObject;
class SpaceLine;

//melee/mcontrol.h
class Control;
class ControlWrapper;

//melee/mship.h, melee/mshpdata, melee/mshppan.h
class Ship;
class ShipData;
class ShipPanel;

//melee/mlog.h  //depends on nothing
class Histograph;
class Log;

//melee/mview.h
class View;

//frame.h
struct DirtyItem;
class Frame;

//util/ppcol.h
struct PPCOL_MASK;

//melee/mshot.
class Shot;
class AnimatedShot;
class Missile;
class HomingMissile;
class Laser;

//melee/mcbodies.
class Animation;
class Asteroid;
class Planet;

//
extern int FULL_REDRAW;

#define ANGLE_RATIO     (3.1415926535 / 180.0)//radians per degree
#define RANGE_RATIO     40.0					//???

extern double distance_ratio;
extern int time_ratio;

//melee/mplayer.cpp
class Player;

//melee/mgame.cpp
class Game;
//blah
class NormalGame; class GobGame;
extern class Physics *physics;
extern class Game *&game;
extern class NormalGame *&normalgame;
extern class GobGame *&gobgame;

//melee/mhelpers.cpp


extern volatile int debug_value;

void init_time();
//void uninit_time();
int get_time();

int scale_frames(double value);
double scale_range(double range);
double scale_turning (double turn_rate) ;
double scale_velocity (double velocity) ;
double scale_acceleration (double acceleration, double hotspot_rate) ;

void show_file(const char *file) ;
void show_text(const char *text) ;

class SoundSystem {
	int sound_initialized;
	int compatible_soundcard;
	int sound_channels, music_channels;
	public:
	int sound_volume;
	int music_volume;
	int sound_on;
	int music_on;
	SoundSystem() ;
	void disable() ;
	void load() ;
	void save() ;
	void init() ;
	int play (SAMPLE *spl, int vol = 256, int pan = 128, int freq = 1000) ;
	void stop (SAMPLE *spl);
	void play_music (Music *mus, int loop = false);
	int is_music_playing();
	void stop_music ();
	void set_volumes() ;
	} extern tw_sound;
#define sound tw_sound

#define random  tw_random
int tw_random();
int tw_random_state_checksum();
void tw_random_seed(int seed);
void tw_random_seed_more(int seed);

struct shiptype_type {
	const char *id;
	Ship *getShip (double x, double y, double angle, unsigned int code);
	Ship *(*_getShip)(double x, double y, double angle, ShipData *data, unsigned int code);
	ShipData           *dat;
	int                cost;
	const char *name;
	};
shiptype_type *shiptype(const char *shiptype_id) ;
extern shiptype_type *shiptypes;
extern int num_shiptypes;
class ShipRegistration {
	public:
	ShipRegistration (const char *id, const char *source_name, 
			Ship *(*getShip)(double x, double y, double a, ShipData *data, unsigned int code));
	};
class AutoShipRegistration {
	public:
	AutoShipRegistration (const char *source_name, 
			Ship *(*getShip)(double x, double y, double a, ShipData *data, unsigned int code)
			);
	};
#define REGISTER_SHIP_EX(a,b) static ShipRegistration ignore_me_rs(a, __FILE__, b);
#define REGISTER_SHIP(ship) static Ship *getShip(double x, double y, double a, ShipData *d, unsigned int c){return(new ship(x,y,a,d,c));}static AutoShipRegistration ignore_me_rs(__FILE__, &getShip);
struct registered_file_type {
	char *fname;
	char *fdate;
	char *ftime;
	};
extern registered_file_type *registered_files;
extern int num_registered_files;
class FileRegistration {
	public:
	FileRegistration (char *fname, char *fdate, char *ftime);
	};
#define REGISTER_FILE static FileRegistration ignore_me_rf(__FILE__, __DATE__, __TIME__);
void init_ships();
void load_ship_data(shiptype_type *type) ;
void unload_ship_data(shiptype_type *type) ;
void load_all_ship_data() ;
void unload_all_ship_data() ;


//melee/mfleet.cpp
extern char fleet_title[];
extern int  fleet_cost;
extern int  fleet_size;
extern const char *fleet_id[MAX_FLEET_SIZE];
extern const char *fleet_name[MAX_FLEET_SIZE];
void init_fleet();
void select_fleet_slot(int slot, shiptype_type *type);
void clear_fleet_slot(int slot);
void save_fleet(const char *filename, const char *section);
void load_fleet(const char *filename, const char *section);
void *serialize_fleet(int *size) ;
void deserialize_fleet(void *data, int size) ;




class SpaceSprite {
	protected:
	int         count;
	char bpp;
	int originaltype;
	int         w;
	int         h;
	PPCOL_MASK **m;
	BITMAP     **b;

	char *type;
	int references;
	char *attributes;
	enum { DEALLOCATE_IMAGE = 0x01, DEALLOCATE_MASK };
	unsigned int general_attributes;
	public:
	enum { MASKED = 0x01, IRREGULAR = 0x02 };
	//sprite_count * rotation new images based upon sprite_count RLE_SPRITES, with gamma correction
	SpaceSprite(const DATAFILE *sprites, int sprite_count, int attributes = MASKED, int rotations = 1);
	SpaceSprite(SpaceSprite &old);
//	SpaceSprite(const char *sourcename, const char *spritename);

	virtual ~SpaceSprite();

	BITMAP     *get_bitmap(int index);

	void overlay ( int index1, int index2, BITMAP *dest);

	void draw(double x, double y, double w, double h, int index, Frame *frame) ;
	//void draw(double x, double y, double w, double h, int index, BITMAP *bmp) ;
	//void draw(double x, double y, int index, Frame *frame) ;
	//void draw(double x, double y, int index, BITMAP *bmp) ;
	//void draw(int x, int y, int w, int h, int index, Frame *frame) ;
	//void draw(int x, int y, int w, int h, int index, BITMAP *bmp) ;
	//void draw(int x, int y, int index, Frame *frame) ;
	void draw(int x, int y, int index, BITMAP *bmp) ;

	void animate(double x, double y, int index, Frame *space);
	void animate_character(double x, double y, int index, int color, Frame *space);

	void draw_character(int x, int y, int index, int color, BITMAP *bmp);
	void draw_character_stretch(int x, int y, int w, int h, int index, int color, BITMAP *bmp);

	virtual void change_color_depth(int dest);

	int collide(int x, int y, int i, int ox, int oy, int oi,
			SpaceSprite *other);
	int collide_line(int lx1, int ly1, int *lx2, int *ly2, int sx, int sy,
			int sindex);

	int    frames();
	double width();
	double height();
	};



char *shipListboxGetter(int index, int *list_size);
char *fleetListboxGetter(int index, int *list_size);


double atan3(double y, double x) ;
double normalize(double value, double max);
double wcorner(double x, double w);
double hcorner(double y, double h);
double min_delta(double from, double to, double max);
double nearest_coord(double from, double to, double max);
double distance_from(double x, double y, double to_x, double to_y);


class ShipData {
	public:
	shiptype_type *type;
	int references;
	DATAFILE    *data;
//	int num_panel_bitmaps;
//	BITMAP     **bitmapPanel;
	SpaceSprite *spritePanel;
	SpaceSprite *spriteShip;
	SpaceSprite *spriteWeapon;
	SpaceSprite *spriteWeaponExplosion;
	SpaceSprite *spriteSpecial;
	SpaceSprite *spriteSpecialExplosion;
	SpaceSprite *spriteExtra;
	SpaceSprite *spriteExtraExplosion;
	int num_weapon_samples;
	SAMPLE     **sampleWeapon;
	int num_special_samples;
	SAMPLE     **sampleSpecial;
	int num_extra_samples;
	SAMPLE     **sampleExtra;

	Music       *moduleVictory;

	ShipData(char *filename);
	~ShipData();
	};

#endif
