
#include "../melee.h"
REGISTER_FILE

#include "mgame.h"
#include "manim.h"

Animation::Animation(SpaceLocation *creator, double ox, double oy, 
	SpaceSprite *osprite, int first_frame, int num_frames, int frame_length, 
	double depth) 
	:
	SpaceObject(creator, ox, oy, 0.0, osprite),
	frame_count(num_frames),
	frame_size(frame_length),
	frame_step(frame_length)
	{
	sprite_index = first_frame;
	layer = LAYER_HOTSPOTS;
	set_depth(depth);
	collide_flag_anyone = collide_flag_sameteam = collide_flag_sameship = 0;
	mass = 0;
	}

void Animation::calculate() {
	frame_step -= frame_time;
	while (frame_step < 0) {
		frame_step += frame_size;
		sprite_index += 1;
		if (sprite_index == sprite->frames()) sprite_index = 0;
		frame_count -= 1;
		if (!frame_count) state = 0;
		}
	SpaceObject::calculate();
	}

FixedAnimation::FixedAnimation(SpaceLocation *creator, SpaceLocation *opos, 
	SpaceSprite *osprite, int first_frame, int num_frames, int frame_length, 
	int olayer) 
	:
	Animation(creator, 0.0, 0.0, osprite, first_frame, num_frames, frame_length, olayer),
	pos(opos)
	{
	if(pos->exists()) {
		x = pos->normal_x();
		y = pos->normal_y();
		} 
	else state = 0;
	}

void FixedAnimation::calculate() {
	if (pos->exists()) {
		x = pos->normal_x();
		y = pos->normal_y();
		Animation::calculate();
		} 
	else state = 0;
	}

PositionedAnimation::PositionedAnimation(SpaceLocation *creator, 
	SpaceLocation *opos, double ox, double oy, SpaceSprite *osprite, int first_frame,
	int num_frames, int frame_length, int olayer)
	:
	FixedAnimation(creator, opos, osprite, first_frame, num_frames, frame_length, olayer) 
	{
	rx = ox; ry = oy; pos = opos;
	double tx = cos(pos->get_angle()*ANGLE_RATIO), ty = sin(pos->get_angle()*ANGLE_RATIO);
	x += ry*tx - rx*ty;
	y += ry*ty + rx*tx;
	}

void PositionedAnimation::calculate() {
	FixedAnimation::calculate();
	if (state == 0) return;
	double tx = cos(pos->get_angle()*ANGLE_RATIO), ty = sin(pos->get_angle()*ANGLE_RATIO);
	x += ry*tx - rx*ty;
	y += ry*ty + rx*tx;
	}
