
#include <allegro.h>
#include "../melee.h"
REGISTER_FILE
#include "../id.h"
#include "../scp.h"

#include "mgame.h"
#include "mview.h"
#include "mcbodies.h"
#include "manim.h"
#include "mship.h"



Asteroid::Asteroid() 
	:
	SpaceObject(NULL, 0, 0, 0, game->asteroidSprite), explosion(game->asteroidExplosionSprite)
	{
	collide_flag_sameteam = ALL_LAYERS;
	collide_flag_sameship = ALL_LAYERS;
	layer = LAYER_CBODIES;
	id    |= SPACE_ASTEROID;
	step  = 0;
	speed = ((random() % 3) + 1);
	mass  = 10 + (random() % 10);

	x = (double)(random() % (int)(X_MAX));
	y = (double)(random() % (int)(Y_MAX));
	angle = (double)(random() % 360);
	vx = speed * cos(angle * ANGLE_RATIO) * 0.05;
	vy = speed * sin(angle * ANGLE_RATIO) * 0.05;
	}

void Asteroid::calculate()
{
  step-= frame_time;
  while(step <= 0) {
    step += speed * time_ratio;
    sprite_index++;
    if(sprite_index == ASTEROID_FRAMES)
      sprite_index = 0;
  }

  SpaceObject::calculate();
}

void Asteroid::handle_damage(SpaceLocation *source) {
	if (!exists()) return;
	if (!damage && !directDamage) return;

	game->addItem(new Animation(this, x, y,
			explosion, 0, explosion->frames(), time_ratio, LAYER_CBODIES));
	damage = 0;
	directDamage = 0;
	state = 0;
	}

void Asteroid::death() {
	game->addItem(new Asteroid());
	return;
	}


Planet::Planet(double x, double y, SpaceSprite *sprite, int index) 
	:
	SpaceObject(NULL, x, y, 0.0, sprite) 
	{
	collide_flag_sameship = ALL_LAYERS;
	layer = LAYER_CBODIES;
	id         |= SPACE_PLANET;
	mass        = 9999999.0;
//use remote .ini file
	game->log_file ("server.ini");
	sprite_index = index;
	gravity_mindist = scale_range(get_config_float("Planet", "GravityMinDist", 0));
	gravity_range = scale_range(get_config_float("Planet", "GravityRange", 0));
	gravity_power = get_config_float("Planet", "GravityPower", 0);
	gravity_force = 
		scale_acceleration(get_config_float("Planet", "GravityForce", 0), 0);
	gravity_whip = get_config_float("Planet", "GravityWhip", 0);

	}

void Planet::inflict_damage(SpaceObject *other) {
	int i = 1;
	if (other->isShip()) {
		i = (int) ceil(((Ship*)other)->crew / 3.0);
		}
	if (other->mass == 0) other->state = 0;
	other->directDamage += i;
	i /= 2;
	if (i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
	if (!other->isShot()) 
		play_sound((SAMPLE *)(melee[MELEE_BOOM + i].dat));
	other->handle_damage(this);
	}

void Planet::calculate() {
	SpaceObject::calculate();
	SpaceObject *o;
	Query a;
	a.begin(this, OBJECT_LAYERS, gravity_range);
	for (;a.currento;a.next()) {
		o = a.currento;
		if (o->mass > 0) {
			double r = distance(o);
			if (r < gravity_mindist) r = gravity_mindist;
			double sr = 1;
//gravity_power truncated here
			if (gravity_power > 0) {
				r /= 40 * 4;
				for (int i = 0; i < gravity_power; i += 1) sr *= r;
				o->accelerate(this, trajectory_angle(o) + 180, frame_time * gravity_force / sr, GLOBAL_MAXSPEED);
				}
			else {
				r = 1 - r/gravity_range;
				for (int i = 0; i < -gravity_power; i += 1) sr *= r;
				o->accelerate(this, trajectory_angle(o) + 180, frame_time * gravity_force * sr, GLOBAL_MAXSPEED);
				}
			}
		}
	/*for (;a.currento;a.next()) {
		o = a.currento;
		if (o->mass > 0) {
			double r = distance(o);
			if (r < gravity_mindist) r = gravity_mindist;
			r = 1 - r / gravity_range;
			o->accelerate(this, trajectory_angle(o) + 180, frame_time * gravity_force * r / 40000., GLOBAL_MAXSPEED);
			}
		}*/
	return;
	}
