

#include <allegro.h>
#include <string.h>

#include "../melee.h"
REGISTER_FILE
#include "../id.h"
#include "../frame.h"
#include "../libs.h"

#include "mframe.h"
#include "mgame.h"
#include "manim.h"
#include "mview.h"
#include "mship.h"
#include "mcbodies.h"

#define ANIMATE_BUFFER_SIZE 2048

#define DEATH_FRAMES 3

#define QUADS_X 16
#define QUADS_Y 16
#define QUADS_TOTAL (QUADS_X * QUADS_Y)
#define QUAD_X (X_MAX / QUADS_X)
#define QUAD_Y (Y_MAX / QUADS_Y)
#define QUADI_X (1.0 / QUAD_X)
#define QUADI_Y (1.0 / QUAD_Y)

Physics *physics = NULL;
Game *&game = (Game *&) physics;
NormalGame *&normalgame = (NormalGame *&) physics;
GobGame *&gobgame = (GobGame *&) physics;

int frame_time = 1;
DATAFILE *melee = NULL;
double X_MAX = 42;
double Y_MAX = 69;


//#define current_invalid1() (!(bit(current->layer) & layers) || (current == target) || (target && (target->distance(current) > range)) || !current->exists())
void Query::begin (SpaceLocation *qtarget, int qlayers, double qrange) {
	layers = qlayers;
	range = qrange;
	target = qtarget;
	current = NULL;
	tx = target->normal_x();
	ty = target->normal_y();
	if ((qrange < X_MAX/2.5) && (qrange < Y_MAX/2.5)) {
		qx_min = (int)floor((target->normal_x() - qrange) * QUADI_X);
		qx_max = (int)ceil ((target->normal_x() + qrange) * QUADI_X) + 1;
		if (qx_min < 0) qx_min += QUADS_X;
		if (qx_max >= QUADS_X) qx_max -= QUADS_X;
		qy_min = (int)floor((target->normal_y() - qrange) * QUADI_Y);
		qy_max = (int)ceil ((target->normal_y() + qrange) * QUADI_Y) + 1;
		if (qy_min < 0) qy_min += QUADS_Y;
		if (qy_max >= QUADS_Y) qy_max -= QUADS_Y;
		}
	else {
		qx_min = 0;
		qx_max = 0;
		qy_min = 0;
		qy_max = 0;
		}
	qy = qy_min;
	qx = qx_min;
	current = physics->quadrant[qy * QUADS_X + qx];
	if (!current) next_quadrant();
	if (!current) return;
	if (current_invalid()) next();
	return;
	}

void Query::begin (double x, double y, int qlayers, double qrange) {
	layers = qlayers;
	range = qrange;
	target = NULL;
	current = NULL;
	tx = x;
	ty = y;
	if ((qrange < X_MAX/2.5) && (qrange < Y_MAX/2.5)) {
		qx_min = (int)floor((tx - qrange) * QUADI_X);
		qx_max = (int)ceil ((tx + qrange) * QUADI_X) + 1;
		if (qx_min < 0) qx_min += QUADS_X;
		if (qx_max >= QUADS_X) qx_max -= QUADS_X;
		qy_min = (int)floor((ty - qrange) * QUADI_Y);
		qy_max = (int)ceil ((ty + qrange) * QUADI_Y) + 1;
		if (qy_min < 0) qy_min += QUADS_Y;
		if (qy_max >= QUADS_Y) qy_max -= QUADS_Y;
		}
	else {
		qx_min = 0;
		qx_max = 0;
		qy_min = 0;
		qy_max = 0;
		}
	qy = qy_min;
	qx = qx_min;
	current = physics->quadrant[qy * QUADS_X + qx];
	if (!current) next_quadrant();
	if (!current) return;
	if (current_invalid()) next();
	return;
	}


void Query::next_quadrant () {
tail_recurse4:
	qx += 1;
	if (qx == QUADS_X) qx = 0;
	if (qx == qx_max) {
		qy += 1;
		if (qy == QUADS_Y) qy = 0;
		if (qy == qy_max) {
			current = NULL;
			qy -= 1;
			qx -= 1;
			return;
			}
		qx = qx_min;
		}
	int tmp = qy * QUADS_X + qx;
	if (tmp < 0) error ("tmp was less than 0");
	if (tmp > QUADS_TOTAL) error ("tmp was too large");
	current = physics->quadrant[qy * QUADS_X + qx];
	if (!current) goto tail_recurse4;
	return;
	}

void Query::next () {
tail_recurse3:
	current = current->qnext;
	if (!current) {
		next_quadrant();
		if (!current) return;
		}
	if (current_invalid()) goto tail_recurse3;
	return;
	}

void Query::end() {
	}
//#undef current_invalid

SpaceLocation::~SpaceLocation() {
	if (data) data->references -= 1;
	}

Presence::Presence() {
	synchronization = NO_SYNCHRONIZATION;
  state = 1;
	}

void Presence::animate(Frame *space) {
	return;
	}
void Presence::animate_predict(Frame *space, int time) {
	animate(space);
	}
void Presence::calculate() {
	}
int Presence::exists() {
	return (state > 0);
	}
Presence::~Presence() {
	}
SpaceLocation *Presence::get_focus() {
	return NULL;
	}
SpaceLocation *SpaceLocation::get_focus() {
	return this;
	}

SpaceLocation::SpaceLocation(SpaceLocation *creator, double lx, double ly, double langle) :
	id(SPACE_LOCATION),
	x(lx),
	y(ly),
	angle(langle),
	vx(0.0),
	vy(0.0),
	layer(0),
	depth(-1),
	damage_factor(0),
	collide_flag_anyone(ALL_LAYERS),
	collide_flag_sameteam(0),
	collide_flag_sameship(0),
	damage(0),
	directDamage(0),
	repair(0),
	fuel_sap(0),
	speed_loss(0.0), 
	qnext(NULL)
{
	synchronization = PASSIVE_SYNCHRONIZATION;
	if (creator) {
		ally_flag = creator->ally_flag;
		ship = creator->ship;
		data = creator->data;
		target = creator->target;
		}
	else {
		ally_flag = 0;
		ship = NULL;
		data = NULL;
		target = NULL;
		}
}

bool SpaceLocation::change_owner(SpaceLocation *new_owner) {
	if (new_owner) {
		ally_flag = new_owner->ally_flag;
		ship = new_owner->ship;
		target = new_owner->target;
		}
	else {
		ally_flag = 0;
		ship = NULL;
		target = NULL;
		}
	return true;
	}

void SpaceLocation::death() {
	}

double SpaceLocation::get_angle()
{
	return(normalize(angle, 360.0));
}

double SpaceLocation::get_vx()
{
  return(vx);
}

double SpaceLocation::get_vy()
{
  return(vy);
}

int SpaceLocation::getID()
{
  return id;
}

double SpaceLocation::normal_x()
{
  return(normalize(x, X_MAX));
}

double SpaceLocation::normal_y()
{
  return(normalize(y, Y_MAX));
}

double SpaceLocation::nearest_x(SpaceLocation *l)
{
  return(nearest_coord(normal_x(), l->normal_x(), X_MAX));
}

double SpaceLocation::nearest_y(SpaceLocation *l)
{
  return(nearest_coord(normal_y(), l->normal_y(), Y_MAX));
}

double SpaceLocation::distance(SpaceLocation *l)
{
  return(distance_from(normal_x(), normal_y(), l->normal_x(), l->normal_y()));
}

void SpaceLocation::handle_damage(SpaceLocation *source) {
	return;
	}

void SpaceLocation::ship_died() {
	ship = NULL;
	}
void SpaceLocation::target_died() {
	target = NULL;
	}

double SpaceLocation::trajectory_angle(SpaceLocation *l) {
	double dx = min_delta(l->normal_x(), normal_x(), X_MAX);
	double dy = min_delta(l->normal_y(), normal_y(), Y_MAX);

	double alpha = atan3(dy, dx) * (180 / PI);
	alpha = normalize(alpha, 360.0);
	return(alpha);
	}

int SpaceLocation::canCollide(SpaceLocation *other) {
	if (sameShip(other)) return ((1 << other->layer) & collide_flag_sameship);
	else if (sameTeam(other)) return ((1 << other->layer) & collide_flag_sameteam);
	return ((1 << other->layer) & collide_flag_anyone);
	}

TeamCode SpaceLocation::get_team() {
	return (ally_flag & team_mask) >> team_shift;
	}

bool SpaceLocation::sameTeam(SpaceLocation *other) {
	return !((ally_flag ^ other->ally_flag) & (team_mask));
	}

int SpaceLocation::isLine() {
	return ((id & BASE_MASK1) == SPACE_LINE);
	}

int SpaceLocation::isObject() {
	return ((id & BASE_MASK1) == SPACE_OBJECT);
	}

int SpaceLocation::isPlanet() {
	return (id == SPACE_PLANET);
	}

int SpaceLocation::isAsteroid() {
	return (id == SPACE_ASTEROID);
	}

int SpaceLocation::isShip() {
	return ((id & BASE_MASK2) == SPACE_SHIP);
	}

int SpaceLocation::isShot() {
	return ((id & BASE_MASK2) == SPACE_SHOT);
	}


int SpaceLocation::isProtected() {
	return false;
	}

int SpaceLocation::isInvisible() {
	return false;
	}

void SpaceLocation::set_depth(double d) {
	depth = int((d+64) * 16384) + (random() & 31);
	}

double SpaceLocation::get_depth() {
	return (depth/16384.)-64;
	}

Planet *SpaceLocation::nearest_planet() {
	Planet *p = NULL;
	double r = 99999999;
	Query q;
	q.begin(this, bit(LAYER_CBODIES), 1600);
	while (q.current) {
		if (q.current->isPlanet()) {
			double t = distance(q.current);
			if (t < r) {
				r = t;
				p = (Planet *) q.current;
				}
			}
		q.next();
		}
	return p;
	}
void SpaceLocation::play_sound (SAMPLE *sample, int vol, int freq) {
	physics->play_sound(sample, this, vol, freq);
	return;
	}
void SpaceLocation::play_sound2 (SAMPLE *sample, int vol, int freq) {
	physics->play_sound2(sample, this, vol, freq);
	return;
	}
bool SpaceLocation::die() {
	if (!exists()) error("SpaceLocation::die - already dead");
	state = 0;
	return true;
	}
int SpaceLocation::translate( double dx, double dy) {
	x = normalize (x + dx, X_MAX);
	y = normalize (y + dy, Y_MAX);
	return true;
	}

int SpaceLocation::accelerate(SpaceLocation *source, double angle, double velocity, double max_speed) {
	double ov, nv;
	double nvx, nvy;

	ov = (vx * vx) + (vy * vy);
	nvx = vx + (cos(angle * ANGLE_RATIO) * velocity);
	nvy = vy + (sin(angle * ANGLE_RATIO) * velocity);
	nv = (nvx * nvx) + (nvy * nvy);
	if ((nv <= max_speed * max_speed) || (nv <= ov)) {
		vx = nvx;
		vy = nvy;
		}
	else {
		if (ov <= max_speed * max_speed) ov = max_speed;
		else ov = sqrt(ov);
		vx = nvx * ov / (ov + velocity);
		vy = nvy * ov / (ov + velocity);
		}
	return true;
	}

int SpaceLocation::accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, double max_speed) {
	Planet *p = nearest_planet();
	if (!p) return SpaceLocation::accelerate(source, angle, velocity, max_speed);
	double tmp;
	tmp = distance(p) / p->gravity_range;
	if (tmp > 1) return SpaceLocation::accelerate(source, angle, velocity, max_speed);
	return SpaceLocation::accelerate(source, angle, velocity, max_speed * (p->gravity_whip * tmp + 1));
	}

void SpaceLocation::animate(Frame *space) {
	return;
	}
void SpaceLocation::animate_predict(Frame *space, int time) {
	double ox = x, oy = y;
	x += vx * time;
	y += vx * time;
	animate(space);
	x = ox;
	y = oy;
	return;
	}
void SpaceLocation::calculate() {
	if (target && !target->exists()) {
		target_died();
		}
	if (ship && !ship->exists()) {
		ship_died();
		}
	return;
	}

SpaceObject::SpaceObject(SpaceLocation *creator, double ox, double oy, 
	double oangle, SpaceSprite *osprite) 
	:
	SpaceLocation(creator, ox, oy, oangle),
	w(osprite->width()),
	h(osprite->height()),
	mass(0),
	sprite(osprite),
	sprite_index(0)
	{
	if (game->friendly_fire) collide_flag_sameteam = ALL_LAYERS;
	id = SPACE_OBJECT;
	}

int SpaceObject::getSpriteIndex()
{
  return(sprite_index);
}
 
SpaceSprite* SpaceObject::getSprite()
{
  return( sprite );
}

double SpaceObject::width()
{
  return(sprite->width());
}

double SpaceObject::height() {
	return(sprite->height());
	}

void SpaceObject::animate(Frame *space) {
	sprite->animate(x, y, sprite_index, space);
	return;
	}

void SpaceObject::collide(SpaceObject *other) {
	double dx, dy;
	double dvx, dvy;
	double tmp;

	int x1, y1;
	int x2, y2;

	if((!canCollide(other)) || (!other->canCollide(this))) return;

	x1 = (int)(normal_x() - (w / 2.0));
	y1 = (int)(normal_y() - (h / 2.0));
	dx = min_delta(normal_x(), other->normal_x(), X_MAX);
	dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
	x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
	y2 = (int)(normal_y() - dy - ((other->h) / 2.0));

	if (!sprite->collide(x1, y1, sprite_index, x2, y2, 
			other->sprite_index, other->sprite)) 
		return;

	inflict_damage(other);
	other->inflict_damage(this);

	if (!exists() || !other->exists()) return;
	if (!mass || !other->mass) return;

	dvx = vx - other->vx;
	dvy = vy - other->vy;

	x1 = (int)(normal_x() - (w / 2.0));
	y1 = (int)(normal_y() - (h / 2.0));
	dx = min_delta(normal_x(), other->normal_x(), X_MAX);
	dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
	x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
	y2 = (int)(normal_y() - dy - ((other->h) / 2.0));

	while ((dx == 0) && (dy == 0)) {
		dx = ((random() % 5) - 2) / 99.0;
		dy = ((random() % 5) - 2) / 99.0;
		}
	double ndx, ndy;
	ndx = dx / (mass + other->mass) / sqrt(dx * dx + dy * dy);
	ndy = dy / (mass + other->mass) / sqrt(dx * dx + dy * dy);
	while (sprite->collide(x1, y1, sprite_index, x2, y2, 
			other->sprite_index, other->sprite)) {
		x += ndx * other->mass;
		y += ndy * other->mass;
		other->x -= ndx * mass;
		other->y -= ndy * mass;
		x1 = (int)(normal_x() - (w / 2.0));
		y1 = (int)(normal_y() - (h / 2.0));
		dx = min_delta(normal_x(), other->normal_x(), X_MAX);
		dy = min_delta(normal_y(), other->normal_y(), Y_MAX);
		x2 = (int)(normal_x() - dx - ((other->w) / 2.0));
		y2 = (int)(normal_y() - dy - ((other->h) / 2.0));
		}
	tmp = dvx * dx + dvy * dy;
	tmp = ( -2 * tmp / (dx * dx + dy * dy));
	tmp = tmp * (mass * other->mass) / (mass + other->mass);
	if (tmp < 0) return;
	vx += dx * tmp / mass;
	vy += dy * tmp / mass;
	other->vx -= dx * tmp / other->mass;
	other->vy -= dy * tmp / other->mass;
	return;
	}

double SpaceObject::collide_line(double lx1, double ly1, double lx2,
  double ly2, double llength)
{
  int collide_x = (int)(lx2);
  int collide_y = (int)(ly2);
  double dx, dy;

  if (sprite->collide_line(
			(int)(lx1), (int)(ly1), &collide_x, &collide_y,
			(int)(lx1 - min_delta(lx1, normal_x(), X_MAX)),
			(int)(ly1 - min_delta(ly1, normal_y(), Y_MAX)), 
			sprite_index)) {
		dx = lx2 - (double)(collide_x);
    dy = ly2 - (double)(collide_y);
    llength = llength - sqrt((dx * dx) + (dy * dy));
  }

  return(llength);
}

void SpaceObject::inflict_damage(SpaceObject *other) {
	int i;
	if(damage_factor > 0) {
		i = damage_factor / 2;
		if(i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
		play_sound((SAMPLE *)(melee[MELEE_BOOM + i].dat));
		other->damage += damage_factor;
		}
	other->handle_damage(this);
	return;
	}

SpaceLine::SpaceLine(SpaceLocation *creator, double lx, double ly, double langle, 
	double llength, int lcolor) 
	:
	SpaceLocation(creator, lx, ly, langle),
	length(llength),
	color(lcolor)
	{
	id = SPACE_LINE;
	layer = LAYER_LINES;
	collide_flag_anyone   &= OBJECT_LAYERS;
	collide_flag_sameteam &= OBJECT_LAYERS;
	collide_flag_sameship &= OBJECT_LAYERS;
	}

 double SpaceLine::edge_x()
{
  return(cos(angle * ANGLE_RATIO) * length);
}

 double SpaceLine::edge_y()
{
  return(sin(angle * ANGLE_RATIO) * length);
}

 double SpaceLine::get_length()
{
  return(length);
}

void SpaceLine::inflict_damage(SpaceObject *other) {
	int i;
	i = damage_factor / 2;
	if(i >= BOOM_SAMPLES)
		i = BOOM_SAMPLES - 1;
	play_sound((SAMPLE *)(melee[MELEE_BOOM + i].dat));
	other->damage += damage_factor;
	other->handle_damage(this);
	collide_flag_anyone = collide_flag_sameship = collide_flag_sameteam = 0;
	physics->addItem(new Animation( this, 
			normal_x() + edge_x(), normal_y() + edge_y(), game->sparkSprite, 0, 
			SPARK_FRAMES, 50, LAYER_EXPLOSIONS));
	return;
	}

void SpaceLine::animate(Frame *space)
{
  double x1, y1, x2, y2;

  x1 = normal_x() - space_x + 800;
	while (x1 < 0) x1 += X_MAX;
	x1 -= 800;
  x2 = x1 + edge_x();
  x1 *= space_zoom;
  x2 *= space_zoom;

  y1 = normal_y() - space_y + 800;
	while (y1 < 0) y1 += Y_MAX;
	y1 -= 800;
  y2 = y1 + edge_y();
  y1 *= space_zoom;
  y2 *= space_zoom;

  line(space->frame, (int)(x1), (int)(y1), (int)(x2), (int)(y2), color);
  space->add_line((int)(x1), (int)(y1), (int)(x2), (int)(y2));
}

void SpaceLine::collide(SpaceObject *o)
{
  double old_length = length;
  
  if((!canCollide(o)) || (!o->canCollide(this)))
    return;

  length = o->collide_line(normal_x(), normal_y(), normal_x() + edge_x(),
    normal_y() + edge_y(), length);
  if(length != old_length)
    inflict_damage(o);
	return;
}

Physics::~Physics() {
	int i;
	for (i = 0; i < num_presences; i += 1) {
		delete presence[i];
		presence[i] = NULL;
		}
	if (presence) free(presence);
	for (i = 0; i < num_items; i += 1) {
		delete item[i];
		item[i] = NULL;
		}
	if (item) free(item);
	if (quadrant) delete quadrant;
	}

void Physics::preinit() {
	width = 3840.0;
	height = 3840.0;
	frame_time = 25;
	quadrant = NULL;
	num_items = max_items = 0;
	item = NULL;
	num_presences = max_presences = 0;
	presence = NULL;
	last_ship = 0;
	last_team = 0;
//	last_deleted = NULL;
	return;
	}

unsigned int Physics::get_code(unsigned int ship, TeamCode team) {
	return (ship << SpaceLocation::ship_shift) | (team << SpaceLocation::team_shift);
	}

unsigned int Physics::new_ship() {
	last_ship += 1;
	return last_ship;
	}
TeamCode Physics::new_team() {
	last_team += 1;
	return last_team;
	}
void Physics::switch_team(unsigned int ship, TeamCode team) {
	int i, j;
	j = (ship & SpaceLocation::ship_mask) | (team << SpaceLocation::team_shift);
	for (i = 0; i < num_items; i += 1) {
		if ((item[i]->ally_flag & SpaceLocation::ship_mask) == (ship << SpaceLocation::ship_shift)) item[i]->ally_flag = j;
		}
	return;
	}
void Physics::merge_teams(TeamCode team1, TeamCode team2) {
	int i;
	for (i = 0; i < num_items; i += 1) {
		if ((item[i]->ally_flag & SpaceLocation::team_mask) == (team2 << SpaceLocation::team_shift)) 
			item[i]->ally_flag = (item[i]->ally_flag & ~ SpaceLocation::team_mask) | (team1 << SpaceLocation::team_shift);
		}
	return;
	}

void Physics::init() {
	int i;
	width = 3840.0;
	height = 3840.0;
	frame_time = 25;
	quadrant = new SpaceLocation*[QUADS_TOTAL];
	for(i = 0; i < QUADS_TOTAL; i += 1) {
		quadrant[i] = NULL;
		}
	max_items += 1024; 
	item = (SpaceLocation**) realloc(item, sizeof(SpaceLocation*) * max_items);
	max_presences += 64; 
	presence = (Presence**) realloc(presence, sizeof(Presence*) * max_presences);
	return;
	}

SpaceLocation *Physics::addItem(SpaceLocation *o) {
	if (o->qnext != NULL) error("uh-oh, spagetti O");
	if (num_items == max_items) {
		max_items += 1024;
		item = (SpaceLocation**) realloc(item, sizeof(SpaceLocation*) * max_items);
		}
	item[num_items] = o;
	num_items += 1;
	if (o->depth == -1) o->set_depth(o->layer);

	SpaceLocation *current = o;
	int q = int(current->normal_x() * QUADI_X) + 
			int(current->normal_y() * QUADI_Y) * QUADS_X;
	current->qnext = quadrant[q];
	quadrant[q] = current;

	return(o);
	}

bool Physics::removeItem(SpaceLocation *o) {
	int i;
	for (i = 0; i < num_items; i += 1) {
		if (item[i] == o) {
			item[i] = item[num_items-1];
			num_items -= 1;
			o->qnext = NULL;
			return true;
			}
		}
	return false;
	}

Presence *Physics::addPresence(Presence *o) {
	if (num_presences == max_presences) {
		max_presences += 256; 
		presence = (Presence**) realloc(presence, sizeof(Presence*) * max_presences);
		}
	presence[num_presences] = o;
	num_presences += 1;
	return(o);
	}

bool Physics::removePresence(Presence *o) {
	int i;
	for (i = 0; i < num_presences; i += 1) {
		if (presence[i] == o) {
			presence[i] = presence[num_items-1];
			num_presences -= 1;
			return true;
			}
		}
	return false;
	}

void Physics::calculate(int time) {
  int i;

	//prepare global variables
	::frame_time = time;
	prepare();

	#ifdef LOTS_OF_CHECKSUMS
		game->compare_checksums();
	#endif

	//move objects
	for (i = 0; i < num_items; i += 1) {
		if (!item[i]->exists()) continue;
		item[i]->x = normalize(item[i]->x + item[i]->vx * frame_time, X_MAX);
		item[i]->y = normalize(item[i]->y + item[i]->vy * frame_time, Y_MAX);
		}

	#ifdef LOTS_OF_CHECKSUMS
		game->compare_checksums();
	#endif

	//call Presence calculate functions
	for (i = 0; i < num_presences; i += 1) {
		if (presence[i]->exists()) presence[i]->calculate();
		#ifdef LOTS_OF_CHECKSUMS
			game->compare_checksums();
		#endif
		}


	//call objects calculate functions
	for (i = 0; i < num_items; i += 1) {
		if (item[i]->exists()) item[i]->calculate();
		#ifdef LOTS_OF_CHECKSUMS
			game->compare_checksums();
		#endif
		}

	//prepare quadrants stuff
	for(i = 0; i < QUADS_TOTAL; i += 1) {
		quadrant[i] = NULL;
		}
	for(i = 0; i < num_items; i += 1) {
		if(item[i]->exists()) {
			int q = int(item[i]->normal_x() * QUADI_X) + 
					int(item[i]->normal_y() * QUADI_Y) * QUADS_X;
			item[i]->qnext = quadrant[q];
			quadrant[q] = item[i];
			}
		}

	//check for collisions
	collide();

	#ifdef LOTS_OF_CHECKSUMS
		game->compare_checksums();
	#endif

	//remove presences that have been dead long enough
	for(i = 0; i < num_presences; i ++) {
		if (presence[i]->state == -DEATH_FRAMES) {
			Presence *tmp = presence[i];
			memmove(&presence[i], &presence[i+1], sizeof(Presence*) * (num_presences-i-1));
/*			presence[i] = presence[num_presences-1];*/
			num_presences -= 1;
			delete tmp;
			}
		else {
			if (!presence[i]->exists()) {
				//if (presence[i]->state == 0) presence[i]->death();
				presence[i]->state -= 1;
				}
			}
		}

	//remove objects that have been dead long enough
	for(i = 0; i < num_items; i ++) {
		if (item[i]->state == -DEATH_FRAMES) {
			SpaceLocation *tmp = item[i];
			item[i] = item[num_items-1];
			num_items -= 1;
			delete tmp;
			}
		else {
			if (!item[i]->exists()) {
				if (item[i]->state == 0) item[i]->death();
				item[i]->state -= 1;
				}
			}
		}

	#ifdef LOTS_OF_CHECKSUMS
		game->compare_checksums();
	#endif

	return;
	}

static int compare_depth (const void *_a, const void *_b) {
	int a = (*(const SpaceLocation**)_a)->depth;
	int b = (*(const SpaceLocation**)_b)->depth;
	return a-b;
	}
static SpaceLocation *animate_buffer[ANIMATE_BUFFER_SIZE];

void Physics::animate (Frame *frame) {
	int i, j;

	//call Presence animate functions
	for (i = 0; i < num_presences; i += 1) {
		if (presence[i]->exists()) presence[i]->animate(frame);
		}

	//call item animate functions
	j = num_items;
	if (j > ANIMATE_BUFFER_SIZE) j = ANIMATE_BUFFER_SIZE;
	memcpy(animate_buffer, item, sizeof(SpaceLocation *) * j);
	qsort(animate_buffer, j, sizeof(SpaceLocation*), compare_depth);
	prepare();
	for (i = 0; i < num_items; i += 1) {
		if (animate_buffer[i]->exists()) animate_buffer[i]->animate(frame);
		}
	return;
	}

void Physics::animate_predict(Frame *frame, int time) {
	int i, j;

	//call Presence calculate functions
	for (i = 0; i < num_presences; i += 1) {
		if (presence[i]->exists()) presence[i]->animate_predict(frame, time);
		}

	//call item calculate functions
	j = num_items;
	if (j > ANIMATE_BUFFER_SIZE) j = ANIMATE_BUFFER_SIZE;
	memcpy(animate_buffer, item, sizeof(SpaceLocation *) * j);
	qsort(animate_buffer, j, sizeof(SpaceLocation*), compare_depth);
	prepare();
	for (i = 0; i < num_items; i += 1) {
		if (animate_buffer[i]->exists()) 
			animate_buffer[i]->animate_predict(frame, time);
		}
	return;
	}

void Physics::collide() {
	int i;
	Query q;
	for (i = 0; i < num_items; i += 1) {
		if (item[i]->exists() && (item[i]->collide_flag_sameship | item[i]->collide_flag_sameteam | item[i]->collide_flag_anyone)) {
			if (item[i]->isObject()) {
				for (q.begin(item[i], OBJECT_LAYERS, (192+
						((SpaceObject*)item[i])->width()+
						((SpaceObject*)item[i])->height())/2); q.current && item[i]->exists(); q.next()) {
					((SpaceObject*)item[i])->collide(q.currento);
					}
				q.end();
				}
			else if (item[i]->isLine()) {
				for (q.begin(item[i], OBJECT_LAYERS, 96 + 
						((SpaceLine*)item[i])->get_length()); q.current && item[i]->exists(); q.next()) {
					((SpaceLine*)item[i])->collide(q.currento);
					}
				q.end();
				}
			}
		}
	return;
	}
void Physics::prepare() {
	::X_MAX = width;
	::Y_MAX = height;
	return;
	}
int Physics::checksum() {
	int i;
	double g = 0;
	//prepare();
	for (i = 0; i < num_items; i += 1) {
		g += item[i]->normal_x () * sqrt(47+i);
		}
//	return (((unsigned int)(fmod(floor(g * 1024), 0xFFffFFffUL))) & 255);
	return (((unsigned int)(fmod(floor(g * 1), 0xFFffFFffUL)) + tw_random_state_checksum()) & 255);
	}
void Physics::play_sound (SAMPLE *sample, SpaceLocation *source, int vol, int freq) {
	sound.play(sample, vol, 128, freq);
	return;
	}
void Physics::play_sound2 (SAMPLE *sample, SpaceLocation *source, int vol, int freq) {
	sound.stop(sample);
	sound.play(sample, vol, 128, freq);
	return;
	}
