#ifndef _MFRAME_H
#include "mframe.h"
#endif

#ifndef _MGAME_H
#define _MGAME_H

class GameType {
	private:
	Game *(*_new_game)();
	public:
	double order;
	char *name;
	Game *new_game();
	GameType ( const char *name, Game *(*new_game)(), double order = 0);
	};
extern int num_games;
extern char **game_names;
extern GameType **games;
#define REGISTER_GAME(b, a) static Game *new_ ## b () {return new b();} static GameType ignoreme_ ## b( a, new_ ## b, 0);
GameType *gametype(const char *name);

class Game : public Physics {
	public:
	GameType *type;

	enum {
		channel_none = -1,    //not a valid channel
		channel_init = 5,     // game type, version, length, etc.. things that need to read by a reader independant of a particular game type
		channel_playback = 6, // used for demo playbacks only
		channel_server  =  8,  //data originating on the server
		channel_client  = 12,  //data originating on the client
		_channel_buffered = 1
		};

	Log *log; //logging system for networking, demo recording/playback, etc.
	void log_char (int channel, char &data);             //helper for using the logging system
	void log_short(int channel, short &data);            //helper for using the logging system
	void log_int  (int channel, int &data);              //helper for using the logging system
	void log_data (int channel, void *data, int length); //helper for using the logging system
	void log_file (const char *fname);
	void log_fleet(int channel);

	virtual void preinit();	//pre-initialization (enough so that the destructor won't crash)
	virtual void init(Log *log);    //initialization (loading data, creating a few game objects, whatever)
	virtual ~Game();        //deallocating memory etc.

	virtual void prepare();   //called before doing anything with game objects contained within you.  
	//This is particularly necessary if you are working with multiple Games simultaneously

	int frame_time;   //the milliseconds passing per frame
	int frame_number; //the total number of frames that have passed so far
	virtual void calculate(int time);
	virtual void play();
	virtual void animate(Frame *frame); // stupid C++
	virtual void animate();

	virtual void ship_died(Ship *who, SpaceLocation *source);

	virtual void compare_checksums() ;
	virtual void handle_desynch( int local_checksum, int server_checksum, int client_checksum ) ;

	virtual bool game_ready();

	virtual void set_resolution (int screen_x, int screen_y);
	virtual void redraw();

	bool friendly_fire;
	double shot_relativity;

	int hiccup_margin;

	int lag_frames;
	int prediction;
	virtual void increase_latency();
	virtual void decrease_latency();

	bool view_locked;

	/*int num_players;
	int num_local_players; //?
	int first_local_player;//?
	Control **control;//
	ShipPanel **panel;//
	char **name;//
	virtual void add_player();
	virtual void player_said(int who, const char *what);*/

	int num_focuses;
	int focus_index;
	Presence **focus;
	virtual void add_focus (Presence *focus, int channel = -1);

	int num_targets;
	SpaceObject **target;
	virtual void add_target (SpaceObject *target);


	virtual Control *create_control (int channel, const char *type, char *config = "Config0", char *file = "scp.ini") ;
	virtual Ship    *create_ship (const char *id, Control *c, double x, double y, double angle, int team=0, const char *ini = NULL);
	virtual Ship    *create_ship (int channel, const char *id, const char *control, double x, double y, double angle, int team=0, const char *ini = NULL);

	virtual void pause();
	virtual void unpause();
	virtual bool is_paused();
	virtual void idle(int time = 5);

	JGMOD *planet_victory;
	SpaceSprite *planetSprite;
	SpaceSprite *asteroidSprite;
	SpaceSprite *asteroidExplosionSprite;
	SpaceSprite *sparkSprite;
	SpaceSprite *hotspotSprite;
	SpaceSprite *kaboomSprite;
	SpaceSprite *panelSprite;

	protected:
	int time_paused;
	int paused_time;
	double turbo;
	double f4_turbo;
	double next_tic_time;
	double msecs_per_render;
	double next_render_time;
	Histograph *tic_history;
	Histograph *render_history;
	double msecs_per_fps;
	double next_fps_time;
	virtual void fps();
	virtual void handle_key(int k);
	virtual void save_screenshot();
	int show_fps;
	};


#endif