
#include <stdio.h>
#include <string.h>
#include <allegro.h>
#include <assert.h>

#ifdef ALLEGRO_MSVC
	#include <winalleg.h>
	#include <windows.h>
	#include <Mmsystem.h>
#endif


#include "../melee.h"
REGISTER_FILE
#include "../libs.h"
#include "../scp.h"

#include "mframe.h"
#include "mgame.h"

#define MSECS_PER_TIC 5

#define TW_SOUND_CHANNELS 12
#define TW_MUSIC_CHANNELS 20

volatile int debug_value = 0;

/*------------------------------
		File Registration System
------------------------------*/
registered_file_type *registered_files = NULL;
int num_registered_files = 0;
FileRegistration::FileRegistration (char *fname, char *fdate, char *ftime) {
	registered_files = (registered_file_type*) realloc(registered_files, sizeof(registered_file_type) * (num_registered_files+1));
	registered_files[num_registered_files].fname = fname;
	registered_files[num_registered_files].fdate = fdate;
	registered_files[num_registered_files].ftime = ftime;
	num_registered_files += 1;
	return;
	}



/*------------------------------
		Timer
------------------------------*/
#if defined ALLEGRO_MSVC && !defined NO_WINMM
	int get_time() {
		return timeGetTime();
		}
	void init_time() {}
#else
	static volatile int actual_time = 0;
	static bool timer_installed = false;
	int get_time() {
		return actual_time;
		}
	static void global_timer() {
		actual_time += MSECS_PER_TIC;
		}
	END_OF_FUNCTION(global_timer);
	void init_time() {
		if (timer_installed) return;
		timer_installed = true;
		LOCK_FUNCTION(global_timer);
		LOCK_VARIABLE(actual_time);
		LOCK_VARIABLE(debug_value);
		if (install_timer() < 0) throw "Allegro timer installation failed";
		install_int(&global_timer, MSECS_PER_TIC);
		}
#endif


/*------------------------------
		Sound playing
------------------------------*/
#ifdef NO_JGMOD
	static void set_mod_volume(int a) {}
	static void play_mod(Music *a, int b) {}
	static void stop_mod() {}
	static bool is_mod_playing() {return false;}
	static void remove_mod() {}
	static void install_mod(int a) {}
	static void pause_mod() {}
	static void resume_mod() {}
#else
  #include <jgmod.h>
#endif

SoundSystem tw_sound;
SoundSystem::SoundSystem() {
	sound_volume = 255;
	music_volume = 255;
	sound_on = true;
	music_on = true;
	sound_initialized = false;
	compatible_soundcard = true;
	return;
	}
int SoundSystem::is_music_playing() {
	return ::is_mod_playing();
	}
void SoundSystem::disable() {
	if (sound_initialized && compatible_soundcard) {
		::remove_mod();
		::remove_sound();
		}
	sound_initialized = TRUE;
	compatible_soundcard = FALSE;
	return;
	}
void SoundSystem::load() {
	sound_on = get_config_int("Sound", "SoundOn", sound_on);
	music_on = get_config_int("Sound", "MusicOn", music_on);
	sound_volume = get_config_int("Sound", "SoundVolume", sound_volume);
	music_volume = get_config_int("Sound", "MusicVolume", music_volume);
	set_volumes ();
	}
void SoundSystem::save() {
	set_config_int("Sound", "SoundOn", sound_on);
	set_config_int("Sound", "MusicOn", music_on);
	set_config_int("Sound", "SoundVolume", sound_volume);
	set_config_int("Sound", "MusicVolume", music_volume);
	}
void SoundSystem::init() {
	if (sound_initialized) return;
	sound_initialized = true;
	if (!compatible_soundcard) return;
	compatible_soundcard = false;
	if (detect_digi_driver(DIGI_AUTODETECT)>=TW_SOUND_CHANNELS + TW_MUSIC_CHANNELS) {
		compatible_soundcard=true;
		::reserve_voices(TW_SOUND_CHANNELS + TW_MUSIC_CHANNELS, 0);
		if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, "")<0) compatible_soundcard=FALSE;
		install_mod(TW_MUSIC_CHANNELS);
		}
	return;
	}
int SoundSystem::play (SAMPLE *spl, int vol, int pan, int freq) {
	if (compatible_soundcard && sound_on) 
		return ::play_sample (spl, (vol * sound_volume) >> 8, pan, freq, false);
	else return 0;
	}
void SoundSystem::stop (SAMPLE *spl) {
	if (compatible_soundcard && sound_on) {
		::stop_sample (spl);
		return;
		}
	else return;
	}
void SoundSystem::play_music (Music *music, int loop) {
	if (compatible_soundcard && music_on) 
		::play_mod(music, loop);
	return;
	}
void SoundSystem::stop_music () {
	if (compatible_soundcard && music_on) 
		::stop_mod();
	return;
	}
void SoundSystem::set_volumes() {
	if (!sound_initialized) {
		init();
		}
	if (sound_volume < 0) throw "sound volume too low";
	if (sound_volume > 255) throw "sound volume too high";
	if (music_volume < 0) throw "music volume too low";
	if (music_volume > 255) throw "music volume too high";
	if (compatible_soundcard) {
		if (music_on) set_mod_volume (music_volume);
		else set_mod_volume (0);
		}
	return;
	}


/*------------------------------
		SC2 Unit Conversion
------------------------------*/
int time_ratio; //1000 milliseconds / SC2 framerate
double distance_ratio;
void init_sc2_unit_conversion() {
	}
int scale_frames(double value) {
	return (int)((value + 1) * time_ratio);
	}
double scale_turning (double turn_rate) {
//  turn_rate = 20.0 / ((turn_rate + 1.0) * 5.0);	
	return (360. / 16) / (turn_rate + 1.0) / time_ratio;
	}
double scale_velocity (double velocity) {
//  velocity = velocity / 7.5;
//velocity = x sc2pixels / sc2frame
//velocity = y twpixels / twframe
//velocity = x (1. / 1600) / (50ms)
//velocity = y (1. / 3840) / (1ms)
// y = x * (3840 / 1600) / 50
// WTF????
	return velocity * distance_ratio / time_ratio;
	}
double scale_acceleration (double acceleration, double raw_hotspot_rate) {
//  accel_rate = accel_rate / 100.0;
	return acceleration * distance_ratio / (1 + raw_hotspot_rate) / time_ratio / time_ratio; 
	}
double scale_range (double range) {
	return range * 40;
	}


/*------------------------------
		HELP DIALOG
------------------------------*/
static DIALOG help_dialog[] =
{
  // (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)     (d1)  (d2)  (dp)
  { d_box_proc,        40,  30,    480,  400,  255,  0,    0,    0,          0,    0,    NULL, NULL, NULL },
  { d_button_proc,     50,  40,    470,  40,   255,  0,    0,    D_EXIT,     0,    0,    (void *)"Exit this screen" , NULL, NULL },
  { d_textbox_proc,    50,  90,    470,  355,  255,  0,    0,    0,          0,    0,    NULL, NULL, NULL },
  { NULL,              0,    0,    0,    0,    255,  0,    0,    0,          0,    0,    NULL, NULL, NULL }
};
void show_file(const char *file) {
	int i;
	char buffy[100000];//fix sometime
	PACKFILE *f;
	f = pack_fopen ("./ingame.txt", F_READ);
	if (!f) sprintf(buffy, "Failed to load files \"%s\"", file);
	else {
		i = pack_fread (buffy, 99999, f);
		pack_fclose(f);
		buffy[i] = 0;
		}
	show_text(buffy);
	return;
	}
void show_text(const char *text) {
	set_dialog_color(help_dialog, pallete_color[0], pallete_color[15]);
	help_dialog[0].w = screen_x - 80;
	help_dialog[0].h = screen_y - 60;
	help_dialog[1].w = screen_x - 100;
	help_dialog[2].w = screen_x - 100;
	help_dialog[2].h = screen_y - 130;

	help_dialog[2].dp = (void *) text;
	help_dialog[2].d1 = 0;
	help_dialog[2].d2 = 0;
	popup_dialog(help_dialog, 1);
	return;
	}

/*------------------------------
		RANDUM NUMBER GENERATOR
------------------------------*/
#define tw_random_length0 5
#define tw_random_length1 8
static int tw_random_state0[tw_random_length0];
static int tw_random_state1[tw_random_length1];
static int tw_random_index0, tw_random_index1;
static int tw_random_carry;
int tw_random() {
	int i;
	i = tw_random_state0[tw_random_index0] + tw_random_state1[tw_random_index1] + tw_random_carry;
	tw_random_carry = (i >> 31) & 1;
	i &= 0x7fffffff;
	tw_random_state0[tw_random_index0] ^= i;
	tw_random_state1[tw_random_index1] ^= i;
	tw_random_index0 -= 1;
	if (tw_random_index0 < 0) tw_random_index0 += tw_random_length0;
	tw_random_index1 -= 1;
	if (tw_random_index1 < 0) tw_random_index1 += tw_random_length1;
	return i;
	}
int tw_random_state_checksum() {
	int i;
	i = tw_random_state0[tw_random_index0] + tw_random_state1[tw_random_index1] + tw_random_carry;
	return i & 0x7fffffff;
	}
void tw_random_seed (int s) {
	tw_random_carry = (s >> 30) & 1;
	int i;
	for (i = 0; i < tw_random_length0; i += 1) {
		tw_random_state0[i] = s;
		s = s*1664525+2401;
		}
	tw_random_index0 = 0;
	s ^= 0x47E51038;
	for (i = 0; i < tw_random_length1; i += 1) {
		tw_random_state1[i] = s;
		s = s*1664525+2401;
		}
	tw_random_index1 = 0;
	for (i = 0; i < 16; i += 1) tw_random();
	return;
	}
void tw_random_seed_more (int s) {
	s ^= ((s >> 30) << 16);
	s = (s << 2) + 2;
	int i;
	for (i = 0; i < 6; i += 1) s = s*(s+1);
	for (i = 0; i < tw_random_length0; i += 1) {
		tw_random_state0[i] ^= s;
		s = s*(s+1);
		}
	tw_random_index0 = 0;
	s ^= 0x00E51030;
	for (i = 0; i < tw_random_length1; i += 1) {
		tw_random_state1[i] += s;
		s = s*1664525+2401;
		}
	tw_random_index1 = 0;
	for (i = 0; i < 16; i += 1) tw_random();
	return;
	}
