#include <allegro.h>
#include <string.h>
#include <stdio.h>

#include "../melee.h"
REGISTER_FILE
#include "../id.h"
#include "../frame.h"

#include "mcbodies.h"
#include "mgame.h"
#include "mcontrol.h"
#include "mship.h"
#include "mshot.h"
#include "manim.h"
#include "mview.h"

/*------------------------------
		Ship Type Registration
------------------------------*/
shiptype_type *shiptypes = NULL;
int num_shiptypes = 0;
static void _register_ship(const char *id, const char *source_name, 
		Ship *(*getShip)(double x, double y, double angle, ShipData *data, 
		unsigned int code)) {
	num_shiptypes += 1;
	shiptypes = (shiptype_type*)realloc(shiptypes, num_shiptypes * sizeof(shiptype_type));
	int i = num_shiptypes-1;
	shiptypes[i].id = id;
	shiptypes[i]._getShip = getShip;
	shiptypes[i].dat = NULL;
	shiptypes[i].cost = 0;
	shiptypes[i].name = NULL;
	return;
	}
ShipRegistration::ShipRegistration(const char *id, const char *source_name, 
		Ship *(*getShip)(double x, double y, double angle, ShipData *data, 
		unsigned int code)) {
	_register_ship(id, source_name, getShip);
	return;
	}
AutoShipRegistration::AutoShipRegistration(const char *source_name, 
		Ship *(*getShip)(double x, double y, double angle, ShipData *data, 
		unsigned int code)) {
	char *tmp = (char*) malloc(6);
	strncpy(tmp, source_name + strlen(source_name) - 5 - 4, 5);
	tmp[5] = 0;
	_register_ship(tmp, source_name, getShip);
	return;
	}
Ship *shiptype_type::getShip (double x, double y, double angle, unsigned int code) {
	load_ship_data(this);
	return _getShip(x, y, angle, dat, code);
	}
shiptype_type *shiptype(const char *shiptype_id) {
	int i;
	for (i = 0; i < num_shiptypes; i += 1) {
		if (!strcmp(shiptype_id, shiptypes[i].id)) {
			return &shiptypes[i];
			}
		}
	return NULL;
	}
void init_ships() {
	int i = num_shiptypes;
	while (i > 0) {
		i -= 1;
		char fname[80];
		sprintf(fname, "ships/%s.ini", shiptypes[i].id);
		if (!exists(fname)) {
			memmove(&shiptypes[i], &shiptypes[num_shiptypes-1], sizeof(shiptype_type));
			num_shiptypes -= 1;
			continue;
			}
		set_config_file(fname);
		sprintf(fname, "ships/shp%s.dat", shiptypes[i].id);
		const char *tmp = get_config_string("Info", "Data", NULL);
		if (!tmp) tmp = fname;
		if (!exists(tmp))  {
			memmove(&shiptypes[i], &shiptypes[num_shiptypes-1], sizeof(shiptype_type));
			num_shiptypes -= 1;
			continue;
			}
		shiptypes[i].cost = get_config_int("Info", "TWCost", 999);
		shiptypes[i].name = get_config_string("Info", "Name", NULL);
		if (!shiptypes[i].name) {
			char sname[1024];
			const char *tmp;
			int l = 0;
			tmp = get_config_string("Info", "Name1", NULL);
			if (!tmp) error ("init_ships - error initializing name (%s)", shiptypes[i].id);
			l += sprintf(sname + l, "%s", tmp);
			int n = 1;
			while (true) {
				char buf[15];
				n += 1;
				sprintf(buf, "Name%d", n);
				tmp = get_config_string("Info", buf, NULL);
				if (!tmp) break;
				l += sprintf(sname + l, " %s", tmp);
				}
			shiptypes[i].name = strdup(sname);
			}
		else shiptypes[i].name = strdup(shiptypes[i].name);

		}
//	if (num_shiptypes == 0) error ("No ship data available.\nProbably caused to an install problem.\nConsult readme.txt");
	return;
	}
void load_ship_data(shiptype_type *type) {
	if(type->dat != NULL) return;
	char fn[256];
	sprintf(fn, "ships/shp%s.dat", type->id);
	type->dat = new ShipData(fn);
	type->dat->type = type;
	return;
	}
void unload_ship_data(shiptype_type *type) {
	if(type->dat == NULL) return;
	delete type->dat;
	type->dat = NULL;
	return;
	}
void load_all_ship_data() {
	int i;
	for (i = 0; i < num_shiptypes; i += 1) {
		load_ship_data(&shiptypes[i]);
		}
	return;
	}
void unload_all_ship_data() {
	int i;
	for (i = 0; i < num_shiptypes; i += 1) {
		unload_ship_data(&shiptypes[i]);
		}
	return;
	}


int hot_color[HOT_COLORS] =
{
  122, 123, 124, 125, 126, 127,
  42,  43,  44,  45,  46,  47
};


Ship::Ship(SpaceLocation *creator, double ox, double oy, double oangle, SpaceSprite *osprite) 
	: 
	SpaceObject(creator, ox, oy, oangle, osprite),
	update_panel(false),
	target_pressed(false),
	control(NULL)
	{
	layer = LAYER_SHIPS;
	collide_flag_sameteam = ALL_LAYERS;
	collide_flag_sameship = ALL_LAYERS;
	nextkeys = 0;
	id = SPACE_SHIP;

	thrust       = FALSE;
	turn_left    = FALSE;
	turn_right   = FALSE;
	fire_weapon  = FALSE;
	fire_special = FALSE;

	spritePanel = NULL;

	angle = oangle;
	sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;
	}

Ship::Ship(double ox, double oy, double shipAngle, ShipData *shipData, 
	unsigned int code) 
	:
	SpaceObject(NULL, ox, oy, shipAngle, shipData->spriteShip),
	update_panel(false),
	target_pressed(false),
	control(NULL)
	{
	ship = this;
	data = shipData;
	ally_flag = code;

	layer = LAYER_SHIPS;
	collide_flag_sameteam = ALL_LAYERS;
	collide_flag_sameship = ALL_LAYERS;
	nextkeys = 0;

	id = SPACE_SHIP;

	//  set_config_data((char *)(data->data[0].dat), data->data[0].size);
	crew     = get_config_int("Ship", "Crew", 0);
	crew_max = get_config_int("Ship", "CrewMax", 0);
	batt     = get_config_int("Ship", "Batt", 0);
	batt_max = get_config_int("Ship", "BattMax", 0);

	recharge_amount  = get_config_int("Ship", "RechargeAmount", 0);
	recharge_rate    = scale_frames(get_config_float("Ship", "RechargeRate", 0));
	recharge_step    = recharge_rate;
	weapon_drain     = get_config_int("Ship", "WeaponDrain", 0);
	weapon_rate      = scale_frames(get_config_float("Ship", "WeaponRate", 0));
	weapon_sample    = 0;
	weapon_recharge  = 0;
	weapon_low       = FALSE;
	special_drain    = get_config_int("Ship", "SpecialDrain", 0);
	special_rate     = scale_frames(get_config_float("Ship", "SpecialRate", 0));
	special_sample   = 0;
	special_recharge = 0;
	special_low      = FALSE;

	double raw_hotspot_rate = get_config_float("Ship", "HotspotRate", 0);
	hotspot_rate  = scale_frames(raw_hotspot_rate);
	hotspot_frame = 0;
	turn_rate     = scale_turning(get_config_float("Ship", "TurnRate", 0));
	turn_step     = 0.0;
	speed_max     = scale_velocity(get_config_float("Ship", "SpeedMax", 0));
	accel_rate    = scale_acceleration(get_config_float("Ship", "AccelRate", 0), raw_hotspot_rate);
	mass          = (get_config_float("Ship", "Mass", 0));

	thrust       = FALSE;
	turn_left    = FALSE;
	turn_right   = FALSE;
	fire_weapon  = FALSE;
	fire_special = FALSE;

	spritePanel  = new SpaceSprite(*(data->spritePanel));

	angle = shipAngle;
	sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

	}

void Ship::death() {
	return;
	}

Ship::~Ship() {
	delete spritePanel;
	}

int Ship::getCrew()
{
  return(crew);
}

 int Ship::getBatt()
{
  return(batt);
}

 int Ship::crewPanelColor()
{
  return pallete_color[2];
}

 int Ship::battPanelColor()
{
  return pallete_color[4];
}

void Ship::locate() {
	while (true) {
		x = (double)(random() % (int)(X_MAX));
		y = (double)(random() % (int)(Y_MAX));
		x = normal_x();
		y = normal_y();
		SpaceLocation *spacePlanet = nearest_planet();
		if (!spacePlanet || (distance(spacePlanet) > 1000)) break;
		}
	return;
	}

void Ship::calculate()
{
	if (control) {
		this->thrust         = 1&&(nextkeys & keyflag::thrust);
		this->turn_left      = 1&&(nextkeys & keyflag::left);
		this->turn_right     = 1&&(nextkeys & keyflag::right);
		this->fire_weapon    = 1&&(nextkeys & keyflag::fire);
		this->fire_special   = 1&&(nextkeys & keyflag::special);
		this->target_next    = 1&&(nextkeys & keyflag::next);
		this->target_prev    = 1&&(nextkeys & keyflag::prev);
		this->target_closest = 1&&(nextkeys & keyflag::closest);
		this->nextkeys = control->keys;
		if (!control->exists()) control = NULL;
		}

  if(batt < batt_max) {
    recharge_step -= frame_time;
    if(recharge_step < 0) {
      batt += recharge_amount;
			if (batt > batt_max) batt = batt_max;
      recharge_step += recharge_rate;
    }
  }

  if(weapon_recharge > 0)
    weapon_recharge -= frame_time;
  if(special_recharge > 0)
    special_recharge -=  frame_time;

	int i;
	if (!target_pressed && control) {
		if (target_next) {
			if (control && game->num_targets) {
				i = control->index;
				while (1) {
					i = (i + 1) % game->num_targets;
					if (control->valid_target(game->target[i])) {
						control->set_target(i);
						break;
						}
					if (i == control->index) break;
					}
				}				
			}
		else if (target_prev) {
			if (control && game->num_targets) {
				i = control->index;
				while (1) {
					i = (i + game->num_targets - 1) % game->num_targets;
					if (control->valid_target(game->target[i])) {
						control->set_target(i);
						break;
						}
					if (i == control->index) break;
					}
				}				
			}
		else if (target_closest) {
			if (control && game->num_targets) {
				int i, j = -1;
				double r = 99999;
				double d;
				for (i = 0; i < game->num_targets; i += 1) {
					if (control->valid_target(game->target[i])) {
						d = distance(game->target[i]);
						if (d < r) {
							r = d;
							j = i;
							}
						}
					}
				control->set_target(j);
				}
			}
		}
	target_pressed = target_next | target_prev | target_closest;
	if (control) target = control->target;

  calculate_turn_left();
  calculate_turn_right();
  calculate_thrust();
  calculate_fire_weapon();
  calculate_fire_special();


  while(fabs(turn_step) > 5.625 / 2) {
    if(turn_step < 0.0) {
      angle -= 5.625;
      turn_step += 5.625;
    } else
    if(turn_step > 0.0) {
      angle += 5.625;
      turn_step -= 5.625;
    }

    if(angle < 0.0)
      angle += 360.0;
    if(angle >= 360.0)
      angle -= 360.0;
  }

  sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

  calculate_hotspots();

  SpaceObject::calculate();
}

void Ship::animate(Frame *space) {
  SpaceObject::animate(space);
	}

void Ship::handle_damage(SpaceLocation *source) {
	batt -= fuel_sap;
	fuel_sap = 0;
	if(batt < 0) batt = 0;
	if(batt > batt_max) batt = batt_max;
	if(speed_loss > 0.0) {
/*		double oh = hotspot_rate;
		hotspot_rate += speed_loss;
		accel_rate *= oh / hotspot_rate;
		speed_max *= oh / hotspot_rate;
		turn_rate = 1. / (1. / turn_rate + 1. / scale_turning(1));
		speed_loss = 0.0;
//should be revised*/
		double sl = (30/(mass+30)) * speed_loss;
		if (sl > 1) error ("Speed loss too large\n(%f)", sl);
		accel_rate *= 1 - sl * accel_rate / (accel_rate + scale_acceleration(2,4));
		hotspot_rate = (int)(hotspot_rate / (1 - sl * accel_rate / (accel_rate + scale_acceleration(2,4)) ) );
		speed_max *= 1 - sl * speed_max / (speed_max + scale_velocity(10));
		turn_rate *=  1 - sl * turn_rate / (turn_rate + scale_turning(4));
		speed_loss = 0;
		}
	crew -= damage;
	damage = 0;
	crew -= directDamage;
	directDamage = 0;
	crew += repair;
	repair = 0;
	if(crew > crew_max) crew = crew_max;
	if(crew <= 0) {
		crew  = 0;
		state = 0;
		play_sound((SAMPLE *)(melee[MELEE_BOOMSHIP].dat));
		game->addItem(new Animation(this, x, y,
				game->kaboomSprite, 0, KABOOM_FRAMES, time_ratio, LAYER_EXPLOSIONS));
		game->ship_died(this, source);
		}
	return;
	}

void Ship::materialize() {
	}

void Ship::calculate_thrust() {
	if (thrust) accelerate_gravwhip(this, angle, accel_rate * frame_time, speed_max);
	return;
	}

void Ship::calculate_turn_left()
{
  if(turn_left) turn_step -= turn_rate * frame_time;
}

void Ship::calculate_turn_right()
{
  if(turn_right) turn_step += turn_rate * frame_time;
}

void Ship::calculate_fire_weapon() {
	weapon_low = FALSE;

	if (fire_weapon) {
		if (batt < weapon_drain) {
			weapon_low = true;
			return;
			}

		if (weapon_recharge > 0)
				return;

		if (!activate_weapon())
				return;

		batt -= weapon_drain;
		if (recharge_amount > 1) recharge_step = recharge_rate;
		weapon_recharge += weapon_rate;

		play_sound2(data->sampleWeapon[weapon_sample]);
		}
	return;
	}

void Ship::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if(batt < special_drain) {
      special_low = TRUE;
      return;
    }

    if(special_recharge > 0)
      return;

    if(!activate_special())
      return;

    batt -= special_drain;
    special_recharge += special_rate;

    play_sound2(data->sampleSpecial[special_sample]);
  }
}

void Ship::calculate_hotspots() {
	if((thrust) && (hotspot_frame <= 0)) {
		game->addItem(new Animation( this, 
				normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
				normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5),
				game->hotspotSprite, 0, HOTSPOT_FRAMES, time_ratio, LAYER_HOTSPOTS));
		hotspot_frame += hotspot_rate;
		}
	if (hotspot_frame > 0) hotspot_frame -= frame_time;
	return;
	}

int Ship::activate_weapon()
{
  return(TRUE);
}

inline int Ship::activate_special()
{
  return(TRUE);

}

Phaser::Phaser(SpaceLocation *creator, double ox, double oy, double _rx, double _ry, 
							 Ship *ship, SpaceSprite *sprite, int osprite_index, int *ocolors, 
							 int onum_colors, int ofsize, int steps, int step_size) :
		SpaceObject(creator, ox, oy, 0.0, sprite),
		rx(_rx), ry(_ry), 
		ship(ship),
		sprite_index(osprite_index),
		colors(ocolors),
		num_colors(onum_colors),
		color_index(0),
		frame_size(ofsize),
		frame_step(ofsize),
		phaser_step_position(0),
		phaser_steps(steps),
		phaser_step_size(step_size)
	{
	layer = LAYER_HOTSPOTS;
	collide_flag_anyone = 0;
	mass = 0;
	return;
	}

void Phaser::animate(Frame *space) {
	sprite->animate_character(x, y, 
		sprite_index, pallete_color[colors[color_index]], space);
	return;
	}

void Phaser::calculate() {
	if (!exists()) return;
	frame_step -= frame_time;
	while (frame_step < 0) {
		frame_step += frame_size;
		color_index++;
		if (color_index == num_colors) state = 0;
		}
	if (phaser_step_position < phaser_step_size) {
		if (ship && !ship->exists()) ship = NULL;
		phaser_step_position += frame_time;
		if (phaser_step_position >= phaser_step_size) {
			if (phaser_steps > 1) {
				double dx = rx / phaser_steps;
				double dy = ry / phaser_steps;
				game->addItem(new Phaser(this, x+dx, y+dy, rx-dx, ry-dy, ship, sprite, sprite_index, colors, num_colors, frame_size, phaser_steps-1, phaser_step_size));
				}
			else if (ship) {
				game->addItem(ship);
				ship->materialize();
				ship = NULL;
				}
			}
		}
	SpaceObject::calculate();
	}

SpaceLocation *Ship::get_ship_phaser() {
	return new Phaser( this, 
			x - cos(angle * PI/180) * PHASE_MAX * w,
			y - sin(angle * PI/180) * PHASE_MAX * h,
			cos(angle * PI/180) * PHASE_MAX * w,
			sin(angle * PI/180) * PHASE_MAX * h,
			this, sprite, sprite_index, hot_color, HOT_COLORS,
			PHASE_DELAY, PHASE_MAX, PHASE_DELAY);
	}
