#ifndef _MFRAME_H
#include "mframe.h"
#endif

#ifndef _MSHIP_H
#define _MSHIP_H

extern double gravity_sling;

typedef short KeyCode;

#define KEYFLAG_LEFT       1
#define KEYFLAG_RIGHT      2
#define KEYFLAG_THRUST     4
#define KEYFLAG_FIRE       8
#define KEYFLAG_SPECIAL   16
#define KEYFLAG_NEXT      32
#define KEYFLAG_PREV      64
#define KEYFLAG_CLOSEST  128
struct keyflag {
	enum { 
		left = 1, 
		right = 2, 
		thrust = 4, 
		fire = 8, 
		special = 16, 
		next = 32, 
		prev = 64, 
		closest = 128
		};
	};

class Phaser : public SpaceObject {
	protected:
	double rx, ry;
	Ship *ship;
	int sprite_index;
	int *colors;
	int num_colors;
	int color_index;
	int frame_size;
	int frame_step;
	int phaser_step_position;
	int phaser_steps;
	int phaser_step_size;

	public:
	Phaser::Phaser(SpaceLocation *creator, double x, double y, double rx, double ry, 
		Ship *ship, SpaceSprite *sprite, int osprite_index, int *ocolors, 
		int onum_colors, int ofsize, int steps, int step_time) ;

	virtual void animate(Frame *space);
	virtual void calculate();
	};

class Ship : public SpaceObject {
	protected:

	int hotspot_frame;
	int recharge_step;
	int weapon_recharge;
	int weapon_low;
	int special_recharge;
	int special_low;

	virtual void calculate_thrust();
	virtual void calculate_turn_left();
	virtual void calculate_turn_right();
	virtual void calculate_fire_weapon();
	virtual void calculate_fire_special();
	virtual void calculate_hotspots();

	virtual int activate_weapon();
	virtual int activate_special();

	public:
	virtual void materialize();
	int    crew;
	int    crew_max;
	int    batt;
	int    batt_max;
	double turn_rate;
	double turn_step;
	double speed_max;
	double accel_rate;
	int    recharge_amount;
	int    recharge_rate;
	int    weapon_drain;
	int    weapon_rate;
	int    weapon_sample;
	int    special_drain;
	int    special_rate;
	int    special_sample;
	int    hotspot_rate;

	SpaceSprite *spritePanel;

	int update_panel;

	KeyCode nextkeys;

	int thrust;
	int turn_left;
	int turn_right;
	int fire_weapon;
	int fire_special;
	int target_next;
	int target_prev;
	int target_closest;

	int target_pressed;
	Control *control;

	friend class ShipPanel;

	Ship(SpaceLocation *creator, double ox, double oy, double shipAngle, 
			SpaceSprite *osprite) ;
	Ship(double ox, double oy, double shipAngle, ShipData *shipData, 
			unsigned int code);
	virtual SpaceLocation *get_ship_phaser() ;
	virtual ~Ship();
	virtual void death();

	virtual int getCrew();
	virtual int getBatt();
	virtual int crewPanelColor();
	virtual int battPanelColor();

	void locate();
  
	void calculate();
	void animate(Frame *space);
	void handle_damage(SpaceLocation *source);
	};

#endif