#include <allegro.h>

#include "../melee.h"
REGISTER_FILE
#include "../id.h"
#include "mgame.h"
#include "mshot.h"
#include "mship.h"
#include "manim.h"

// Define shot spark

Shot::Shot(SpaceLocation *creator, double ox, double oy, double oangle, double ov, 
	int odamage, double orange, int oarmour, SpaceLocation *opos, 
	SpaceSprite *osprite, double relativity) 
	:
  SpaceObject(creator, opos->normal_x(), opos->normal_y(), oangle, osprite),
  v(ov),
  d(0.0),
  range(orange),
  armour(oarmour),
  explosionSprite(game->sparkSprite),
  explosionSample(NULL),
  explosionFrameCount(SPARK_FRAMES),
  explosionFrameSize(scale_frames(0))
{
  layer = LAYER_SHOTS;
	double alpha, tx, ty;
	alpha = (opos->get_angle() * ANGLE_RATIO);
	tx = cos(alpha);
	ty = sin(alpha);
// angle conventions fucked up??
	x += oy * tx - ox * ty;
	y += oy * ty + ox * tx;

  vx = (v * cos(angle * ANGLE_RATIO)) + opos->get_vx() * relativity;
  vy = (v * sin(angle * ANGLE_RATIO)) + opos->get_vy() * relativity;

  damage_factor = odamage;

  id |= SPACE_SHOT;

	if (range < 0) range = 999999999.0;
}

void Shot::calculate() {
	SpaceObject::calculate();
	d += v * frame_time;
	if(d >= range) state = 0;
	return;
	}

void Shot::animate_predict(Frame *space, int time) {
	if (range - d > v * time) {
		SpaceObject::animate_predict(space, time);
		}
	return;
	}

void Shot::handle_damage(SpaceLocation *source) {
	if (!exists()) return;
	if((damage > 0) || (directDamage > 0)) {
		armour -= damage;
		armour -= directDamage;
		damage = 0;
		directDamage = 0;
		if(armour <= 0) {
			armour = 0;
			state = 0;
			animateExplosion();
			soundExplosion();
			}
		}
	return;
	}

void Shot::inflict_damage(SpaceObject *other) {
	other->damage += damage_factor;
	other->handle_damage(this);
	if (!other->isShot()) state = 0;
	if (state == 0) {
		animateExplosion();
		soundExplosion(); 
		}
	return;
	}

void Shot::animateExplosion() {
	game->addItem(new Animation(this, normal_x(),
			normal_y(), explosionSprite, 0, explosionFrameCount,
			explosionFrameSize, LAYER_EXPLOSIONS));
	return;
	}

void Shot::soundExplosion() {
	if(explosionSample) {
		play_sound2(explosionSample);
		}
	else if(damage_factor > 0) {
		int i = damage_factor / 2;
		if(i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
		play_sound((SAMPLE *)(melee[MELEE_BOOM + i].dat));
		}
	return;
	}

void Shot::stop() {
	vx = 0.0;
	vy = 0.0;
	return;
	}

void Shot::destroy() {
	state = 0;
	return;
	}

void Shot::changeDirection (double oangle) {
	angle = normalize(oangle,360);
	//v = sqrt(vx*vx + vy*vy);
	vx = (v * cos(angle * ANGLE_RATIO));
	vy = (v * sin(angle * ANGLE_RATIO));
	return;
	}

int Shot::isHomingMissile() {
	return ((id & BASE_MASK3) == SPACE_HOMING_MISSILE);
	}

AnimatedShot::AnimatedShot(SpaceLocation *creator, double ox, double oy, 
	double oangle, double ov, int odamage, double orange, int oarmour, SpaceLocation *opos,
	SpaceSprite *osprite, int ofcount, int ofsize, double relativity) 
	:
	Shot(creator, ox, oy, oangle, ov, odamage, orange, oarmour, opos, osprite, relativity),
	frame_count(ofcount),
	frame_size(ofsize),
	frame_step(ofsize)
	{
	}

void AnimatedShot::calculate() {
	Shot::calculate();
	frame_step -= frame_time;
	while (frame_step < 0) {
		frame_step += frame_size;
		sprite_index++;
		if (sprite_index == frame_count) sprite_index = 0;
		}
	return;
	}

Missile::Missile(SpaceLocation *creator, double ox, double oy, double oangle, 
	double ov, int odamage, double orange, int oarmour, 
	SpaceLocation *opos, SpaceSprite *osprite, double relativity) 
	:
	Shot(creator, ox, oy, oangle, ov, odamage, orange, oarmour, opos, osprite, relativity)
	{
	sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;
	}

void Missile::changeDirection (double oangle) {
	Shot::changeDirection(oangle);
	sprite_index = (int)(oangle / 5.625) + 16;
	sprite_index &= 63;
	return;
	}

HomingMissile::HomingMissile(SpaceLocation *creator, double ox, double oy, 
	double oangle, double ov, int odamage, double orange, int oarmour, 
	double otrate, SpaceLocation *opos, SpaceSprite *osprite, SpaceObject *otarget) 
	:
	Missile(creator, ox, oy, oangle, ov, odamage, orange, oarmour, opos, osprite, 0),
	turn_rate(otrate),
	turn_step(0.0)
	{
	target = otarget;
	id = SPACE_HOMING_MISSILE;
	}

void HomingMissile::calculate() {
	Missile::calculate();
	if (target && !target->isInvisible()) {
		double d_a = normalize(trajectory_angle(target) - (angle + turn_step), 360);
		if (d_a > 180) d_a -= 360;
		double ta = turn_rate * frame_time;
		if (fabs(d_a) < ta) ta = fabs(d_a);
		if (d_a > 0) turn_step += ta;
		else turn_step -= ta;
		while(fabs(turn_step) > 5.625/2) {
			if(turn_step < 0.0) {
				angle -= 5.625;
				turn_step += 5.625;
				}
			else if (turn_step > 0.0) {
				angle += 5.625;
				turn_step -= 5.625;
				}
			}
		angle = normalize(angle, 360);
		}

	sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

	vx = v * cos(angle * ANGLE_RATIO);
	vy = v * sin(angle * ANGLE_RATIO);
	return;
	}

void HomingMissile::animate_predict(Frame *space, int time) {
	double old_angle = this->angle;
	int old_sprite_index = this->sprite_index;
	double old_vx = this->vx;
	double old_vy = this->vy;

	double da = 0;
	if (target && !target->isInvisible()) {
		double d_a = normalize(trajectory_angle(target) - (angle + turn_step), 360);
		if (d_a > 180) d_a -= 360;
		double ta = turn_rate * time;
		if (fabs(d_a) < ta) ta = fabs(d_a);
		if (d_a > 0) turn_step += ta;
		else turn_step -= ta;
		while(fabs(turn_step) > 5.625/2) {
			if(turn_step < 0.0) {
				da -= 5.625;
				turn_step += 5.625;
				}
			else if (turn_step > 0.0) {
				da += 5.625;
				turn_step -= 5.625;
				}
			}
		}
	angle = normalize(angle + da, 360);

	sprite_index = (int)(angle / 5.625) + 16;
	sprite_index &= 63;

	vx = v * cos((angle-da/2) * ANGLE_RATIO);
	vy = v * sin((angle-da/2) * ANGLE_RATIO);

	Shot::animate_predict(space, time);
	angle = old_angle;
	sprite_index = old_sprite_index;
	vx = old_vx;
	vy = old_vy;
	return;
	}

Laser::Laser(SpaceLocation *creator, double langle, int lcolor, double lrange, int ldamage,
  int lfcount, SpaceLocation *opos, double rel_x, double rel_y, bool osinc_angle) 
	:
  SpaceLine(creator, 0.0, 0.0, langle, lrange, lcolor),
  frame(0),
  frame_count(lfcount),
  pos(opos),
  pos_x(rel_x),
  pos_y(rel_y),
  sinc_angle(osinc_angle)
{
  double alpha;

  id            = SPACE_LASER;
  damage_factor = ldamage;
  relative_angle = angle - pos->get_angle();

	alpha = (pos->get_angle() * ANGLE_RATIO);
	double tx, ty;
	tx = cos(alpha);
	ty = sin(alpha);
// angle conventions fucked up??
	x = pos->normal_x() + pos_y * tx - pos_x * ty;
	y = pos->normal_y() + pos_y * ty + pos_x * tx;
	vx = pos->get_vx();
	vy = pos->get_vy();

	if(!pos->exists()) state = 0;
}


void Laser::calculate() {
	double alpha, tx, ty;
	if((frame < frame_count) && (pos->exists())) {
		alpha = (pos->get_angle() * ANGLE_RATIO);
		tx = cos(alpha);
		ty = sin(alpha);
// angle conventions fucked up??
		x = pos->normal_x() + pos_y * tx - pos_x * ty;
		y = pos->normal_y() + pos_y * ty + pos_x * tx;
		vx = pos->get_vx();
		vy = pos->get_vy();
		if (sinc_angle) angle = normalize(pos->get_angle() + relative_angle, 360);
		SpaceLine::calculate();
		frame += frame_time;
		}
	else state = 0;
	return;
	}

PointLaser::PointLaser(SpaceLocation *creator, int lcolor, int ldamage,
  int lfcount, SpaceLocation *lsource, SpaceObject *ltarget, double rel_x, double rel_y) :
  Laser(creator, lsource->trajectory_angle(ltarget), lcolor, lsource->distance(ltarget), ldamage, lfcount, lsource, rel_x, rel_y),
  target(ltarget),
  pos_x(rel_x),
  pos_y(rel_y)
{
	collide_flag_anyone = bit(target->layer);
	collide_flag_sameteam = bit(target->layer);
	collide_flag_sameship = bit(target->layer);
	double alpha;
	alpha = (pos->get_angle() * ANGLE_RATIO);
	double tx, ty;
	tx = cos(alpha);
	ty = sin(alpha);
// angle conventions fucked up??
	x = pos->normal_x() + pos_y * tx - pos_x * ty;
	y = pos->normal_y() + pos_y * ty + pos_x * tx;
	vx = pos->get_vx();
	vy = pos->get_vy();
	angle = trajectory_angle(target);

	if (!target->canCollide(this) || !canCollide(target)) state = 0;
	if(!pos->exists()) state = 0;
}

int PointLaser::canCollide(SpaceObject *other) {
	if (other != target) return false;
	return Laser::canCollide(other);
	}

void PointLaser::calculate() {
	double alpha;
	alpha = (pos->get_angle() * ANGLE_RATIO);
	Laser::calculate();
	if (target) {
		if (target->exists() && canCollide(target) && target->canCollide(this)) {
			inflict_damage(target);
//			length = distance(target);
			}
		angle = trajectory_angle(target);
		if (!target->exists()) target = NULL;
		}
	return;
	}
