#ifndef _MFRAME_H
#include "mframe.h"
#endif

#ifndef _MSHOT_H
#define _MSHOT_H

#define SHOT_RELATIVITY 0.5

class Shot : public SpaceObject {
	public:
	double v;
	double d;
	double range;
	int    armour;
  
	SpaceSprite *explosionSprite;
	SAMPLE      *explosionSample;
	int          explosionFrameCount;
	int          explosionFrameSize;

	Shot(SpaceLocation *creator, double ox, double oy, double oangle, double ov, int odamage,
			double orange, int oarmour, SpaceLocation *opos, SpaceSprite *osprite, double relativity = game->shot_relativity);

	virtual void animate_predict(Frame *space, int time);

	virtual void calculate();
	virtual void handle_damage(SpaceLocation *source);
	virtual void inflict_damage(SpaceObject *other);

	virtual void animateExplosion();
	virtual void soundExplosion();

	void stop();
	void destroy();

	virtual void changeDirection(double oangle);
	int isHomingMissile();
	};

class AnimatedShot : public Shot {
	protected:
	int frame_count;
	int frame_size;
	int frame_step;

	public:
	AnimatedShot(SpaceLocation *creator, double ox, double oy, double oangle, 
			double ov, int odamage, double orange, int oarmour, SpaceLocation *opos,
			SpaceSprite *osprite, int ofcount, int ofsize, double relativity = game->shot_relativity);

	virtual void calculate();
	};

class Missile : public Shot {
	public:
	Missile(SpaceLocation *creator, double ox, double oy, double oangle, double ov, int odamage,
			double orange, int oarmour, SpaceLocation *opos,
			SpaceSprite *osprite, double relativity = game->shot_relativity);

	virtual void changeDirection(double oangle);
//	virtual void animate_predict(Frame *space, int time);
	};

class HomingMissile : public Missile {
	protected:
	double turn_rate;
	double turn_step;

	public:
	HomingMissile(SpaceLocation *creator, double ox, double oy, double oangle, double ov, int odamage,
			double orange, int oarmour, double otrate, SpaceLocation *opos,
			SpaceSprite *osprite, SpaceObject *target);

	virtual void animate_predict(Frame *space, int time);
	virtual void calculate();
	};

class Laser : public SpaceLine {
  protected:
  double frame;
  double frame_count;

  SpaceLocation *pos;
  double pos_x;
  double pos_y;
	double relative_angle;
	bool sinc_angle;

  public:
  Laser(SpaceLocation *creator, double langle, int lcolor, double lrange, int ldamage, int lfcount,
    SpaceLocation *opos, double rel_x, double rel_y, bool osinc_angle=false);

  void calculate();
};

class PointLaser : public Laser {
  protected:

  SpaceObject *target;
  double pos_x;
  double pos_y;

  public:
	PointLaser::PointLaser(SpaceLocation *creator, int lcolor, int ldamage, int lfcount, 
		SpaceLocation *lsource, SpaceObject *ltarget, double rel_x, double rel_y) ;

  void calculate();
  int canCollide(SpaceObject *other);
};


#endif