
#include <allegro.h>

#include "../melee.h"
REGISTER_FILE
#include "mgame.h"
#include "mshppan.h"
#include "mship.h"

int PANEL_WIDTH = 64;
int PANEL_HEIGHT = 100;

int CAPTAIN_X = 4;
int CAPTAIN_Y = 65;
int PANEL_DEATH_FRAMES = 2500;
int crew_x = 8;
int crew_y = 53;
int batt_x = 56;
int batt_y = 53;


ShipPanel::ShipPanel(Ship *_ship) {
	panel   = create_bitmap(64, 100);
	captain = create_bitmap(CAPTAIN_WIDTH, CAPTAIN_HEIGHT);

	crew_old   = 0;
	batt_old   = 0;
	batt_light = FALSE;

	panel_needs_update = false;
	captain_needs_update = false;

	old_thrust       = FALSE;
	old_turn_left    = FALSE;
	old_turn_right   = FALSE;
	old_fire_weapon  = FALSE;
	old_fire_special = FALSE;

	deathframe = 0;
	ship = _ship;

	ship->spritePanel->draw(0, 0, 0, panel);
	ship->spritePanel->draw(0, 0, 1, captain);
	draw_stuff (crew_x, crew_y, 2, 1, -3, -2, 2, ship->crew, ship->crew_max, ship->crewPanelColor(), 0);
	draw_stuff (batt_x, batt_y, 2, 1, -3, -2, 2, ship->batt, ship->batt_max, ship->battPanelColor(), 0);

	hide ();
	}

void ShipPanel::locate(int x, int y) {
	int ox = panel_x, oy = panel_y;
	panel_x = x;
	panel_y = y;
	captain_x = panel_x + CAPTAIN_X;
	captain_y = panel_y + CAPTAIN_Y;
	if ((ox != x) || (oy != y)) {
		acquire_screen();
		blit(panel, screen, 0, 0, panel_x, panel_y, 64, 100);
		blit(captain, screen, 0, 0, captain_x, captain_y, captain->w, captain->h);
		release_screen();
		}
	return;
	}

void ShipPanel::hide() {
	panel_x = -999;
	panel_y = -999;
	captain_x = panel_x + 4;
	captain_y = panel_y + 65;
	return;
	}

ShipPanel::~ShipPanel()
{
  destroy_bitmap(captain);
  destroy_bitmap(panel);
}

int ShipPanel::exists() {
	if ((ship == NULL) && (deathframe <= 0)) return false;
	return true;
	}

void ShipPanel::refresh() {
	int x = panel_x, y = panel_y;
	hide();
	locate(x, y);
	return;
	}

void ShipPanel::calculate() {
	if (!ship) {
		deathframe -= frame_time;
		return;
		}
	if (!ship->exists()) {//ship is dieing
		ship->spritePanel->draw(0, 0, 0, panel);
		panel_needs_update = true;
		ship = NULL;	  
		deathframe = PANEL_DEATH_FRAMES;
		return;
		} 
	if(ship->update_panel) {
		ship->update_panel = false;
		panel_needs_update = true;
		captain_needs_update = true;
		}
  if (
			(ship->thrust != old_thrust) ||
			(ship->turn_left != old_turn_left) ||
			(ship->turn_right != old_turn_right) ||
			(ship->fire_weapon != old_fire_weapon) ||
			(ship->fire_special != old_fire_special))
		captain_needs_update = true;
	return;
	}

void ShipPanel::animate(Frame *space) {
	if (FULL_REDRAW) {
		panel_needs_update = true;
		captain_needs_update = true;
		}
	if (!ship) {
	  double w, h;
		if (panel_needs_update) {
			acquire_screen();
			blit(panel, screen, 0, 0, panel_x, panel_y, 64, 100);
			blit(captain, screen, 0, 0, captain_x, captain_y, CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
			release_screen();
			}
		if (deathframe < 0) return;
		rectfill(captain, 0, 0, CAPTAIN_WIDTH, CAPTAIN_HEIGHT, 0);
		w = CAPTAIN_WIDTH * deathframe / (double)PANEL_DEATH_FRAMES;
		h = CAPTAIN_HEIGHT * deathframe / (double)PANEL_DEATH_FRAMES;
		if(w < 1.0) w = 1.0;
		if(h < 1.0) h = 1.0;
		rectfill(captain,
				(int)(((double)(CAPTAIN_WIDTH) - w) / 2.0),
				(int)(((double)(CAPTAIN_HEIGHT) - h) / 2.0),
				(int)((double)(CAPTAIN_WIDTH) - (((double)(CAPTAIN_WIDTH) - w) / 2.0)),
				(int)((double)(CAPTAIN_HEIGHT) - (((double)(CAPTAIN_HEIGHT) - h) / 2.0)),
				pallete_color[128 - (7 * deathframe / PANEL_DEATH_FRAMES)]);
		acquire_screen();
		blit(captain, screen, 0, 0, captain_x, captain_y, CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
		release_screen();
		return;
		}

	if (panel_needs_update) {
		panel_needs_update = false;
		ship->spritePanel->draw(0, 0, 0, panel);
		crew_old = -1;
		batt_old = -1;
		acquire_screen();
		blit(panel, screen, 0, 0, panel_x, panel_y, 64, 100);
		blit(captain, screen, 0, 0, captain_x, captain_y, CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
		release_screen();
		}

	if (captain_needs_update) {
		captain_needs_update = false;
		old_thrust       = ship->thrust;
		old_turn_left    = ship->turn_left;
		old_turn_right   = ship->turn_right;
		old_fire_weapon  = ship->fire_weapon;
		old_fire_special = ship->fire_special;

		ship->spritePanel->draw(0, 0, 1, captain);
		if(ship->thrust)       ship->spritePanel->overlay(1, 2, captain);
		if(ship->turn_right)   ship->spritePanel->overlay(1, 3, captain);
		if(ship->turn_left)    ship->spritePanel->overlay(1, 4, captain);
		if(ship->fire_weapon)  ship->spritePanel->overlay(1, 5, captain);
		if(ship->fire_special) ship->spritePanel->overlay(1, 6, captain);
		acquire_screen();
		blit(captain, screen, 0, 0, captain_x, captain_y, CAPTAIN_WIDTH, CAPTAIN_HEIGHT);
		release_screen();
		}

	if (ship->crew != crew_old) {		
		draw_stuff (crew_x, crew_y, 2, 1, -3, -2, 2, ship->crew, ship->crew_max, ship->crewPanelColor(), 0);
		acquire_screen();
		blit(panel, screen, crew_x - 3, crew_y - ship->crew_max + 1,
				panel_x + crew_x - 3, panel_y + crew_y - ship->crew_max + 1, 5,
				ship->crew_max + 1);
		release_screen();
		crew_old = ship->crew;
		}

	if(ship->batt != batt_old) {
		draw_stuff (batt_x, batt_y, 2, 1, -3, -2, 2, ship->batt, ship->batt_max, ship->battPanelColor(), 0);
		acquire_screen();
		blit(panel, screen, batt_x - 3, batt_y - ship->batt_max + 1,
				panel_x + batt_x - 3, panel_y + batt_y - ship->batt_max + 1, 5,
				ship->batt_max + 1);
		release_screen();
		batt_old = ship->batt;
		}

	if((!batt_light) && (ship->weapon_low || ship->special_low)) {
		acquire_screen();
		game->panelSprite->draw(panel_x + 40, panel_y + 58, 2, screen);
		release_screen();
		batt_light = TRUE;
		}
	else if(batt_light && (!ship->weapon_low) && (!ship->special_low)) {
		acquire_screen();
		game->panelSprite->draw(panel_x + 40, panel_y + 58, 1, screen);
		release_screen();
		batt_light = FALSE;
		}

	return;
	}

void ShipPanel::draw_stuff (int x, int y, int w, int h, int dx, int dy, int m, int value, int max, int color, int bcolor) {
	int i;
	w -= 1;
	h -= 1;
	if (value > max) value = max;
	for (i = 0; i < value; i += 1) {
		int _x = x + dx * (i % m);
		int _y = y + dy * (i / m);
		rectfill(panel, _x, _y, _x+w, _y+h, color);
		}
	for (i = value; i < max; i += 1) {
		int _x = x + dx * (i % m);
		int _y = y + dy * (i / m);
		rectfill(panel, _x, _y, _x+w, _y+h, bcolor);
		}
	return;
	}