
#include <allegro.h>

#include <math.h>
#include <string.h>
#include <stdio.h>


#include "../melee.h"
REGISTER_FILE
#include "../libs.h"
#include "../util/ppcol.h"
#include "../util/aastr.h"
#include "../frame.h"
#include "../scp.h"

#include "mview.h"

void convert_bitmap(BITMAP *src, BITMAP *dest, bool masked) {
	//requires that both be memory bitmaps
	int obpp = bitmap_color_depth(src);
	int bpp = bitmap_color_depth(dest);
	if ((src->w != dest->w) || (src->h != dest->h)) error("convert_bitmap - wrong size");
	//if (obpp == bpp) error("convert_bitmap - color depths match");
	if ((obpp == bpp) || !masked) {
		blit(src, dest, 0, 0, 0, 0, src->w, src->h);
		return;
		}
	int x, y, om, nm, nnm;
	om = bitmap_mask_color(src);
	nm = bitmap_mask_color(dest);
	if (nm == 0) {
		int i = 0;
		nnm = 0;
		while (nnm == nm) {
			nnm = makecol_depth(bpp, i, 0, i);
			i += 1;
			}
		}
	else nnm = nm - 1;
	for (y = 0; y < src->h; y += 1) {
		for (x = 0; x < src->w; x += 1) {
			int oc = getpixel(src, x, y);
			if (oc == om) {
				putpixel(dest, x, y, nm);
				continue;
				}
			int r, g, b;
			r = getr_depth(obpp, oc);
			g = getg_depth(obpp, oc);
			b = getb_depth(obpp, oc);
			int nc = makecol_depth(bpp, r, g, b);
			if (nc == nm) nc = nnm;
			putpixel(dest, x, y, nc);
			}
		}
	return;
	}

void gamma_correct_bitmap(BITMAP *bmp, int gamma, int masked) {
	static int old_gamma, inited = 0;
	static unsigned int table[256];
	int x, y, w, h;
	if (!gamma) return;
	if (!inited) {
		inited = true;
		old_gamma = ~gamma;
		}
	if (gamma != old_gamma) {
		for (int i = 0; i < 256; i += 1) {
			table[i] = (int)floor(256. * pow(i / 255.5, 1-gamma/300.));
			}
		old_gamma = gamma;
		}
	w = bmp->w;
	h = bmp->h;
	int bmc = bitmap_mask_color(bmp);
	for (y = 0; y < h; y += 1) {
		for (x = 0; x < w; x += 1) {
			int r, g, b;
			b = getpixel(bmp, x, y);
			if ((b == bmc) && masked) continue;
			r = getr(b);
			g = getg(b);
			b = getb(b);
			r = table[r];
			g = table[g];
			b = table[b];
			int c = makecol(r, g, b);
			if ((c == bmc) && masked) {
				if (c == 0) c = 255;
				else c = makecol(255, 10, 255);
				}
			putpixel(bmp, x, y, c);
			}
		}
	return;
	}

void SpaceSprite::change_color_depth(int newbpp) {
	int i;
	for (i = 0; i < count; i += 1) {
		BITMAP *tmp = create_bitmap_ex(newbpp, w, h);
		convert_bitmap(b[i], tmp, general_attributes & MASKED);
		if (attributes[i] & DEALLOCATE_IMAGE) destroy_bitmap(b[i]);
		attributes[i] |= DEALLOCATE_IMAGE;
		b[i] = tmp;
		}
	return;
	}

SpaceSprite::SpaceSprite(const DATAFILE *images, int sprite_count, int _attributes, int rotations) {
	int i, j, obpp;
	BITMAP *bmp, *tmp = NULL;

	count = sprite_count * rotations;

	if ((rotations < 1) || (count < 1)) error("SpaceSprite::SpaceSprite - bad parameters");

	general_attributes = _attributes;
	bpp = color_depth;

	originaltype = images[0].type;
	switch (originaltype) {
		case DAT_RLE_SPRITE: {
			w = ((RLE_SPRITE *)images[0].dat)->w;
			h = ((RLE_SPRITE *)images[0].dat)->h;
			obpp = ((RLE_SPRITE *)images[0].dat)->color_depth;
			if (obpp != bpp) tmp = create_bitmap_ex(obpp, w, h);
			}
		break;
		case DAT_BITMAP: {
			w = ((BITMAP *)images[0].dat)->w;
			h = ((BITMAP *)images[0].dat)->h;
			obpp = bitmap_color_depth((BITMAP *)images[0].dat);
			}
		break;
		default: {
			error("SpaceSprite::SpaceSprite - bad data file");
			}
		break;
		}
	if (general_attributes & IRREGULAR) goto irregular;
	for(i = 1; i < sprite_count; i++) {
		if (images[i].type != originaltype) error ("SpaceSprite - bad data file");
		switch (originaltype) {
			case DAT_RLE_SPRITE: {
				if ((w != ((RLE_SPRITE *)images[i].dat)->w) || 
						(h != ((RLE_SPRITE *)images[i].dat)->h) ) error("SpaceSprite - size changed");
				if (((RLE_SPRITE *)images[i].dat)->color_depth != obpp) error("SpaceSprite - changing source color depth not yet allowed");
				}
			break;
			case DAT_BITMAP: {
				if ((w != ((BITMAP *)images[i].dat)->w) || 
						(h != ((BITMAP *)images[i].dat)->h) ) error("SpaceSprite - size changed");
				if (bitmap_color_depth((BITMAP *)images[i].dat) != obpp) error("SpaceSprite - changing source color depth not yet allowed");
				}
			break;
			}
		}

	m = new PPCOL_MASK*[count];
	b = new BITMAP*    [count];
	attributes  = new char [count];

	for(i = 0; i < sprite_count; i += 1) {
		bmp = create_bitmap_ex(bpp, w, h);
		if (!tmp) tmp = bmp;
		if (general_attributes & MASKED) clear_to_color(bmp, bitmap_mask_color(bmp));
		if (tmp != bmp) clear_to_color(tmp, bitmap_mask_color(tmp));
		switch (originaltype) {
			case DAT_RLE_SPRITE: {
				draw_rle_sprite(tmp, (RLE_SPRITE *)(images[i].dat), 0, 0);
				if (tmp != bmp) convert_bitmap(tmp, bmp, general_attributes & MASKED);
				}
			break;
			case DAT_BITMAP: {
				if (obpp != bpp) {
					convert_bitmap((BITMAP *)(images[i].dat), bmp, general_attributes & MASKED);
					}
				else {
					if (general_attributes & MASKED) draw_sprite(bmp, (BITMAP*)images[i].dat, 0, 0);
					else blit((BITMAP*)images[i].dat, bmp, 0, 0, 0, 0, w, h);
					}
				}
			break;
			}
		gamma_correct_bitmap(bmp, gamma_correction, general_attributes & MASKED);
		if (tmp == bmp) tmp = NULL;
		for (j = 1; j < rotations; j += 1) {
			BITMAP *tmp = create_bitmap_ex(bpp, w, h);
			clear_to_color(tmp, bitmap_mask_color(tmp));
			rotate_sprite(tmp, bmp, 0, 0, j * ((1<<24)/rotations));
			m[j + (i * rotations)] = create_ppcol_mask(tmp);
			b[j + (i * rotations)] = tmp;
			attributes[j + (i * rotations)] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
			}
		m[(i * rotations)] = create_ppcol_mask(bmp);
		b[(i * rotations)] = bmp;
		attributes[(i * rotations)] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
		}
	return;//end of normal/masked/autorotated

	irregular:

	if (rotations != 1) error (" irregular SpaceSprites are not permitted to be autorotated");

	m = new PPCOL_MASK*[count];
	b = new BITMAP*    [count];
	attributes  = new char [count];

	for(i = 0; i < sprite_count; i += 1) {
		switch (originaltype) {
			case DAT_RLE_SPRITE: {
				w = ((RLE_SPRITE *)images[i].dat)->w;
				h = ((RLE_SPRITE *)images[i].dat)->h;
				obpp = ((RLE_SPRITE *)images[i].dat)->color_depth;
				tmp = create_bitmap_ex ( obpp, w, h);
				draw_rle_sprite(tmp, (RLE_SPRITE *)(images[i].dat), 0, 0);
				if (bpp == obpp) bmp = tmp;
				else {
					bmp = create_bitmap_ex ( bpp, w, h);
					convert_bitmap(tmp, bmp, general_attributes & MASKED);
					destroy_bitmap(tmp);
					}
				}
			break;
			case DAT_BITMAP: {
				w = ((BITMAP *)images[i].dat)->w;
				h = ((BITMAP *)images[i].dat)->h;
				bmp = create_bitmap_ex ( bpp, w, h);
				convert_bitmap((BITMAP *)images[i].dat, bmp, general_attributes & MASKED);
				}
			break;
			}
		gamma_correct_bitmap(bmp, gamma_correction, general_attributes & MASKED);
		m[(i * rotations)] = create_ppcol_mask(bmp);
		b[(i * rotations)] = bmp;
		attributes[(i * rotations)] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
		}
	return;//end of irregular/masked
	}
/*SpaceSprite::SpaceSprite(int _attributes, const DATAFILE *images, int sprite_count) {
	int i, obpp;
	BITMAP *bmp;
	count = sprite_count;
	if (count < 1) error("SpaceSprite::SpaceSprite - bad parameters");
	m = new PPCOL_MASK*[count];
	b = new BITMAP*    [count];
	attributes  = new unsigned char [count];
	general_attributes = _attributes;
	w = ((BITMAP *)images[0].dat)->w;
	h = ((BITMAP *)images[0].dat)->h;
	obpp = bitmap_color_depth((BITMAP *)images[0].dat);
	bpp = bitmap_color_depth(screen);
	for(i = 0; i < sprite_count; i++) {
		if (obpp != bpp) {
			bmp = create_bitmap_ex(bpp, w, h);
			clear_to_color(bmp, bitmap_mask_color(bmp));
			if (!(general_attributes & MASKED)) 
				blit(((BITMAP*)images[i].dat), bmp, 0, 0, 0, 0, w, h);
			else _aa2_stretch_blit(((BITMAP*)images[i].dat), bmp, 0, 0, w, h, true);
			}
		else {
			bmp = create_bitmap_ex(bpp, w, h);
			blit(((BITMAP*)images[i].dat), bmp, 0, 0, 0, 0, w, h);
			}
		m[i] = create_ppcol_mask(bmp);
		gamma_correct_bitmap(bmp, gamma_correction, general_attributes & MASKED);
		b[i] = bmp;
		attributes[i] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
		}
	}*/
SpaceSprite::SpaceSprite(SpaceSprite &old) {
	int i;
	BITMAP *bmp;
	count = old.count;
	m = new PPCOL_MASK*[count];
	b = new BITMAP*    [count];
	attributes  = new char [count];
	general_attributes = old.general_attributes;
	bpp = old.bpp;
	w = old.w;
	h = old.h;
	for(i = 0; i < count; i++) {
		bmp = create_bitmap(old.b[i]->w, old.b[i]->h);
		blit(old.b[i], bmp, 0, 0, 0, 0, old.b[i]->w, old.b[i]->h);
		m[i] = create_ppcol_mask(bmp);
		b[i] = bmp;
		attributes[i] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
		}
	}
/*
SpaceSprite::SpaceSprite(const char *sourcename, const char *spritename) {
	int i;

	char buf[512];
	const char *tmpstr, *extension;

	sprintf(buf, "%s/%s.ini", sourcename, spritename);
	set_config_file(buf);
	tmpstr = get_config_string("Main", "Type", "BadType");
	if (strcmp(tmpstr, "SpaceSprite")) error("SpaceSprite(%s # %s) : %s != SpaceSprite", sourcename, spritename, tmpstr);
	count = get_config_int("SpaceSprite", "Number", 0);
	w = get_config_int("SpaceSprite", "Width", 0);
	h = get_config_int("SpaceSprite", "Height", 0);
	//tmpstr = get_config_string("SpaceSprite", "SubType", "Normal");
	extension = get_config_string("SpaceSprite", "Extension", NULL);

	m = new PPCOL_MASK*[count];
	b = new BITMAP*    [count];
	attributes  = new unsigned char [count];

	general_attributes = MASKED;

	if (count) {
		for (i = 0; i < count; i += 1) {
			if (strchr(extension, '.')) {
				sprintf(buf, "%s/%s%03d%s", sourcename, spritename, i, extension);
				b[i] = load_bitmap(buf, NULL);
				m[i] = create_ppcol_mask(b[i]);
				attributes[i] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
				}
			else {
				sprintf(buf, "%s%03d", spritename, i);
				//maybe force upper case here?
				b[i] = (BITMAP*)(load_datafile_object(sourcename, buf)->dat);
				m[i] = create_ppcol_mask(b[i]);
				attributes[i] = DEALLOCATE_IMAGE | DEALLOCATE_MASK;
				}
			}
		}
	}
/*
void save_spacesprite(SpaceSprite *ss, const char *sname, const char *dname, const char *extension) {
	int i;
	char buf[512];
	if (!ss) return;
	if (ss->frames()) {
		for (i = 0; i < ss->frames(); i += 1) {
			if (strchr(extension, '.')) {
				sprintf(buf, "tmp/%s%03d%s", sname, i, extension);
				}
			else {
				sprintf(buf, "tmp/%s%03d.bmp", sname, i);
				}
			save_bitmap(buf, ss->get_bitmap(i), NULL);
			}
		chdir("tmp");
//		if (strchr(extension, '.')) sprintf(buf, "dat ../ships/%s.dat -k -t data -a *", dname);
//		else sprintf(buf, "dat ../ships/%s.dat -t %s -a *", dname, extension);
//		system(buf);
		sprintf(buf, "md ..\\ships\\%s", dname);
		system(buf);
		sprintf(buf, "move * ..\\ships\\%s", dname);
		system(buf);
		delete_file("*");
		chdir("..");
		}
	sprintf(buf, "tmp/%s.ini", sname);
	set_config_file(buf);
	set_config_string("Main", "Type", "SpaceSprite");
	set_config_int("SpaceSprite", "Number", ss->frames());
	set_config_int("SpaceSprite", "Width", ss->width());
	set_config_int("SpaceSprite", "Height", ss->height());
	set_config_string("SpaceSprite", "SubType", "Normal");
	set_config_string("SpaceSprite", "Extension", extension);
	chdir("tmp");
	sprintf(buf, "dat ../ships/%s.dat -k -a *", dname);
//	system(buf);
	sprintf(buf, "move * ..\\ships\\%s", dname);
	system(buf);
//	delete_file("*");
	chdir("..");
	return;
	}
SpaceSprite::SpaceSprite(const char *package, const char *name) {
	const char *ext;
	char buf[512];

	sprintf(buf, "%s#%s.ini", package);
	set_config_file(buf);
	count = get_config_int(name, "Number", 0);
	rotations = get_config_int(name, "Rotation", 1);
	w = get_config_int(name, "Width", 1);
	h = get_config_int(name, "Height", 1);
	if (strcmp(get_config_string(name, "Type", NULL), "SpaceSprite"))
		error("Error loading SpaceSprite: that's not a SpaceSprite!");
	ext = get_config_string(name, "Extension", "");

	m = new PPCOL_MASK*[count * rotations];
	b = new BITMAP*    [count * rotations];
	for (int i = 0; i < count; i += 1) {
		sprintf(buf, "%s/%s%03d%s", package, name, i, ext);
		BITMAP *bmp = load_bitmap(buf, NULL);
		b[i * rotations] = bmp;
		m[i * rotations] = create_ppcol_mask(bmp);
		if (!m[i * rotations]) error ("Error loading SpaceSprite: image %s not found", buf);
		for (int j = 1; j < rotations; j += 1) {
			BITMAP *tmp = create_bitmap(w, h);
			clear_to_color(tmp, bitmap_mask_color(tmp));
			aa_rotate_sprite(tmp, bmp, 0, 0, j * (1<<24)/rotations);
			b[j + i * rotations] = tmp;
			m[j + i * rotations] = create_ppcol_mask(tmp);
			}
		}
	count *= rotations;

	return;
	}

/*
SpaceSprite::SpaceSprite(BITMAP **sprites, int sprite_count) {
	int i, j;
	BITMAP *bmp;
	rotations = 1;
	count = sprite_count;
	if (count < 1) error("SpaceSprite::SpaceSprite - bad parameters (2)");
	m = new PPCOL_MASK*[count * rotations];
	b = new BITMAP*    [count * rotations];
	w = sprites[0]->w;
	h = sprites[0]->h;
	for(i = 0; i < count; i++) {
		m[(i * rotations)] = create_ppcol_mask(sprites[i]);
		b[(i * rotations)] = sprites[i];
		}
	}
*/
SpaceSprite::~SpaceSprite() {
	int i;

	for(i = 0; i < count; i++) {
		if (attributes[i] & DEALLOCATE_IMAGE) destroy_ppcol_mask(m[i]);
		}
	delete m;
	m = NULL;

	for(i = 0; i < count; i++) {
		if (attributes[i] & DEALLOCATE_MASK) destroy_bitmap(b[i]);
		}
	delete b;
	b = NULL;

	delete attributes;
	attributes = NULL;
	}

BITMAP *SpaceSprite::get_bitmap(int index)
{
	if (index > count) error("SpaceSprite::get_bitmap - index %d > count %d", index, count);
  return(b[index]);
}

void SpaceSprite::animate_character(double x, double y, int index, int color, Frame *space) {
	if (index > count) error("SpaceSprite::animate_character - index %d > count %d", index, count);
	int ix = (int)wcorner(x, w);
	int iy = (int)hcorner(y, h);
	int spr_w = int(w * space_zoom);
	int spr_h = int(h * space_zoom);
	if(spr_w < 1) spr_w = 1;
	if(spr_h < 1) spr_h = 1;
	draw_character_stretch(ix, iy, spr_w, spr_h, index, color, space->frame);
	space->add_box(ix, iy, spr_w, spr_h);
	return;
	}

void SpaceSprite::overlay (int index1, int index2, BITMAP *dest) {
  int x, y;
	if (index1 > count) error("SpaceSprite::overlay - index1 %d > count %d", index1, count);
	if (index2 > count) error("SpaceSprite::overlay - index2 %d > count %d", index2, count);

	for(y = 0; y < h; y += 1) {
		for(x = 0; x < w; x += 1) {
			if (getpixel(b[index1], x, y) != getpixel(b[index2], x, y)) {
				putpixel(dest, x, y, getpixel(b[index2], x, y));
				}
			}
		}
	return;
	}

void SpaceSprite::draw(double x, double y, double w, double h, int index, Frame *frame) {
	if (index > count) error("SpaceSprite::draw - index %d > count %d", index, count);
	int ix, iy, iw, ih;
	ix = (int) floor(x);
	iy = (int) floor(y);
	switch (antia_on) {
		case 2: {// custom Anti-Aliasing algorithm
			_aa2_stretch_blit(b[index], frame->frame, x, y, w, h, general_attributes & MASKED);
			iw = (int) ceil(x+w) + 1 - ix;
			ih = (int) ceil(y+h) + 1 - iy;
			}
		break;
		case 1: {// generic Anti-Aliasing algorithm
			iw = (int) ceil(w);
			ih = (int) ceil(h);
			_aa_stretch_blit(b[index], frame->frame, 0, 0, b[index]->w, b[index]->h, 
					ix, iy, iw, ih, general_attributes & MASKED);
			}
		break;
		default: {// no Anti-Aliasing algorithm (fast)
			iw = (int) ceil(w);
			ih = (int) ceil(h);
			if (general_attributes & MASKED) 
				masked_stretch_blit (b[index], frame->frame, 0, 0, b[index]->w, b[index]->h, ix, iy, iw, ih);
			else 
				stretch_blit        (b[index], frame->frame, 0, 0, b[index]->w, b[index]->h, ix, iy, iw, ih);
			}
		break;
		}
	frame->add_box(ix, iy, iw, ih);
	return;
	}

void SpaceSprite::draw(int x, int y, int index, BITMAP *surface) {
	if (index > count) error("SpaceSprite::draw - index %d > count %d", index, count);
	if (general_attributes & MASKED) 
		masked_blit (b[index], surface, 0, 0, x, y, b[index]->w, b[index]->h);
	else 
		blit        (b[index], surface, 0, 0, x, y, b[index]->w, b[index]->h);
	return;
	}

void SpaceSprite::animate(double x, double y, int index, Frame *space) {
	if (index > count) error("SpaceSprite::animate - index %d > count %d", index, count);
	draw(wcorner(x, w), hcorner(y, h), b[index]->w * space_zoom, b[index]->h * space_zoom, index, space);
	return;
	}

void SpaceSprite::draw_character(int x, int y, int index, int color, BITMAP *bmp) {
	if (index > count) error("SpaceSprite::draw_character - index %d > count %d", index, count);
	PPCOL_MASK *mask = m[index];
	int row, chunk;
	for(row = 0; row < mask->h; row++) {
		for(chunk = 0; chunk <= mask->max_chunk; chunk++) {
			int c = mask->sp_mask[row][chunk];
			int cx = x + (chunk << 5);
			while (c) {
				if (c & 0x80000000) putpixel(bmp, cx, y + row, color);
				c = c << 1;
				cx += 1;
				}
			}
		}
	}

void SpaceSprite::draw_character_stretch(int x, int y, int w, int h, int index, int color, BITMAP *bmp) {
	if (index > count) error("SpaceSprite::draw_character_stretch - index %d > count %d", index, count);
	int _x, _xmin, _y, _w, _h;
	PPCOL_MASK *mask = m[index];
	if (y >= 0) _y = 0; else _y = -y;
	if (y + h <= bmp->h) _h = h; else _h = bmp->h - y;
	if (x >= 0) _xmin = 0; else _xmin = -x;
	if (x + w <= bmp->w) _w = w; else _w = bmp->w - x;
	int scale = (this->w << 16) / w;
	switch (bitmap_color_depth(bmp)) {
		case 8: {
			bmp_select(bmp);
			while (_y < _h) {
				unsigned long addr = bmp_write_line(bmp, y + _y) + x * sizeof(char);
				unsigned long *dat = mask->sp_mask[_y * this->h / h];
				_x = _xmin;
				int tx = _x * scale;
				while (_x < _w) {
					if ( (dat[(tx>>21)] << ((tx>>16) & 31)) & 0x80000000UL )
						bmp_write8(addr+_x*sizeof(char), color);
					tx += scale;
					_x += 1;
					}
				bmp_unwrite_line(bmp);
				_y += 1;
				}
			}
		break;
		case 15: {
			bmp_select(bmp);
			while (_y < _h) {
				unsigned long addr = bmp_write_line(bmp, y + _y) + x * sizeof(short);
				unsigned long *dat = mask->sp_mask[_y * this->h / h];
				_x = _xmin;
				int tx = _x * scale;
				while (_x < _w) {
					if ( (dat[(tx>>21)] << ((tx>>16) & 31)) & 0x80000000UL )
						bmp_write15(addr+_x*sizeof(short), color);
					tx += scale;
					_x += 1;
					}
				bmp_unwrite_line(bmp);
				_y += 1;
				}
			}
		break;
		case 16: {
			bmp_select(bmp);
			while (_y < _h) {
				unsigned long addr = bmp_write_line(bmp, y + _y) + x * sizeof(short);
				unsigned long *dat = mask->sp_mask[_y * this->h / h];
				_x = _xmin;
				int tx = _x * scale;
				while (_x < _w) {
					if ( (dat[(tx>>21)] << ((tx>>16) & 31)) & 0x80000000UL )
						bmp_write16(addr+_x*sizeof(short), color);
					tx += scale;
					_x += 1;
					}
				bmp_unwrite_line(bmp);
				_y += 1;
				}
			}
		break;
		case 24: {
			bmp_select(bmp);
			while (_y < _h) {
				unsigned long addr = bmp_write_line(bmp, y + _y) + x * 3;
				unsigned long *dat = mask->sp_mask[_y * this->h / h];
				_x = _xmin;
				int tx = _x * scale;
				while (_x < _w) {
					if ( (dat[(tx>>21)] << ((tx>>16) & 31)) & 0x80000000UL )
						bmp_write24(addr+_x*3, color);
					tx += scale;
					_x += 1;
					}
				bmp_unwrite_line(bmp);
				_y += 1;
				}
			}
		case 32: {
			bmp_select(bmp);
			while (_y < _h) {
				unsigned long addr = bmp_write_line(bmp, y + _y) + x * sizeof(long);
				unsigned long *dat = mask->sp_mask[_y * this->h / h];
				_x = _xmin;
				int tx = _x * scale;
				while (_x < _w) {
					if ( (dat[(tx>>21)] << ((tx>>16) & 31)) & 0x80000000UL )
						bmp_write32(addr+_x*sizeof(long), color);
					tx += scale;
					_x += 1;
					}
				bmp_unwrite_line(bmp);
				_y += 1;
				}
			}
		break;
		}
	return;
	}

int SpaceSprite::collide(int x, int y, int i, int ox, int oy, int oi,
		SpaceSprite *other)
{
	if (i > count) error("SpaceSprite::collide - index1 %d > count1 %d", i, count);
	if (oi > other->count) error("SpaceSprite::collide - index2 %d > count2 %d", oi, other->count);
  return(check_ppmask_collision(m[i], other->m[oi], x, y, ox, oy));
}

int line_collide;
int rect_x;
int rect_y;
int rect_w;
int rect_h;
int collide_x;
int collide_y;

PPCOL_MASK *rect_mask;

void check_line_collision(BITMAP *bmp, int x, int y, int d)
{
  int row, col;

  if(line_collide)
    return;

  col = x - rect_x;
  row = y - rect_y;

  if((col >= 0) && (col < rect_w) &&
     (row >= 0) && (row < rect_h)) {
    if((rect_mask->sp_mask[row][col >> 5] & (0x80000000 >> (col & 31))) != 0) {
      line_collide = TRUE;
      collide_x = x;
      collide_y = y;
    }
  }
}

int SpaceSprite::collide_line(int lx1, int ly1, int *lx2, int *ly2,
  int sx, int sy, int sindex)
{
  line_collide = FALSE;
  rect_x = sx - (w / 2);
  rect_y = sy - (h / 2);
  rect_w = w;
  rect_h = h;
	if ((rect_x   > lx1) && (rect_x   > *lx2)) return FALSE;
	if ((rect_x+w < lx1) && (rect_x+w < *lx2)) return FALSE;
	if ((rect_y   > ly1) && (rect_y   > *ly2)) return FALSE;
	if ((rect_y+h < ly1) && (rect_y+h < *ly2)) return FALSE;
  rect_mask = m[sindex];
  do_line(NULL, lx1, ly1, *lx2, *ly2, 0, check_line_collision);
  if(line_collide) {
    *lx2 = collide_x;
    *ly2 = collide_y;
    return(TRUE);
  }

  return(FALSE);
}

 int SpaceSprite::frames()
{
  return(count);
}

 double SpaceSprite::width()
{
  return((double)(w));
}

 double SpaceSprite::height()
{
  return((double)(h));
}
