#include "../ship.h"
REGISTER_FILE

class EarthlingCruiser2 : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurnRate;
  double       weaponSpread;
  int          weaponFocus;

  int    specialColor;
  double specialRange;
  int    specialFrames;
  int    specialDamage;

  public:
  EarthlingCruiser2(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
};

class EarthlingFusionBlast : public AnimatedShot {
public:
  EarthlingFusionBlast(double ox, double oy, double oangle, double ov,
        int odamage, double orange, int oarmour, double otrate, SpaceLocation *ocreator,
        SpaceObject *otarget, SpaceSprite *osprite, int ofcount, int ofsize, int ofocus);
  virtual void calculate();
protected:
  int focus;
  int growing;
  double turn_rate;
};

EarthlingCruiser2::EarthlingCruiser2(double ox, double oy, double shipAngle,
  ShipData *shipData, unsigned int code) 
	:
  Ship(ox, oy, shipAngle, shipData, code) {

        weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
        weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
        weaponDamage   = get_config_int("Weapon", "Damage", 0);
        weaponArmour   = get_config_int("Weapon", "Armour", 0);
        weaponTurnRate = scale_turning(get_config_float("Weapon", "TurnRate", 0));
        weaponSpread   = get_config_float("Weapon", "Spread", 10);
        weaponFocus    = get_config_int("Weapon", "Focus", 2);

        specialColor  = get_config_int("Special", "Color", 0);
        specialRange  = scale_range(get_config_float("Special", "Range", 0));

        specialFrames = get_config_int("Special", "Frames", 0);
        specialDamage = get_config_int("Special", "Damage", 0);

}

int EarthlingCruiser2::activate_weapon() {
        game->addItem(new EarthlingFusionBlast(0.0, h*0.45, angle, weaponVelocity,
                          weaponDamage, weaponRange, weaponArmour, weaponTurnRate, this,
                          target, data->spriteSpecial, 6, 70, weaponFocus));
        game->addItem(new EarthlingFusionBlast(0.0, h*0.45, angle + weaponSpread, weaponVelocity,
                          weaponDamage, weaponRange, weaponArmour, weaponTurnRate, this,
                          target, data->spriteSpecial, 6, 70, weaponFocus));
        game->addItem(new EarthlingFusionBlast(0.0, h*0.45, angle - weaponSpread, weaponVelocity,
                          weaponDamage, weaponRange, weaponArmour, weaponTurnRate, this,
                          target, data->spriteSpecial, 6, 70, weaponFocus));
        return(TRUE);
}

int EarthlingCruiser2::activate_special() {
  int fire = FALSE;
  SpaceObject *o;

	Query a;
	for (a.begin(this, bit(LAYER_SHIPS) + bit(LAYER_SHOTS) + bit(LAYER_SPECIAL) + 
			bit(LAYER_CBODIES), specialRange); a.current; a.next()) {
		o = a.currento;
		if (!o->isInvisible() && !o->sameTeam(this) && (o->collide_flag_anyone&bit(LAYER_LINES))) {
			game->addItem(new PointLaser(this, pallete_color[specialColor], 1, 
					specialFrames, this, o, 0.0, 0.0));
			fire = TRUE;
			}
		}
	if (fire) sound.play((SAMPLE *)(melee[MELEE_BOOM + 0].dat));

	return(fire);
	}

EarthlingFusionBlast::EarthlingFusionBlast(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, SpaceLocation *ocreator,
    SpaceObject *otarget, SpaceSprite *osprite, int ofcount, int ofsize, int ofocus) :
    AnimatedShot(ocreator, ox, oy, oangle, ov, odamage, orange, oarmour, ocreator,
                 osprite, ofcount, ofsize)
{
        growing = true;
        turn_rate = otrate;
        target = otarget;
        focus = ofocus;
        explosionSprite     = data->spriteWeaponExplosion;
        explosionFrameCount = 11;
        explosionFrameSize  = 35;
        explosionSample = data->sampleWeapon[1];
}

void EarthlingFusionBlast::calculate() {
        Shot::calculate();
        frame_step -= frame_time;
        while (frame_step < 0) {
                frame_step += frame_size;
                sprite_index++;
                if (sprite_index == frame_count) {
                        sprite_index = 0;
                        if (growing) {
                                sprite = data->spriteWeapon;
                                frame_size = 150;
                                frame_step = frame_size;
                                growing = false; }
                }
        }
                
        if (target==NULL) return;
        else    if (!target->exists()) return;
        if ((!target->isInvisible())&&(ship)) {

                double d_a = normalize(trajectory_angle(target) - angle, 360);
                if (d_a > 180) d_a -= 360;
                double maneur_v;
                if (d * focus > range) maneur_v = turn_rate * frame_time;
                else             maneur_v = turn_rate *frame_time * focus * d / range;
                if (fabs(d_a) < maneur_v)
                        angle = trajectory_angle(target);
                else
                        if (d_a > 0) angle += maneur_v;
                        else         angle -= maneur_v;
                changeDirection(angle);
        }
 
}




REGISTER_SHIP(EarthlingCruiser2)
