#include "../ship.h"
REGISTER_FILE
#include <string.h>

class EstionPlatform;
class EstionGunner : public Ship {
	double weaponRange;
	double weaponVelocity;
	int    weaponDamage;
	int    weaponArmour;

	int    specialArmour;

	public:
	EstionGunner(double ox, double oy, double shipAngle,
		ShipData *shipData, unsigned int code);

	int num_platforms;
	int max_platforms;
	EstionPlatform **platform;

	virtual int activate_weapon();
	virtual int activate_special();
	};

class EstionShot : public Shot {
	public:
	EstionGunner *mother_ship;
	EstionShot(double x, double y, double angle, double velocity, int damage, 
			double range, int armour, EstionGunner *ship);
	SpaceLocation *last_shooter;
	virtual void inflict_damage(SpaceObject *other) ;
	};

class EstionPlatform : public SpaceObject {
	int health;
	int rotate_time;
	int rotate_direction;
	public:
	EstionPlatform (EstionGunner *ship, int health);
	virtual void handle_damage (SpaceLocation *source) ;
	virtual void ship_died();
	virtual void calculate();
	virtual void death();
	};

void EstionPlatform::handle_damage (SpaceLocation *source) {
	health -= damage + directDamage * 2;
	if (health < 0) state = 0;
	damage = 0;
	directDamage = 0;
	return;
	}
void EstionPlatform::ship_died() {
	state = 0;
	}
void EstionPlatform::calculate() {
	SpaceObject::calculate();
	rotate_time -= frame_time;
	while (rotate_time < 0) {
		rotate_time += 50;
		sprite_index = (sprite_index + rotate_direction) & 7;
		}
	vx *= 1 - .0005 * frame_time;
	vy *= 1 - .0005 * frame_time;
	if (vx * vx + vy * vy < 0.05 * 0.05) {
		vx *= 0;
		vy *= 0;
		}
	return;
	}
void EstionPlatform::death() {
	if (!ship) return; //ship is already dead; don't remove platform from list
	if (ship->data != data) return; //ship is somehow not an Estion; perhaps we've been stolen
	for (int i = 0; i < ((EstionGunner*)ship)->num_platforms; i += 1) {
		if (((EstionGunner*)ship)->platform[i] == this) {
			((EstionGunner*)ship)->platform[i] = NULL;
			((EstionGunner*)ship)->num_platforms -= 1;
			memmove(&((EstionGunner*)ship)->platform[i], &((EstionGunner*)ship)->platform[i+1], 
				(((EstionGunner*)ship)->num_platforms-i) * sizeof(EstionPlatform*));
			return;
			}
		}
	return;
	}

EstionPlatform::EstionPlatform (EstionGunner *ship, int health) : 
		SpaceObject(ship, ship->normal_x(), ship->normal_y(), ship->get_angle(), 
		ship->data->spriteSpecial)
	{
	layer = LAYER_SPECIAL;
	mass = 3;
	collide_flag_sameship = bit(LAYER_SHOTS);
	this->health = health;
	this->ship = ship;
	vx = ship->get_vx() + .3 * cos(angle*PI/180);
	vy = ship->get_vy() + .3 * sin(angle*PI/180);
	rotate_time = 0;
	if (random() & 1) rotate_direction = 1; else rotate_direction = -1;
	}

EstionGunner::EstionGunner(double ox, double oy, double shipAngle,
		ShipData *shipData, unsigned int code) :
  Ship(ox, oy, shipAngle, shipData, code)
	{
	if (!data->spriteSpecial || (data->spriteSpecial->frames() != 8))
		error("you have the wrong\n version of shpestgu.dat");

	weaponDamage = get_config_int("Weapon", "Damage", 0);
	weaponArmour = get_config_int("Weapon", "Armour", 0);
	weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
	weaponVelocity  = scale_velocity(get_config_float("Weapon", "Velocity", 0));

	specialArmour = get_config_int( "Special", "Armour", 0);
	max_platforms = get_config_int( "Special", "Number", 0);
	num_platforms = 0;
	platform = new EstionPlatform*[max_platforms];
	for (int i = 0; i < max_platforms; i += 1) {
		platform[i] = NULL;
		}
	}

EstionShot::EstionShot(double x, double y, double angle, double velocity, 
											 int damage, double range, int armour, EstionGunner *ship) :
		Shot(ship, x, y, angle, velocity, damage, range, armour, ship, 
				ship->data->spriteWeapon, 0)
	{
	collide_flag_sameship = bit(LAYER_SPECIAL);
	last_shooter = ship;
	mother_ship = ship;
	}

void EstionShot::inflict_damage(SpaceObject *other) {
	if (!sameShip(other)) {
		Shot::inflict_damage(other);
		return;
		}
	if (other == last_shooter) return;
	if (!target || target->isInvisible()) return;
	last_shooter = other;
	d = 0.0;
	SpaceLocation *tmp = ship->target;
	if (!tmp) return;
	double mr = distance(ship->target);
	for (int i = 0; i < mother_ship->num_platforms; i += 1) {
		double r = mother_ship->platform[i]->distance(ship->target);
		if ((mother_ship->platform[i] != other) && (r < mr) && (distance(mother_ship->platform[i]) < range)) {
			tmp = mother_ship->platform[i];
			mr = r;
			}
		}
	double tx, ty, rr;
	tx = min_delta(x, tmp->normal_x(), X_MAX);
	ty = min_delta(y, tmp->normal_y(), Y_MAX);
	rr = sqrt(tx*tx + ty*ty) / v;
	tx -= tmp->get_vx() * rr;
	ty -= tmp->get_vy() * rr;
	changeDirection( (PI+atan3(ty, tx)) * 180/PI );
	return;
	}

int EstionGunner::activate_weapon() {
	Shot *shot = new EstionShot(0.0, (height() / 4.0),
		angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
		this);
	game->addItem(shot);
	return(TRUE);
	}

int EstionGunner::activate_special() {
	if (num_platforms == max_platforms) {
		num_platforms -= 1;
		platform[0]->state = 0;
		memcpy(&platform[0], &platform[1], sizeof(EstionPlatform*) * num_platforms);
		platform[num_platforms] = NULL;
		}
	EstionPlatform *tmp = new EstionPlatform( this, specialArmour);
	platform[num_platforms] = tmp;
	num_platforms += 1;
	game->addItem( tmp );
	return(TRUE);
	}




REGISTER_SHIP(EstionGunner)
