#include "../ship.h"
REGISTER_FILE

# define NORMAL_SPEED 0
# define TURBO_SPEED 1
# define SUPER_SPEED 2
# define HYPER_SPEED 3

class KterbiSaber : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  int          boostlevel;
  int          normal_recharge;
  double       normal_speed;
  double       normal_acc;
  int          normal_rate;
  int          normal_rof;
  double       normal_turning;


  public:
  KterbiSaber(double ox, double oy, double shipAngle, ShipData *data, unsigned int code);

  virtual void calculate_fire_weapon();
  virtual void calculate_fire_special();
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void handle_damage(SpaceLocation *source);

};

class KterbiIonBlast : public AnimatedShot {

  int min_damage;

  public:
  KterbiIonBlast(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
};

KterbiSaber::KterbiSaber(double ox, double oy, double shipAngle,
  ShipData *data, unsigned int code) :
  Ship(ox, oy, shipAngle, data, code)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  boostlevel     = NORMAL_SPEED;
  normal_recharge= recharge_amount;
  normal_rate    = recharge_rate;
  normal_speed   = speed_max;
  normal_acc     = accel_rate;
  normal_rof     = weapon_rate;
  normal_turning = turn_rate;
  collide_flag_sameship |= bit(LAYER_SHOTS);
}

void KterbiSaber::calculate_fire_weapon()
{
  if ((batt < weapon_drain) && (crew > 1)) {
    crew--;
    batt += batt_max; }
  Ship::calculate_fire_weapon();
  if ((batt > batt_max) && (crew < crew_max)) {
    crew++;
    batt -= batt_max;
  }
  if (batt > batt_max)
    batt = batt_max;
}

void KterbiSaber::calculate_fire_special()
{
  if ((batt < special_drain) && (crew > 1)) {
    crew--;
    batt += batt_max; }
  Ship::calculate_fire_special();
  if ((batt > batt_max) && (crew < crew_max)) {
    crew++;
    batt -= batt_max;
  }
  if (batt > batt_max)
    batt = batt_max;
}

int KterbiSaber::activate_weapon()
{
  game->addItem(new KterbiIonBlast(-width()*.35, height()/3,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 10 ,1));
  game->addItem(new KterbiIonBlast(width()*.35, height()/3,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 10 ,1));
  return(TRUE);
}

int KterbiSaber::activate_special()
{
    if (boostlevel < 3) {
    boostlevel++;
    sound.stop(data->sampleExtra[1]);
    sound.play(data->sampleExtra[1]);
    if (boostlevel == TURBO_SPEED) {
        speed_max = normal_speed * 1.5;
        recharge_step = 0;
        recharge_amount = 0;
        accel_rate = normal_acc * 1.5;
        turn_rate = normal_turning * 1.33;
        weapon_rate = int(normal_rof / 1.33);
    } else if (boostlevel == SUPER_SPEED) {
        recharge_amount = -normal_recharge;
        recharge_step = 0;
        speed_max = normal_speed * 2;
        accel_rate = normal_acc * 2;
        turn_rate = normal_turning * 1/66;
        weapon_rate = int(normal_rof / 1.66);
    } else if (boostlevel == HYPER_SPEED) {
        recharge_rate = (normal_rate/2);
        speed_max = normal_speed * 2.5;
        recharge_step= 0;
        accel_rate = normal_acc * 2.5;
        turn_rate = normal_turning * 2;
        weapon_rate = int(normal_rof / 2);
    }
    return (TRUE); }
    else return (FALSE);
}



void KterbiSaber::calculate()
{
  if (((!fire_special) || ((crew == 1) && (batt == 0)) || (crew_max <= 2)) && (boostlevel != 0)) {
      boostlevel = 0;
      recharge_amount = normal_recharge;
      recharge_rate = normal_rate;
      speed_max = normal_speed;
      accel_rate = normal_acc;
      turn_rate = normal_turning;
      weapon_rate = normal_rof;
      recharge_step = 0;}

  if (boostlevel >= 1) {
      int chance;
      chance = random() % 10000;
      if (chance < (3 * frame_time)) {
        crew_max -= 2;
        if (crew > crew_max)
          crew = crew_max;
			   blit(spritePanel->get_bitmap(7), spritePanel->get_bitmap(0),0,0,3,50 - (crew_max),9,4);
      ship->update_panel = TRUE;
      sound.stop(data->sampleExtra[0]);
      sound.play(data->sampleExtra[0]);

      }
    }

  Ship::calculate();

  if ((batt < 0) && (crew!=1)) {
    batt += batt_max;
    crew--;
  } else if (batt < 0)
    batt=0;

  if ((crew!=crew_max) && (batt >= batt_max)) {
    batt -= batt_max;
    crew++;
  }
}

void KterbiSaber::handle_damage(SpaceLocation *source) {
  if (speed_loss > 0) {
    speed_max = normal_speed;
    accel_rate = normal_acc;
    Ship::handle_damage(source);
    normal_speed = speed_max;
    normal_acc = accel_rate;
    if (boostlevel == TURBO_SPEED) {
      speed_max = normal_speed * 1.5;
      accel_rate = normal_acc * 2; }
    else if (boostlevel == SUPER_SPEED) {
        speed_max = normal_speed * 2;
        accel_rate = normal_acc * 2; }
      else if (boostlevel == HYPER_SPEED) {
          speed_max = normal_speed * 2.5;
          accel_rate = normal_acc * 2.5; } }
  else
    Ship::handle_damage(source);
}


KterbiIonBlast::KterbiIonBlast(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(oship, ox, oy, oangle, ov, odamage, orange, oarmour, oship,
      osprite, ofcount, ofsize,1)
{
  min_damage=odamage;
}

void KterbiIonBlast::calculate()
{
  AnimatedShot::calculate();
  if (d < (range/2))
    damage_factor = min_damage;
  else
    damage_factor = min_damage * 2;
}



REGISTER_SHIP(KterbiSaber)
