#include "../ship.h"

#define RAALRITH_POISON_ID     0x2233

class RaalrithMenacer : public Ship {

  int          weaponFrames;
  int 	       drillFrames;

  int          suckRate;
  int          suckStart;

  int          latchOnFrames;
  int          latchOnTimer;

  int          kickDamage;

  Ship         *grabbed;
  double       grabangle;
  double       grabdistance;
  double       grabshipangle;

  int          phaseFrames;
  int          phaseTimer;
  int          phase;
  int          phaseIn;
  int          phaseOld;

  int          lock;
  int          lockFrames;
  int          lockTimer;
  
  public:
  RaalrithMenacer(double ox, double oy, double shipAngle,
		ShipData *shipData, unsigned int code);

  int          latched;
  int          poisonDuration;

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual int canCollide(SpaceObject *other);
  virtual void animate(Frame *space);
  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage(SpaceLocation *source);
  
};

class RaalrithPoison : public SpaceObject {
  public:

  int duration;
  int start;
  Ship *oship;
  RaalrithMenacer *owner;

  RaalrithPoison(RaalrithMenacer *creator, int duration, Ship *kill_it, SpaceSprite *osprite);
  virtual void calculate();
  virtual void animate(Frame *space);
};

RaalrithMenacer::RaalrithMenacer(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code) 
	:
  Ship(ox, oy, shipAngle, shipData, code)

{
  weaponFrames   = get_config_int("Weapon", "Frames", 0);
  drillFrames    = 0;

  suckRate       = get_config_int("Weapon", "SuckRate", 0);
  suckStart      = TRUE;

  latchOnFrames  = get_config_int("Weapon", "LatchFrames", 0);
  latchOnTimer   = 0;

  kickDamage     = get_config_int("Weapon", "KickDamage", 0);

  latched        = FALSE;
  grabbed        = NULL;

  lockFrames     = get_config_int("Special", "LockFrames", 0);
  lock           = 0;
  lockTimer      = 0;

  phaseFrames    = get_config_int("Extra", "PhaseMax", 0);
  phaseTimer     = phaseFrames;
  phase          = 100;
  phaseIn        = FALSE;
  phaseOld       = 0;

  poisonDuration = get_config_int("Extra", "PoisonDuration", 0);

}

int RaalrithMenacer::activate_weapon()
{
  if (drillFrames <= 0 && !(latched)) {
    drillFrames = weaponFrames;
       return(TRUE);
  }
  if ((latched) && (grabbed != NULL) && (latchOnTimer <= 0)) {
    grabbed->damage += kickDamage;
    grabbed->handle_damage(this);
    latched = FALSE;
    suckStart = TRUE;
    accelerate (this, angle + 180, 15, GLOBAL_MAXSPEED);
    target->accelerate (this, angle, 30, GLOBAL_MAXSPEED);
    return(TRUE);
  }
  return(FALSE);
}

int RaalrithMenacer::activate_special()
{
  if(lockTimer <= 0) {
    lockTimer = lockFrames;
    lock = phase;
    return(TRUE);
  }
  return(FALSE);
}

void RaalrithMenacer::calculate() {
   if(drillFrames > 0) {
     drillFrames-= frame_time;
     if ((drillFrames <= 0) && (!latched)) {
       sound.stop(data->sampleWeapon[0]);
       sound.play(data->sampleWeapon[0]);
     }
   }

   if (latched) {
     if (suckStart) {
       suckStart = FALSE;
       latchOnTimer = latchOnFrames;
       game->addItem( new RaalrithPoison(this, poisonDuration, (Ship*)grabbed, data->spriteWeapon));
     }
     if (latchOnTimer > 0)
       latchOnTimer -= frame_time;
   }

   phaseTimer -= frame_time;
   if (phaseTimer < 0) {
     if (phaseIn)
       phase++;
     else
       phase--;
     phaseTimer = phaseFrames;
   }   
   if (phase <= 0) {
     phase = 0;
     phaseIn = TRUE;
   }
   if (phase >= 100) {
     phase = 100;
     phaseIn = FALSE;
   }
   if (int((double(phase)/100)*30) != int((double(phaseOld)/100)*30)) { 
		 BITMAP *bmp = spritePanel->get_bitmap(0);
     rectfill(bmp, 15, 14, 48, 15, pallete_color[0]);
     rectfill(bmp, 15, 14, int(14 + (double(phase)/100)*30), 15, pallete_color[9]);
     phaseOld = phase;
     ship->update_panel = TRUE;
   }

   if (grabbed != NULL)
     if (!(grabbed ->exists())){
       latched = FALSE;
       latchOnTimer = -50;
       grabbed = NULL;
     }
   if (latched) {
     if ((random() % suckRate) <= grabbed->crew) {
       grabbed->damage += 1;
       crew +=1;
       grabbed->handle_damage(this);
     }
     grabangle = (grabbed->get_angle() - grabshipangle) + grabangle;
     angle=grabangle;
     grabshipangle = grabbed->get_angle();
     nextkeys &= ~(KEYFLAG_LEFT | KEYFLAG_RIGHT | KEYFLAG_THRUST);

     x = grabbed->normal_x()-((cos(grabangle * ANGLE_RATIO)) * grabdistance);
     y = grabbed->normal_y()-((sin(grabangle * ANGLE_RATIO)) * grabdistance);
   }

   if(lockTimer > 0) {
     lockTimer -= frame_time;
     phase = lock;
   }

   if (phase <= 10)
     collide_flag_anyone=0;
   else 
     collide_flag_anyone=ALL_LAYERS;

   Ship::calculate();
}

int RaalrithMenacer::canCollide(SpaceObject *other)
{
  if ((latched) && (grabbed!=NULL) && (grabbed->exists())) {
    if (grabbed == other)
      return (FALSE);
  }
  return (Ship::canCollide(other));
}
 
void RaalrithMenacer::animate(Frame *space)
{
  if(drillFrames > 0)
    data->spriteWeapon->animate( x, y, sprite_index, space);
  else
    sprite->animate( x, y, sprite_index, space);
  if(latched)
    data->spriteWeapon->animate( x, y, sprite_index, space);
}


void RaalrithMenacer::inflict_damage(SpaceObject *other)
{
  if (drillFrames > 0)
    if (!latched)
      if ((!(sameTeam(other))) && (other->isShip())) {
        latched=TRUE;
        grabbed= (Ship *) other;
        grabangle= (trajectory_angle(other) );
        grabdistance = (distance(other) * 1.1);
        grabshipangle = (other->get_angle());
        sound.stop(data->sampleExtra[1]);
        sound.play(data->sampleExtra[1]);
      }
  Ship::inflict_damage(other);
}

int round(double a) {
  return (a - int(a)) > 0.5 ? int(a) + 1 : int(a);
}

void RaalrithMenacer::handle_damage(SpaceLocation *source) {
  damage = round((double)damage*((double)phase/50)); // full phase = 2x damage, full out = 1/2 damage
  Ship::handle_damage(source);
  return;
}


/******* Poison Definitions ***********************************************************/

RaalrithPoison::RaalrithPoison(RaalrithMenacer *creator, int nduration, Ship *nship, SpaceSprite *osprite) :
        SpaceObject (creator, nship->normal_x(), nship->normal_y(), 0.0, osprite),
	duration(nduration),
	oship(nship)
{

  owner = creator;
  target = oship;
  id |= RAALRITH_POISON_ID;
  layer = LAYER_HOTSPOTS;
  start = TRUE;
  collide_flag_anyone = 0;
  Query q;
  for (q.begin(oship, bit(LAYER_HOTSPOTS), 10); q.current; q.next()) {
    if ((q.current->getID() == getID()) && (((RaalrithPoison*)q.current)->oship == oship)) {
      ((RaalrithPoison*)q.current)->duration = duration;
      state = 0;
    }
  }
  q.end();
}

void RaalrithPoison::calculate() {
  int chance;

  x = oship->normal_x();
  y = oship->normal_y();
  vx = oship->get_vx();
  vy = oship->get_vy();
  duration -= frame_time;

  if (duration < 0) {
    blit(oship->data->spritePanel->get_bitmap(0), oship->spritePanel->get_bitmap(0), 16 , 18, 16, 18, 32, 30);
    oship->update_panel = TRUE;
    state = 0;
    return;
  }

  if (owner != NULL)
    if (owner->latched)
      duration = owner->poisonDuration;

  if (!oship->exists()) {
    state = 0;
  } else {
    if (start) {
      start = FALSE;
      draw_sprite(oship->spritePanel->get_bitmap(0), data->spriteExtra->get_bitmap(0),16,18);
      oship->update_panel = TRUE;
    }
    chance = random() % 2000;
    if (chance < frame_time) {
      oship->directDamage++;
      sound.play(data->sampleExtra[0]);
      oship->handle_damage(this);
    }
  }

  return;
}

void RaalrithPoison::animate(Frame *space)
{ 
}

REGISTER_SHIP (RaalrithMenacer)
