#include "../ship.h"
REGISTER_FILE

class TauHJPortal;

class TauHJ  :  public Ship
{
  double        weaponRange;
  double        weaponVelocity;
  int           weaponLength;
  double        weaponSpread;
  bool          weaponAlternating;
  int           weaponNumber;
  double        weaponRelativity;

  double        specialRange;
  double        specialVelocity;
  int           specialDamage;
  int           specialArmour;
  double        specialBlastRadius;

  double        extraVelocity;
  double        extraDrain;
  double        extraExitVelocityFactor;
  int           extraFrameTime;

  double        residual_drain;
  int           weapon_side;
  int           ship_recharge_amount;
  int           exit_countdown;
  double        ex, ey, vvx, vvy;
  double        s_a;

  public:

  double        extraCriticalRange;
  double        extraCriticalAngle;
  bool          holding_spec;
  bool          in_jump;
  bool          just_exited;

  TauHJ        (double ox, double oy, double shipAngle, ShipData *shipData, unsigned int code);

  virtual  int  activate_weapon();
          void  jump_in(TauHJPortal *portal);
  virtual void  calculate_fire_special();
  virtual  int  activate_special();
  virtual void  calculate();
  virtual void  handle_damage(SpaceLocation* source);
  virtual  int  canCollide(SpaceLocation *other);
  virtual  int  translate(double dx, double dy);
  virtual  int  accelerate(SpaceLocation *source, double angle, double vel, double max_speed);
  virtual void  animate(Frame *space);
  virtual  int  isProtected();
  virtual  int  isInvisible();
  virtual void  calculate_hotspots();
  virtual void  calculate_turn_left();
  virtual void  calculate_turn_right();
};


class TauHJLaser : public SpaceLine
{
  double        range, d, vel;

  public:

  TauHJLaser   (double ox, double oy, double oangle, double ov, double orange, double olength, SpaceLocation *creator, double relativity);

  virtual void  calculate();
  virtual void  inflict_damage(SpaceObject *other);
};


class TauHJPortal : public Animation {

  TauHJ        *ship;
  
  public:
  TauHJPortal  (SpaceLocation *creator, double ox, double oy, SpaceSprite *osprite, int oft, TauHJ *oship);
  virtual void  calculate();
};


class TauHJShot : public Shot
{
  double        blast_damage, blast_radius;
  int           extraFrameTime;
  SpaceSprite  *expl_sprite, *portal_sprite;
  TauHJ        *ship;
  
  public:
  
  TauHJShot    *other_shot;
  
  TauHJShot    (double ox, double oy, double oangle, double ov, int odamage, double orange, double obr, int oarmour, TauHJ *creator,
                SpaceSprite *osprite, SpaceSprite *esprite, SpaceSprite *psprite, int oft, TauHJShot *other = NULL);
  virtual void  calculate();
};




TauHJ::TauHJ (double ox, double oy, double shipAngle, ShipData *shipData, unsigned int code) :
        Ship(ox, oy, shipAngle, shipData, code)
{
        weaponRange             = scale_range(get_config_float("Weapon", "Range", 0));
        weaponVelocity          = scale_velocity(get_config_float("Weapon", "Velocity", 0));
        weaponLength            = get_config_int("Weapon", "Length", 0);
        weaponSpread            = get_config_float("Weapon", "Spread", 0);
        weaponAlternating       = (get_config_int("Weapon", "Alternating", 0) != 0);
        weaponNumber            = get_config_int("Weapon", "Number", 1);
        weaponRelativity        = get_config_float("Weapon", "Relativity", 0.0);

        specialRange            = scale_range(get_config_float("Special", "Range", 0));
        specialVelocity         = scale_velocity(get_config_float("Special", "Velocity", 0));
        specialDamage           = get_config_int("Special", "Damage", 0);
        specialArmour           = get_config_int("Special", "Armour", 0);
        specialBlastRadius      = get_config_float("Special", "BlastRadius", 0);

        extraVelocity           = scale_velocity(get_config_float("Extra", "Velocity", 0));
        extraDrain              = get_config_float("Extra", "Drain", 0) / 1000.0;
        extraExitVelocityFactor = get_config_float("Extra", "ExitVelocityFactor", 1.0);
        extraCriticalAngle      = get_config_float("Extra", "CriticalAngle", 60);
        extraCriticalRange      = get_config_float("Extra", "CriticalRange", 0);
        extraFrameTime          = get_config_int("Extra", "FrameTime", 50);

        weapon_side             = 1 - 2 * (random()%2);
        in_jump                 = false;
        ship_recharge_amount    = recharge_amount;
        exit_countdown          = 0;
        holding_spec            = false;
        just_exited             = false;
}

int TauHJ::activate_weapon()
{
        double  a, b, c;
        int     i;
        if (in_jump) return false;
        if (!weaponAlternating) {
                for (i = 0; i < weaponNumber; i++) {
                        a = (1 - (random()%201)/100.0);
                        b = (1 - 0.1*(random()%201)/100.0);
                        c = (1 - 0.1*(random()%201)/100.0);
                        game->addItem(new TauHJLaser(32 * weapon_side, 4, angle + weaponSpread * a,
                                          weaponVelocity * c, weaponRange * (1 - 0.2*fabs(a)) * b,
                                          weaponLength, this, weaponRelativity)); }
                weapon_side *= -1; }

        for (i = 0; i < weaponNumber; i++) {
                a = (1 - (random()%201)/100.0);
                b = (1 - 0.1*(random()%201)/100.0);
                c = (1 - 0.1*(random()%201)/100.0);
                game->addItem(new TauHJLaser(32 * weapon_side, 4, angle + weaponSpread * a,
                                  weaponVelocity * c, weaponRange * (1 - 0.2*fabs(a)) * b,
                                  weaponLength, this, weaponRelativity)); }
        weapon_side *= -1;
        return true;
}

void TauHJ::jump_in(TauHJPortal *portal)
{
        double a = normalize(atan3(vy, vx) * 180 / PI - angle, 360);
        if (a > 180) a -= 360;
        if (fabs(a) > extraCriticalAngle) return;
        in_jump = true;
        residual_drain  = 0;
        recharge_amount = 0;
        vvx = extraVelocity * cos(angle * ANGLE_RATIO);
        vvy = extraVelocity * sin(angle * ANGLE_RATIO);
        vx = 0; vy = 0;
}

void TauHJ::calculate_fire_special()
{
        special_low = false;
        if (!fire_special) {
                holding_spec = false;
                if (in_jump && (exit_countdown <= 0)) {
                        exit_countdown = extraFrameTime * 16;
                        ex = normalize(x + vvx * extraFrameTime * 16, X_MAX);
                        ey = normalize(y + vvy * extraFrameTime * 16, Y_MAX);
                        game->addItem(new Animation(this, ex, ey, data->spriteSpecial, 0, 41, extraFrameTime, LAYER_HOTSPOTS)); } }
         else {
                Ship::calculate_fire_special();
                holding_spec = true; }
}

int TauHJ::activate_special()
{
        if (holding_spec || in_jump) return false;
        TauHJShot *l = (TauHJShot*)game->addItem(new TauHJShot(-5, 36, angle, specialVelocity, specialDamage, specialRange, specialBlastRadius, specialArmour, this,
                       data->spriteWeapon, data->spriteWeaponExplosion, data->spriteSpecial, extraFrameTime));
        TauHJShot *r = (TauHJShot*)game->addItem(new TauHJShot(+5, 36, angle, specialVelocity, specialDamage, specialRange, specialBlastRadius, specialArmour, this,
                       data->spriteWeapon, data->spriteWeaponExplosion, data->spriteSpecial, extraFrameTime, l));
        l->other_shot = r;
        return true;
}

void TauHJ::calculate()
{
        just_exited = false;
        Ship::calculate();
        if (in_jump) {
                vx = 0; vy = 0;
                if (exit_countdown > 0) {
                        exit_countdown -= frame_time;
                        if (exit_countdown <= 0) {
                                in_jump = false;
                                just_exited = true;
                                x = ex; y = ey;
                                recharge_amount = ship_recharge_amount;
                                double vvv = sqrt(vvx*vvx + vvy*vvy);
                                vx = speed_max * extraExitVelocityFactor * vvx / vvv;
                                vy = speed_max * extraExitVelocityFactor * vvy / vvv; }
                        else {
                                x = normalize(ex - vvx * exit_countdown, X_MAX);
                                y = normalize(ey - vvy * exit_countdown, Y_MAX); } }
                else {
                        vvx = extraVelocity * cos(angle * ANGLE_RATIO);
                        vvy = extraVelocity * sin(angle * ANGLE_RATIO);
                        x = normalize(x + vvx * frame_time, X_MAX);
                        y = normalize(y + vvy * frame_time, Y_MAX);
                        if (batt <= extraDrain*extraFrameTime*16) activate_special(); }
                residual_drain += extraDrain * frame_time;
                batt -= (int)floor(residual_drain);
                if (batt < 0) batt = 0;
                residual_drain -= floor(residual_drain); }
}
void TauHJ::handle_damage(SpaceLocation *source) {
        if (in_jump) {
                damage = 0;
                directDamage = 0;
                return; }
        if (just_exited && source->isPlanet())
                damage = crew+1;
        Ship::handle_damage(source);
}

int TauHJ::canCollide(SpaceLocation *other)
{
        if (in_jump)
                return false;
        else    return Ship::canCollide(other);
}

int TauHJ::translate(double dx, double dy)
{
        if (!in_jump) return Ship::translate(dx, dy);
        return false;
}

int TauHJ::accelerate(SpaceLocation *source, double oangle, double vel, double omax_speed)
{
        if (!in_jump) return Ship::accelerate(source, oangle, vel, omax_speed);
        return false;
}

void TauHJ::animate(Frame* space)
{
        if (!in_jump) Ship::animate(space);
}

int TauHJ::isProtected()
{
        if (in_jump)
                return true;
        else    return Ship::isProtected();
}

int TauHJ::isInvisible()
{
        if (in_jump)
                return true;
        else    return Ship::isInvisible();
}

void TauHJ::calculate_hotspots()
{
        if (!in_jump) Ship::calculate_hotspots();
}

void TauHJ::calculate_turn_left()
{
        if (exit_countdown <= 0) Ship::calculate_turn_left();
}

void TauHJ::calculate_turn_right()
{
        if (exit_countdown <= 0) Ship::calculate_turn_right();
}


TauHJLaser::TauHJLaser (double ox, double oy, double oangle, double ov, double orange, double olength, SpaceLocation *creator, double relativity) :
        SpaceLine(creator, creator->normal_x(), creator->normal_y(), oangle, olength, makecol(255,255,255)), range(orange), d(0), vel(ov)
{
        double alpha = creator->get_angle() * ANGLE_RATIO;
        double tx = cos(alpha), ty = sin(alpha);
        x += oy * tx - ox * ty;
        y += oy * ty + ox * tx;
        damage_factor = 1;
        vx = ov * cos (angle * ANGLE_RATIO) + creator->get_vx() * relativity;
        vy = ov * sin (angle * ANGLE_RATIO) + creator->get_vy() * relativity;
}

void TauHJLaser::calculate()
{
        double r = (d) / range;
        int g = (int)floor(255 - r*300);
        if (g < 0) g = 0;
        int b = (int)floor(255 - r*400);
        if (b < 0) b = 0;
        color = makecol(255 - int(r*150), g, b);
        SpaceLine::calculate();
        d += vel * frame_time;
        if (d > range) state = 0;
}

void TauHJLaser::inflict_damage(SpaceObject *other)
{
        SpaceLine::inflict_damage(other);
        state = 0;
}


TauHJPortal::TauHJPortal(SpaceLocation *creator, double ox, double oy, SpaceSprite *osprite, int oft, TauHJ *oship) :
        Animation(creator, ox, oy, osprite, 0, 41, oft, LAYER_HOTSPOTS), ship(oship)
{
}

void TauHJPortal::calculate()
{
        Animation::calculate();
        if (ship) {
                if ((!ship->exists()) || (!ship->holding_spec) || ship->in_jump) {
                        ship = NULL; return; }
                if ((sprite_index >= 10) && (sprite_index <= 25) && (distance(ship) < ship->extraCriticalRange))
                        ship->jump_in(this); }
}


TauHJShot::TauHJShot (double ox, double oy, double oangle, double ov, int odamage, double orange, double obr, int oarmour, TauHJ *creator,
                   SpaceSprite *osprite, SpaceSprite *esprite, SpaceSprite *psprite, int oft, TauHJShot *other) :
        Shot(creator, ox, oy, oangle, ov, 1, orange, oarmour, creator, osprite, 1.0)
{
        other_shot     = other;
        expl_sprite    = esprite;
        portal_sprite  = psprite;
        ship           = creator;
        extraFrameTime = oft;
        blast_radius   = obr;
        blast_damage   = odamage;
}

void TauHJShot::calculate()
{
        if (state == 0) return;
        sprite_index = (int)floor(20 * d / range);
        damage_factor = (int)ceil(blast_damage * d / range);
        Shot::calculate();
        if (other_shot) if (!other_shot->exists()) other_shot = NULL;
        if (ship) if ((!ship->exists()) || (!ship->holding_spec) || ship->in_jump) ship = NULL;
        if (d >= range) {
                state = 1;
                if (ship && other_shot) if (distance(other_shot) < 30) {
                        double a = normalize(ship->get_angle() - angle, 360);
                        if (a > 180) a -= 360;
                        if (fabs(a) < ship->extraCriticalAngle) {
                                state = 0;
                                x = normalize((x + other_shot->normal_x())/2.0, X_MAX);
                                y = normalize((y + other_shot->normal_y())/2.0, Y_MAX);
                                other_shot->state = 0;
                                game->addItem(new TauHJPortal(this,x, y, portal_sprite, extraFrameTime, ship)); } }
                if (state != 0) {
                        game->addItem(new Animation(this, x, y, expl_sprite, 0, 9, 30, LAYER_EXPLOSIONS));
                        damage_factor = (int)blast_damage;
                        soundExplosion();
                        Query q;
                        double r;
                        for (q.begin(this, OBJECT_LAYERS, blast_radius); q.currento; q.next())
                                if (q.currento != other_shot) {
                                        r = distance(q.currento) / blast_radius;
                                        q.currento->damage += (int)ceil(blast_damage * (1 - r*r*r));
                                        q.currento->handle_damage(this); } }
                state = 0;
                return; }
}



REGISTER_SHIP(TauHJ)
