#include "../ship.h"
REGISTER_FILE

class TauHunterPortal;

class TauHunter  :  public Ship
{
  double        weaponRange;
  double        weaponVelocity;
  int           weaponLength;
  double        weaponSpread;
  bool          weaponAlternating;
  int           weaponNumber;
  double        weaponRelativity;

  double        specialRange;
  double        specialVelocity;
  int           specialDamage;
  int           specialArmour;

  double        extraVelocity;
  double        extraDrain;
  double        extraExitVelocityFactor;
  int           extraFrameTime;

  double        residual_drain;
  int           weapon_side;
  int           ship_recharge_amount;
  int           exit_countdown;
  double        ex, ey, vvx, vvy;
  double        s_a;

  int           batt_delay, weapon_delay, special_delay, extra_delay;

  public:

  double        extraCriticalRange;
  double        extraCriticalAngle;
  bool          holding_spec;
  bool          in_jump;
  bool          just_exited;

  TauHunter    (double ox, double oy, double shipAngle, ShipData *shipData, unsigned int code);

  virtual  int  activate_weapon();
          void  jump_in(TauHunterPortal *portal);
  virtual void  calculate_fire_special();
  virtual void  calculate();
  virtual void  handle_damage(SpaceLocation* source);
  virtual  int  canCollide(SpaceLocation *other);
  virtual  int  translate(double dx, double dy);
  virtual  int  accelerate(SpaceLocation *source, double angle, double vel, double max_speed);
  virtual void  animate(Frame *space);
  virtual  int  isProtected();
  virtual  int  isInvisible();
  virtual void  calculate_hotspots();
  virtual void  calculate_turn_left();
  virtual void  calculate_turn_right();
  virtual int   battPanelColor();
};


class TauHunterLaser : public SpaceLine
{
  double        range, d, vel;

  public:

  TauHunterLaser   (double ox, double oy, double oangle, double ov, double orange, double olength, SpaceLocation *creator, double relativity);

  virtual void  calculate();
  virtual void  inflict_damage(SpaceObject *other);
};


class TauHunterPortal : public Animation
{
  TauHunter    *ship;
  
  public:
  TauHunterPortal  (SpaceLocation *creator, double ox, double oy, SpaceSprite *osprite, int oft, TauHunter *oship);
  virtual void  calculate();
};


class TauHunterShot : public Shot
{
  int           extraFrameTime;
  SpaceSprite  *expl_sprite, *portal_sprite;
  TauHunter    *ship;
  
  public:
  
  TauHunterShot(double ox, double oy, double oangle, double ov, int odamage, double orange, int oarmour, TauHunter *creator,
                SpaceSprite *osprite, SpaceSprite *esprite, SpaceSprite *psprite, int oft);
  virtual void  calculate();
};




TauHunter::TauHunter (double ox, double oy, double shipAngle, ShipData *shipData, unsigned int code) :
        Ship(ox, oy, shipAngle, shipData, code)
{
        weapon_delay            = scale_frames(get_config_float("Weapon", "RechargeDelay", 0));
        special_delay           = scale_frames(get_config_float("Special", "RechargeDelay", 0));
        extra_delay             = scale_frames(get_config_float("Extra", "RechargeDelay", 0));

        weaponRange             = scale_range(get_config_float("Weapon", "Range", 0));
        weaponVelocity          = scale_velocity(get_config_float("Weapon", "Velocity", 0));
        weaponLength            = get_config_int("Weapon", "Length", 0);
        weaponSpread            = get_config_float("Weapon", "Spread", 0);
        weaponAlternating       = (get_config_int("Weapon", "Alternating", 0) != 0);
        weaponNumber            = get_config_int("Weapon", "Number", 1);
        weaponRelativity        = get_config_float("Weapon", "Relativity", 0.0);

        specialRange            = scale_range(get_config_float("Special", "Range", 0));
        specialVelocity         = scale_velocity(get_config_float("Special", "Velocity", 0));
        specialDamage           = get_config_int("Special", "Damage", 0);
        specialArmour           = get_config_int("Special", "Armour", 0);

        extraVelocity           = scale_velocity(get_config_float("Extra", "Velocity", 0));
        extraDrain              = get_config_float("Extra", "Drain", 0) / 1000.0;
        extraExitVelocityFactor = get_config_float("Extra", "ExitVelocityFactor", 1.0);
        extraCriticalAngle      = get_config_float("Extra", "CriticalAngle", 60);
        extraCriticalRange      = get_config_float("Extra", "CriticalRange", 0);
        extraFrameTime          = get_config_int("Extra", "FrameTime", 50);

        weapon_side             = 1 - 2 * (random()%2);
        in_jump                 = false;
        ship_recharge_amount    = recharge_amount;
        exit_countdown          = 0;
        holding_spec            = false;
        just_exited             = false;
        batt_delay              = 0;
}

int TauHunter::activate_weapon()
{
        double  a, b, c;
        int     i;
        if (in_jump) return false;
        if (!weaponAlternating) {
                for (i = 0; i < weaponNumber; i++) {
                        a = (1 - (random()%201)/100.0);
                        b = (1 - 0.1*(random()%201)/100.0);
                        c = (1 - 0.1*(random()%201)/100.0);
                        game->addItem(new TauHunterLaser(22 * weapon_side, 9, angle + weaponSpread * a,
                                          weaponVelocity * c, weaponRange * (1 - 0.2*fabs(a)) * b,
                                          weaponLength, this, weaponRelativity)); }
                weapon_side *= -1; }

        for (i = 0; i < weaponNumber; i++) {
                a = (1 - (random()%201)/100.0);
                b = (1 - 0.1*(random()%201)/100.0);
                c = (1 - 0.1*(random()%201)/100.0);
                game->addItem(new TauHunterLaser(22 * weapon_side, 9, angle + weaponSpread * a,
                                  weaponVelocity * c, weaponRange * (1 - 0.2*fabs(a)) * b,
                                  weaponLength, this, weaponRelativity)); }
        weapon_side *= -1;
        if (batt_delay < weapon_delay)
                batt_delay = weapon_delay;
        recharge_amount = 0;
        return true;
}

void TauHunter::jump_in(TauHunterPortal *portal)
{
        double a = normalize(atan3(vy, vx) * 180 / PI - angle, 360);
        if (a > 180) a -= 360;
        if (fabs(a) > extraCriticalAngle) return;
        in_jump = true;
        batt = batt_max;
        update_panel = true;
        residual_drain  = 0;
        recharge_amount = 0;
        vvx = extraVelocity * cos(angle * ANGLE_RATIO);
        vvy = extraVelocity * sin(angle * ANGLE_RATIO);
        vx = 0; vy = 0;
}

void TauHunter::calculate_fire_special()
{
        special_low = false;
        if (!fire_special) {
                holding_spec = false;
                if (in_jump && (exit_countdown <= 0)) {
                        exit_countdown = extraFrameTime * 16;
                        ex = normalize(x + vvx * extraFrameTime * 16, X_MAX);
                        ey = normalize(y + vvy * extraFrameTime * 16, Y_MAX);
                        SpaceLocation *s = game->addItem(new Animation(this, ex, ey, data->spriteSpecial, 0, 41, extraFrameTime, LAYER_HOTSPOTS));
                        s->play_sound(data->sampleExtra[0]); }}
        else {
/*                if (in_jump && (exit_countdown <= 0) && !holding_spec) {
                        exit_countdown = extraFrameTime * 16;
                        ex = normalize(x + vvx * extraFrameTime * 16, X_MAX);
                        ey = normalize(y + vvy * extraFrameTime * 16, Y_MAX);
                        SpaceLocation *s = game->addItem(new Animation(this, ex, ey, data->spriteSpecial, 0, 41, extraFrameTime, LAYER_HOTSPOTS));
                        s->play_sound(data->sampleExtra[0]);
                        return; }*/
                if ((special_recharge > 0) || in_jump)
                        return;
                if (batt < special_drain) {
                        special_low = true; return; }
                batt -= special_drain;
                special_recharge += special_rate;
                SpaceLocation *s = game->addItem(new TauHunterShot(0, 16, angle, specialVelocity, specialDamage, specialRange, specialArmour, this,
                                                     data->spriteWeapon, data->spriteWeaponExplosion, data->spriteSpecial, extraFrameTime));
                if (batt_delay < special_delay)
                        batt_delay = special_delay;
                recharge_amount = 0;
                s->play_sound2(data->sampleSpecial[0]);
                holding_spec = true; }
}

void TauHunter::calculate()
{
        just_exited = false;

        if (batt_delay > 0) batt_delay -= frame_time;
        else    if (!in_jump)
                        recharge_amount = ship_recharge_amount;
                        
        Ship::calculate();
        
        if (in_jump) {
                vx = 0; vy = 0;
                if (exit_countdown > 0) {
                        exit_countdown -= frame_time;
                        if (exit_countdown <= 0) {
                                in_jump = false;
                                just_exited = true;
                                x = ex; y = ey;
                                batt = 0;
                                if (batt_delay < extra_delay)
                                        batt_delay = extra_delay;
                                special_recharge = special_rate;
                                double vvv = sqrt(vvx*vvx + vvy*vvy);
                                vx = speed_max * extraExitVelocityFactor * vvx / vvv;
                                vy = speed_max * extraExitVelocityFactor * vvy / vvv; }
                        else {
                                x = normalize(ex - vvx * exit_countdown, X_MAX);
                                y = normalize(ey - vvy * exit_countdown, Y_MAX); } }
                else {
                        vvx = extraVelocity * cos(angle * ANGLE_RATIO);
                        vvy = extraVelocity * sin(angle * ANGLE_RATIO);
                        x = normalize(x + vvx * frame_time, X_MAX);
                        y = normalize(y + vvy * frame_time, Y_MAX);
                        if (batt <= extraDrain*extraFrameTime*16) {
                                exit_countdown = extraFrameTime * 16;
                                ex = normalize(x + vvx * extraFrameTime * 16, X_MAX);
                                ey = normalize(y + vvy * extraFrameTime * 16, Y_MAX);
                                SpaceLocation *s = game->addItem(new Animation(this, ex, ey, data->spriteSpecial, 0, 41, extraFrameTime, LAYER_HOTSPOTS));
                                s->play_sound(data->sampleExtra[0]); } }
                residual_drain += extraDrain * frame_time;
                batt -= (int)floor(residual_drain);
                if (batt < 0) batt = 0;
                residual_drain -= floor(residual_drain); }
}
void TauHunter::handle_damage(SpaceLocation *source) {
        if (in_jump) {
                damage = 0;
                directDamage = 0;
                return; }
        if (just_exited && source->isPlanet())
                damage = crew+1;
        Ship::handle_damage(source);
}

int TauHunter::canCollide(SpaceLocation *other)
{
        if (in_jump)
                return false;
        else    return Ship::canCollide(other);
}

int TauHunter::translate(double dx, double dy)
{
        if (!in_jump) return Ship::translate(dx, dy);
        return false;
}

int TauHunter::accelerate(SpaceLocation *source, double oangle, double vel, double omax_speed)
{
        if (!in_jump) return Ship::accelerate(source, oangle, vel, omax_speed);
        return false;
}

void TauHunter::animate(Frame* space)
{
        if (!in_jump) Ship::animate(space);
}

int TauHunter::isProtected()
{
        if (in_jump)
                return true;
        else    return Ship::isProtected();
}

int TauHunter::isInvisible()
{
        if (in_jump)
                return true;
        else    return Ship::isInvisible();
}

void TauHunter::calculate_hotspots()
{
        if (!in_jump) Ship::calculate_hotspots();
}

void TauHunter::calculate_turn_left()
{
        if (exit_countdown <= 0) Ship::calculate_turn_left();
}

void TauHunter::calculate_turn_right()
{
        if (exit_countdown <= 0) Ship::calculate_turn_right();
}

int TauHunter::battPanelColor()
{
        if (in_jump) return makecol(50,50,170);
        return Ship::battPanelColor();
}


TauHunterLaser::TauHunterLaser (double ox, double oy, double oangle, double ov, double orange, double olength, SpaceLocation *creator, double relativity) :
        SpaceLine(creator, creator->normal_x(), creator->normal_y(), oangle, olength, makecol(255,255,255)), range(orange), d(0), vel(ov)
{
        double alpha = creator->get_angle() * ANGLE_RATIO;
        double tx = cos(alpha), ty = sin(alpha);
        x += oy * tx - ox * ty;
        y += oy * ty + ox * tx;
        damage_factor = 1;
        vx = ov * cos (angle * ANGLE_RATIO) + creator->get_vx() * relativity;
        vy = ov * sin (angle * ANGLE_RATIO) + creator->get_vy() * relativity;
}

void TauHunterLaser::calculate()
{
        double r = (d) / range;
        double r2 = r*r;
        double r3 = r2*r;
        int g = (int)floor(225*(1 - 2*r2 + r3));
        if (g < 0) g = 0;
        int b = (int)floor(255*(1 - 6*r2+5*r3));
        if (b < 0) b = 0;
        color = makecol(int(235/(0.7*r+1)), g, b);
        SpaceLine::calculate();
        d += vel * frame_time;
        if (d > range) state = 0;
}

void TauHunterLaser::inflict_damage(SpaceObject *other)
{
        SpaceLine::inflict_damage(other);
        state = 0;
}


TauHunterPortal::TauHunterPortal(SpaceLocation *creator, double ox, double oy, SpaceSprite *osprite, int oft, TauHunter *oship) :
        Animation(creator, ox, oy, osprite, 0, 41, oft, LAYER_HOTSPOTS), ship(oship)
{
}

void TauHunterPortal::calculate()
{
        Animation::calculate();

        if (ship) {
                if ((!ship->exists()) || (!ship->holding_spec) || ship->in_jump) {
                        ship = NULL; return; }
                if ((sprite_index >= 10) && (sprite_index <= 25) && (distance(ship) < ship->extraCriticalRange))
                        ship->jump_in(this); }
}


TauHunterShot::TauHunterShot (double ox, double oy, double oangle, double ov, int odamage, double orange, int oarmour, TauHunter *creator,
                   SpaceSprite *osprite, SpaceSprite *esprite, SpaceSprite *psprite, int oft) :
        Shot(creator, ox, oy, oangle, ov, odamage, orange, /*oarmour*/1999999999, creator, osprite, 1.0)
{
        expl_sprite    = esprite;
        portal_sprite  = psprite;
        ship           = creator;
        extraFrameTime = oft;
        collide_flag_anyone = 0;
        collide_flag_sameteam = 0;
        collide_flag_sameship = 0;
}

void TauHunterShot::calculate()
{
        if (state == 0) return;
        sprite_index = (int)floor(20 * d*d / (range*range));
        if (ship) if ((!ship->exists()) || (!ship->holding_spec) || ship->in_jump)
                ship = NULL;
        Shot::calculate();
        if (d >= range)
                game->addItem(new TauHunterPortal(this,x, y, portal_sprite, extraFrameTime, ship));
}



REGISTER_SHIP(TauHunter)
