#include "../ship.h"
REGISTER_FILE
//#include "mview.h"

#define LEVIATHAN_SLIME_FOOD 0x12001457
#define LEVIATHAN_JAMMER 0x2050

class TauLeviathan : public Ship {
  
  double        weaponRange, weaponVelocity, weaponDamage, weaponArmour;
  double        weaponSpread;
  
  double        specialRange, specialVelocity, specialDamage, specialArmour;
  double        specialTurnRate;
  
  int           extraTime;
  double        extraSpeed;

  int           food2batt;
  int           healing_amount;
  int           healing_rate;
  int           healing_step;
  int           missile_side;

  public:

  TauLeviathan(double ox, double oy, double shipAngle, ShipData *shipData, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();

  virtual int crewPanelColor();
//  virtual int battPanelColor();

  virtual void calculate();
  virtual void handle_damage(SpaceLocation *source);
  virtual void calculate_hotspots();
};


class LeviathanSlimeBall : public AnimatedShot {
  protected:
  int l_time;
  double food_velocity;

  public:

  LeviathanSlimeBall(double ox, double oy, double oangle, double ov, int odamage, int otime, double ov2,
                 double orange, int oarmour, SpaceLocation *creator, SpaceLocation *opos,
                 SpaceSprite *oscprite, int ofcount, int ofsize);
  virtual void inflict_damage(SpaceObject *other);
};


class LeviathanSlimeFood : public AnimatedShot {
  protected:
  int lifetime;
  public:
  LeviathanSlimeFood(double oangle, double ov, int otime,
                 SpaceLocation *creator, SpaceLocation *opos,
                 SpaceSprite *osprite, int ofcount, int ofsize);
  virtual void calculate();
  virtual int isAlly(SpaceLocation *other);
  virtual void inflict_damage(SpaceObject *other);
};


class LeviathanJammer : public FixedAnimation
{
  public:
  
  Ship          *victim;
  
  LeviathanJammer (SpaceLocation *creator, SpaceLocation *opos, SpaceSprite *osprite);
  virtual void calculate();
};

class LeviathanMissile : public HomingMissile
{
  SpaceSprite *expl_sprite;
  SAMPLE *extraSample;

  public:
  
  LeviathanMissile(double ox, double oy, double oangle, double ov, int odamage,
                   double orange, int oarmour, double otrate, SpaceLocation *creator,
                   SpaceSprite *osprite, SpaceSprite *esprite, SpaceObject *otarget);
  virtual void inflict_damage(SpaceObject *other);
};



LeviathanMissile::LeviathanMissile(double ox, double oy, double oangle, double ov, int odamage,
                   double orange, int oarmour, double otrate, SpaceLocation *creator,
                   SpaceSprite *osprite, SpaceSprite *esprite, SpaceObject *otarget) :
        HomingMissile(creator, ox, oy, oangle, ov, odamage, orange, oarmour, otrate, creator, osprite, otarget)
{
        expl_sprite = esprite;
        extraSample = data->sampleSpecial[1];
}

void LeviathanMissile::inflict_damage(SpaceObject *other)
{
        other->damage += damage_factor;
        other->handle_damage(this);
        if (!other->isShot()) {
                state = 0;
                if (other->isShip()) {
                        game->addItem(new LeviathanJammer(this, other, expl_sprite));
                        play_sound(extraSample); }
                else {
                        animateExplosion();
                        soundExplosion(); } }
}


TauLeviathan::TauLeviathan(double ox, double oy, double shipAngle,
                ShipData *shipData, unsigned int code) :
        Ship(ox, oy, shipAngle, shipData, code)
{
        healing_amount  = get_config_int("Ship", "HealingAmount", 0);
        healing_rate    = scale_frames(get_config_int("Ship", "HealingRate", 0));
        healing_step    = healing_rate;
        food2batt       = get_config_int("Ship", "Food2Batt", 0);
        weaponRange     = scale_range(get_config_float("Weapon", "Range", 0));
        weaponVelocity  = scale_velocity(get_config_float("Weapon", "Velocity", 0));
        weaponSpread    = get_config_float("Weapon", "Spread", 0);
        weaponDamage    = get_config_int("Weapon", "Damage", 0);
        weaponArmour    = get_config_int("Weapon", "Armour", 0);

        specialRange    = scale_range(get_config_float("Special", "Range", 0));
        specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
        specialTurnRate = scale_turning(get_config_float("Special", "TurnRate", 0));
        specialDamage   = get_config_int("Special", "Damage", 0);

        extraTime       = (int)(get_config_float("Extra", "Lifetime", 1)*1000);
        extraSpeed      = scale_velocity(get_config_float("Extra", "Velocity", 0));

        missile_side    = 1 - 2 * (random()%2);
}

int TauLeviathan::activate_weapon()
{
        game->addItem(new LeviathanSlimeBall(0.0, h/2.6, normalize(angle + weaponSpread*(1.0-(random() % 2001)/1000.0), 360),
                                  weaponVelocity, weaponDamage, extraTime, extraSpeed, weaponRange,
                                  weaponArmour, this, this, data->spriteWeapon, 10, 70));
        return true;
}

int TauLeviathan::activate_special()
{
        game->addItem(new LeviathanMissile(25*missile_side, 10, angle + 15*missile_side,
                          specialVelocity, specialDamage, specialRange, specialArmour,
                          specialTurnRate, this, data->spriteSpecial, data->spriteSpecialExplosion,
                          target));
        missile_side *= -1;
        return true;
}

void TauLeviathan::calculate()
{
        Ship::calculate();

        if (healing_step < 0) {
                healing_step += healing_rate;
                crew += healing_amount;
                if (crew > crew_max) crew = crew_max; }
        else  healing_step -= frame_time;

        Query q;
	for (q.begin(this, bit(LAYER_SHOTS), h*4.0); q.currento; q.next())
                if (q.currento->getID() == LEVIATHAN_SLIME_FOOD) {
                        double r = distance(q.currento);
                        if (r>h*2) r = scale_velocity(1)*frame_time*(h*4-r)/(h*2)/30.0;
                        else       r = scale_velocity(1)*frame_time/30.0;
                        q.currento->accelerate(this, normalize(180+trajectory_angle(q.currento),360), r ,GLOBAL_MAXSPEED); }
}

void TauLeviathan::handle_damage(SpaceLocation *source) {
        if (state == 0) return;
        if (source->getID() == LEVIATHAN_SLIME_FOOD) {
                repair++; source->state = 0;
                play_sound(data->sampleExtra[0]); }
        if (repair > 0) {
                batt += food2batt * repair;
                if (batt > batt_max) batt = batt_max; }
//        fuel_sap = 0;
        Ship::handle_damage(source);
}

void TauLeviathan::calculate_hotspots()
{
        if (!fire_special) Ship::calculate_hotspots();
}

int TauLeviathan::crewPanelColor(){
        return makecol(159,79,0);
}

/*int TauLeviathan::battPanelColor(){
        return makecol(129,50,99);//palette_color[15];
}*/

LeviathanSlimeBall::LeviathanSlimeBall(double ox, double oy, double oangle, double ov, int odamage,
                                       int otime, double ov2, double orange, int oarmour, SpaceLocation *creator, SpaceLocation *opos,
                                       SpaceSprite *osprite, int ofcount, int ofsize) :
        AnimatedShot(creator, ox, oy, oangle , ov, odamage, orange, oarmour, opos,
        osprite, ofcount, ofsize)
{
        explosionSprite     = data->spriteWeaponExplosion;
        explosionFrameCount = 16;
        explosionFrameSize  = 50;
        l_time = otime;
        food_velocity = ov2;
}

void LeviathanSlimeBall::inflict_damage(SpaceObject *other) {

        if (other->isShip()) {
                int c = ((Ship*)other)->getCrew();
                AnimatedShot::inflict_damage(other);
                c -= ((Ship*)other)->getCrew();
                if (((Ship*)other)->crewPanelColor() == palette_color[2])
                        for (int i=0; i<c; i++)
                                game->addItem(new LeviathanSlimeFood(random()%361, food_velocity*(1-0.90*(random()%101)/100.0),
                                                  (int)(l_time * (1-0.00*(random()%101)/100.0)), this, this, data->spriteExtra, 10, 70));
        }
        else
        AnimatedShot::inflict_damage(other);
};

LeviathanSlimeFood::LeviathanSlimeFood(double oangle, double ov, int otime,
                                       SpaceLocation *creator, SpaceLocation *opos,
                                       SpaceSprite *osprite, int ofcount, int ofsize) :
        AnimatedShot(creator, 0, 0, oangle , ov, 0, 99999999, 1, creator, osprite, ofcount, ofsize)
{
        id = LEVIATHAN_SLIME_FOOD;
        layer = LAYER_SHOTS;
        collide_flag_anyone = ALL_LAYERS;
        collide_flag_sameship = bit(LAYER_SHIPS);
        collide_flag_sameteam = bit(LAYER_SHIPS);
        lifetime = otime;
//        mass = 0.001;
};

void LeviathanSlimeFood::inflict_damage(SpaceObject *other)
{
        other->handle_damage(this);
};

void LeviathanSlimeFood::calculate()
{
        AnimatedShot::calculate();
        if (1 - 0.0018*frame_time > 0) {
                vx *= (1 - 0.0018*frame_time);
                vy *= (1 - 0.0018*frame_time); }
        else {  vx = 0; vy = 0; }
        d = 0;
        lifetime -= frame_time;
        if (lifetime <= 0) state = 0;
}


int LeviathanSlimeFood::isAlly(SpaceLocation *other) {
        return true;
}

LeviathanJammer::LeviathanJammer (SpaceLocation *creator, SpaceLocation *opos, SpaceSprite *osprite) :
        FixedAnimation(creator, opos, osprite, 0, 20, 50, LAYER_EXPLOSIONS)
{
        id = LEVIATHAN_JAMMER;
        if (opos->isShip()) {
                victim = (Ship*)opos;
                Query q;
                for (q.begin(victim, bit(LAYER_EXPLOSIONS), 1); q.current; q.next())
                        if (q.currento->getID() == LEVIATHAN_JAMMER)
                        if (((LeviathanJammer*)q.current)->victim == victim)
                                ((LeviathanJammer*)q.current)->victim = NULL; }
        else    victim = NULL;
};

void LeviathanJammer::calculate()
{
        FixedAnimation::calculate();
        if (!exists()) return;
        if (victim) {
                victim->nextkeys &= ~(keyflag::left + keyflag::right + keyflag::thrust);
                double vvx = victim->get_vx();
                double vvy = victim->get_vy();
                double vvv = sqrt(vvx*vvx + vvy*vvy);
                if (vvv < 1e-5) return;
                vvv = (vvv - victim->accel_rate * frame_time) / vvv;
                if (vvv < 0) vvv = 0;
                vvx *= (vvv - 1); vvy *= (vvv - 1);
                vvv = sqrt(vvx*vvx + vvy*vvy);
                victim->accelerate(victim, atan3(vvy,vvx)*180/PI, vvv, GLOBAL_MAXSPEED); }
}



REGISTER_SHIP(TauLeviathan)
