#include "../ship.h"
REGISTER_FILE
//#include "../melee/mview.h"

class XXXAssimilator : public Ship {

	int          weaponFrames;
  int          weaponDamage;

	int          orig_Damage;
	int					 orig_WeaponDrain;
	double			 orig_Speed;
	double       orig_Acceleration;
	int					 orig_HotspotRate;
	

	double			 weaponRange;
	double			 weaponVelocity;
	int					 weaponArmour;

  bool         AssimilateReset;
	double			 TheSpotAngle;
	int					 TheSpotDelay;
	int					 TheSpotDelayTimer;
	int					 TheSpotRadius;
	int					 RelX;
	int					 RelY;

  int          BodyFrames;
  int          CurrBodyFrame;
  int          ToggleDirection;

  public:
  XXXAssimilator(double ox, double oy, double shipAngle,
		ShipData *shipData, unsigned int code);

  int crewPanelColor();

  virtual int activate_weapon();
  virtual int activate_special();
	virtual void calculate();
	virtual void collide(SpaceObject *other);
  virtual void animate(Frame *space);
	virtual void handle_damage(SpaceLocation *source);
	virtual void calculate_hotspots();
  
};

XXXAssimilator::XXXAssimilator(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code) 
	:
  Ship(ox, oy, shipAngle, shipData, code)

{
  weaponFrames    = get_config_int("Weapon", "Frames", 0);
  weaponRange			= scale_range(get_config_float("Weapon", "Range", 0));
  weaponDamage		= get_config_int("Weapon", "Damage", 0);
	weaponVelocity	= scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponArmour		= get_config_int("Weapon", "Armour", 0);

	orig_Damage				= weaponDamage;
	orig_WeaponDrain	= weapon_drain; 
	orig_Speed        = speed_max;
	orig_Acceleration = accel_rate;
	orig_HotspotRate	= hotspot_rate;

	TheSpotDelay 			 = get_config_int("Special", "TheSpotDelay", 0);
	TheSpotRadius			 = get_config_int("Special", "TheSpotRadius", 0);
	TheSpotAngle			 = 0;
	TheSpotDelayTimer  = 0;

  BodyFrames				 = weaponFrames;
  CurrBodyFrame			 = 0;
  ToggleDirection		 = FALSE;

}

int XXXAssimilator::crewPanelColor()
{
  return pallete_color[8];
}

int XXXAssimilator::activate_weapon()
{ 

	double ShotAngle = normalize(TheSpotAngle + get_angle(),360);

	game->addItem(new AnimatedShot(this, RelX, RelY,
		ShotAngle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
		this, data->spriteWeapon,4,4,0));

	return(TRUE);
}

int XXXAssimilator::activate_special()
{
	weaponDamage = orig_Damage;
	weapon_drain = orig_WeaponDrain;
	speed_max    = orig_Speed;
	accel_rate   = orig_Acceleration;
	hotspot_rate = orig_HotspotRate;

	return(TRUE);
}

void XXXAssimilator::calculate()
{
	Ship::calculate();

	TheSpotDelayTimer++;


	if (TheSpotDelayTimer > TheSpotDelay)
		{
			TheSpotDelayTimer   = 0;
			TheSpotAngle				+= 5.625;
			TheSpotAngle        = normalize(TheSpotAngle,360);
		}

	

	RelX = int(cos((-TheSpotAngle+90) * ANGLE_RATIO) * TheSpotRadius);
	RelY = int(sin((-TheSpotAngle+90) * ANGLE_RATIO) * TheSpotRadius);

	game->addItem(new PositionedAnimation(this, this, RelX, RelY,
										data->spriteExtra, 0, 1, 50, LAYER_EXPLOSIONS));
}

void XXXAssimilator::collide(SpaceObject *other)
{

	Ship::collide(other);

	if (other->isShip())
		{
		if (other->ship->speed_max > speed_max)
			{
			 speed_max = other->ship->speed_max;
		   play_sound(data->sampleExtra[0]);

			}

		if (other->ship->accel_rate  > accel_rate) 
			{
			 accel_rate		= other->ship->accel_rate;
			 hotspot_rate = other->ship->hotspot_rate;
	     play_sound(data->sampleExtra[0]);

			}
		}

}

void XXXAssimilator::animate(Frame *space)
{
  BodyFrames -= frame_time * 5;

  if (BodyFrames <= 0)
  {
          if (!( (CurrBodyFrame > 0) && (CurrBodyFrame < 2) ))
             ToggleDirection = !ToggleDirection;
        
          if (ToggleDirection) ++CurrBodyFrame;
          else --CurrBodyFrame;

          BodyFrames = weaponFrames;
  }
  sprite->animate(x, y, sprite_index  + (64 * CurrBodyFrame) , space);

}

void XXXAssimilator::handle_damage(SpaceLocation *source)
{
	if (source->damage_factor > weaponDamage)
		{
			weaponDamage = source->damage_factor;
			weapon_drain = weaponDamage;
	    play_sound(data->sampleExtra[0]);
		}

	Ship::handle_damage(source);
}

void XXXAssimilator::calculate_hotspots()
{
	if((thrust) && (hotspot_frame <= 0)) {
		game->addItem(new Animation(this,
				normal_x() - (cos(angle * ANGLE_RATIO) * 11),
				normal_y() - (sin(angle * ANGLE_RATIO) * 11),
				game->hotspotSprite, 0, HOTSPOT_FRAMES, time_ratio, LAYER_HOTSPOTS));

    game->addItem(new PositionedAnimation(this, this, 0, -10, 
			data->spriteExtraExplosion ,0, rand()%2, 100, LAYER_SHOTS));
		hotspot_frame += hotspot_rate;
		}
	if (hotspot_frame > 0) hotspot_frame -= frame_time;

}


REGISTER_SHIP(XXXAssimilator)
