#include "../ship.h"
#include "../melee/mview.h"
REGISTER_FILE

class YevMech : public Ship {
	public:

	int          SaberOn;
	int          ShieldOn;
	double       ShieldAngle;
	double       DegreesMoved;
	double       DegreeTreshhold;
	bool         SaberDestroyed;
	bool         ShieldDestroyed;


	double       SaberAngle;

	int          SaberSlash;
	int          SaberSlashCurrentFrame;
	int          SaberSlashTotalFrames;

	int          FramePassedR;
	int          FramePassedL;

	int          DTLowerLimit;
	int          DTHigherLimit;

	int          SaberAttack;


  YevMech(double ox, double oy, double shipAngle,
			ShipData *shipData, unsigned int code) ;
	int crewPanelColor();


	virtual void calculate_turn_left();
	virtual void calculate_turn_right();

	virtual int activate_weapon();
	virtual int activate_special();

	virtual void calculate();
	};

class YevShield : public SpaceObject {
  YevMech  *ship;

  public:
  YevShield(YevMech *oship, SpaceSprite *osprite);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void animate(Frame* space);
  virtual int canCollide(SpaceLocation* other);
  virtual void handle_damage(SpaceLocation *source);

};

int YevMech::crewPanelColor()
{
  return pallete_color[8];
}

class YevSaber : public SpaceObject {
  YevMech  *ship;
  int frame;
  int frame_size;
  int frame_count;
  int frame_step;

  public:
  YevSaber(YevMech *oship, SpaceSprite *osprite);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void animate(Frame* space);
  virtual int canCollide(SpaceLocation* other);
  virtual void handle_damage(SpaceLocation *source);

};


YevMech::YevMech(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code) 
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  DegreeTreshhold =  get_config_float("Shield", "DegreeTreshhold", 0);
  DTLowerLimit  = get_config_int("Extra", "DTLowerLimit", 0);
  DTHigherLimit = get_config_int("Extra", "DTHigherLimit", 0);


  SaberOn       = false;
  ShieldOn      = false;

  ShieldAngle   = 0;
  SaberAngle       = 45;

  SaberSlash             = false;
  SaberSlashCurrentFrame = 0;
  SaberSlashTotalFrames  = 300;

  SaberDestroyed = false;
  ShieldDestroyed = false;


  SaberAttack   = 0;  //0= default - sword spin; 1 = forward thrust; 2 = right slash; 3 = left slash

  FramePassedR = 0;
  FramePassedL = 0;

  game->addItem(new YevShield(this, data->spriteSpecial));
  game->addItem(new YevSaber(this,  data->spriteWeapon));

}

void YevMech::calculate_turn_left()
{
	if (!SaberDestroyed) {
		FramePassedL += frame_time;
		if (SaberOn ) {
		   if (FramePassedL >= DTLowerLimit && FramePassedL <= DTHigherLimit && turn_left) {
			 if (ship->batt > ship->special_drain) {

			   vx = ship->get_vx() + .3 * cos((angle-90)*PI/180);
			   vy = ship->get_vy() + .3 * sin((angle-90)*PI/180);

			   ship->batt -= ship->special_drain;

			   FramePassedL = 0;} 
		   } else if (FramePassedL > DTHigherLimit) FramePassedL = 0;
		}

	   if (turn_left && FramePassedL != 0) {
		 if (!fire_special || !ShieldOn) 
		  Ship::calculate_turn_left();
		 FramePassedL = 0;
		 FramePassedR = 0;
	   }
	} if (!fire_special || !ShieldOn) 
		  Ship::calculate_turn_left();


}


void YevMech::calculate_turn_right()
{

	if (!SaberDestroyed) {
		FramePassedR += frame_time;
		if (SaberOn ) {
		   if (FramePassedR >= DTLowerLimit && FramePassedR <= DTHigherLimit && turn_right) {
			 if (ship->batt > ship->special_drain) {
  
				vx = ship->get_vx() + .3 * cos((angle+90)*PI/180);
				vy = ship->get_vy() + .3 * sin((angle+90)*PI/180);

			   ship->batt -= ship->special_drain;

			   FramePassedR = 0;}
		   } else if (FramePassedR > DTHigherLimit) FramePassedR = 0;
		}

	   if (turn_right && FramePassedR != 0) {
		 if (!fire_special || !ShieldOn) 
		  Ship::calculate_turn_right();
		 FramePassedL = 0;
		 FramePassedR = 0;
	   }
	} if (!fire_special || !ShieldOn) 
		  Ship::calculate_turn_right();

}


int YevMech::activate_weapon() {
	if (!SaberDestroyed) {
	  if (SaberOn) {
		SaberSlash = TRUE;
	  } else {
		ShieldOn = FALSE;
		SaberOn  = TRUE;
	  }
	  return(TRUE);
	} return(FALSE);
 
}

int YevMech::activate_special()
{
	if (!ShieldDestroyed) {
	  if (!ShieldOn)  {
		ShieldAngle = 315; // Start at left side @ 45 degree angle
		DegreesMoved = 0;
		ShieldOn = TRUE;
		SaberOn  = FALSE;
		return(TRUE);
	  }
	  else return(FALSE);
	} return(FALSE);
}

void YevMech::calculate() {

  if (fire_special && ShieldOn ) {
    DegreesMoved += 5.625;

	if (DegreesMoved >= DegreeTreshhold) {
		ship->batt -= 1;
		DegreesMoved = 0;
	}
	if (ship->batt > 0) 
		if (turn_left) ShieldAngle -= 5.625;
		  else if (turn_right) ShieldAngle += 5.625;

  } else

  if (fire_special && SaberOn ) {
	  if (turn_left) {
		  //left slash
		 SaberAttack   = 3; 
	  }
	  else if (turn_right) {
		  //right slash
		SaberAttack   = 2;
	  }
	  else if (thrust) {
		  //forward thrust
		SaberAttack   = 1;
	  }
	  else {
		  //saber spin
		SaberAttack   = 0;
	  }

  }
  

  Ship::calculate();
}


YevShield::YevShield(YevMech *oship,SpaceSprite *osprite) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), 0.0, osprite),
  ship(oship)
{
layer = LAYER_SHIPS;
//layer = LAYER_EXPLOSIONS;
id    = SPACE_SHOT;
calculate();

}

void YevShield::calculate() {

	if (ship==NULL) {
		state = 0;
		return;
	}

	if(!ship->exists()) {
		state = 0;
		return;
	}


  x = ship->normal_x() + (cos((ship->get_angle() + ship->ShieldAngle) * ANGLE_RATIO) * width()  * .45);
  y = ship->normal_y() + (sin((ship->get_angle() + ship->ShieldAngle) * ANGLE_RATIO) * height() * .45);

  sprite_index = ((int)( (ship->get_angle() + ship->ShieldAngle) / 5.625) + 16) & 63;
}

int YevShield::canCollide(SpaceLocation* other) {
  if (ship->ShieldOn) return(TRUE);
   else return(FALSE);
}
void YevShield::inflict_damage(SpaceObject *other) {
  if (ship->ShieldOn) {
	  if (!(other->isShip() || other->isAsteroid()) )  {
      other->damage += 9999;
	  }

    SpaceObject::inflict_damage(other);

  }
}

void YevShield::animate(Frame* space) {
  if (ship->ShieldOn) SpaceObject::animate(space);
}

void YevShield::handle_damage(SpaceLocation *source) {
	source->damage=0;
	source->directDamage=0;
	this->damage=0;
	this->directDamage=0;
	if (this->state==0) ship->ShieldDestroyed=true;
}


YevSaber::YevSaber(YevMech *oship,SpaceSprite *osprite) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), 0.0, osprite),
  ship(oship)
{

  calculate();
}

void YevSaber::calculate() {
  int		   si;

  if (ship==NULL) {
	  state = 0; 
	  return;
  }

  if(!ship->exists()) {
	  state = 0;
	  return;
  }

  x = ship->normal_x() + cos( ship->get_angle() * ANGLE_RATIO) * width()  * .40;
  y = ship->normal_y() + sin( ship->get_angle() * ANGLE_RATIO) * height() * .40;

  if (ship->SaberSlash) ship->SaberSlashCurrentFrame += frame_time;

  switch (ship->SaberAttack) {
	  case 0:
		  if (ship->SaberSlashCurrentFrame == 0)  si = 0;
		   else {
			if (ship->SaberSlashCurrentFrame <= ship->SaberSlashTotalFrames * .75) si = 1; 
			 else
			if (ship->SaberSlashCurrentFrame <= ship->SaberSlashTotalFrames * .81) si = 2; 
			 else
			if (ship->SaberSlashCurrentFrame <= ship->SaberSlashTotalFrames * .87) si = 3; 
			 else
			if (ship->SaberSlashCurrentFrame <= ship->SaberSlashTotalFrames * .93) si = 4; 
			 else {
			  si = 3;
			  ship->SaberSlashCurrentFrame = 0;
			  ship->SaberSlash = FALSE;
			 }
			 break;
	  default:
		     si = 0;

			 break;
	   }

  }
   sprite_index = (((int)( ship->get_angle() / 5.625) + 16)  & 63) + (64 * si) ;

}

int YevSaber::canCollide(SpaceLocation* other) {
  if (ship->SaberOn && other != ship) return(TRUE);
   else return(FALSE);
}

void YevSaber::inflict_damage(SpaceObject *other) {
	if (ship != NULL) 
		if (ship->SaberSlash && ship->SaberOn ) {
			if (ship->batt>0) {
			other->damage += ship->batt;
			ship->batt = 0;
			} else other->damage = 1;
			sound.play(ship->data->sampleExtra[0]);
			SpaceObject::inflict_damage(other);
		}
}

void YevSaber::animate(Frame* space) {
  if (!ship->SaberOn) return;
  SpaceObject::animate(space);
}

void YevSaber::handle_damage(SpaceLocation *source) {
	if (this->state==0) ship->SaberDestroyed=true;
	
}


REGISTER_SHIP(YevMech)
