#include "../ship.h"
REGISTER_FILE

class AndrosynthGuardian : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  SpaceSprite *specialSprite;
  SpaceSprite *shipSprite;
  double       specialVelocity;
  double       specialTurnRate;
  int          specialDamage;
  int          specialBounce;
  int          specialActive;
  double       shipTurnRate;
  int          shipRechargeAmount;
	double specialMass;
	double normalMass;

  public:
  AndrosynthGuardian(double ox, double oy, double angle, ShipData *data, unsigned int code);

  virtual void calculate();
  virtual void calculate_thrust();
  virtual void calculate_hotspots();

  virtual void inflict_damage(SpaceObject *other);

  virtual int activate_weapon();
  virtual int activate_special();
};

class AndrosynthBubble : public AnimatedShot {
  int courseFrames;

  public:
  AndrosynthBubble(double ox, double oy, double oangle, double ov,
    int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  void calculate();
};

AndrosynthGuardian::AndrosynthGuardian(double ox, double oy, double angle, ShipData *data, unsigned int code) :
  Ship(ox, oy, angle, data, code)
{
  weaponDamage       = get_config_int("Weapon", "Damage", 1);
  weaponArmour       = get_config_int("Weapon", "Armour", 1);
  weaponRange        = scale_range(get_config_float("Weapon", "Range", 10));
  weaponVelocity     = scale_velocity(get_config_float("Weapon", "Velocity", 100));

  specialSprite   = data->spriteSpecial;
  shipSprite      = sprite;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialBounce   = get_config_int("Special", "Bounce", 0);
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialTurnRate = scale_turning(get_config_float("Special", "TurnRate", 0));
  specialActive   = FALSE;

	normalMass = get_config_float("Ship", "Mass", 1);
	specialMass = get_config_float("Special", "Mass", 1);

  shipTurnRate    = turn_rate;
	shipRechargeAmount = recharge_amount;
}

void AndrosynthGuardian::calculate()
{
	Ship::calculate();
	if(specialActive && (batt == -1)) {
		sprite = shipSprite;
		w      = shipSprite->width();
		h      = shipSprite->height();
		damage_factor = 0;
		specialActive = FALSE;
		vx = 0.0;
		vy = 0.0;
		turn_rate = shipTurnRate;
		recharge_amount = shipRechargeAmount;
		batt = 0;
		mass = normalMass;
		}
	return;
	}

void AndrosynthGuardian::calculate_thrust() {
	if(specialActive) {
		double dv = specialVelocity / 2 * frame_time / 150.;
		accelerate(this, angle, dv, specialVelocity);
		accelerate(this, angle, dv, specialVelocity);
		}
	else Ship::calculate_thrust();
	}

void AndrosynthGuardian::calculate_hotspots()
{
  if(specialActive)
    return;
  Ship::calculate_hotspots();
}

void AndrosynthGuardian::inflict_damage(SpaceObject *other) {
	if (damage_factor > 0) {
		if (other->mass) {
			game->addItem(new Animation(this, 
					x + (cos(trajectory_angle(other) * ANGLE_RATIO) * 20.0),
					y + (sin(trajectory_angle(other) * ANGLE_RATIO) * 20.0),
					game->sparkSprite, 0, SPARK_FRAMES, 50, LAYER_EXPLOSIONS));
			translate(-specialBounce*cos(angle*ANGLE_RATIO), -specialBounce*sin(angle*ANGLE_RATIO));
			int i = damage_factor / 2;
			if (i >= BOOM_SAMPLES) i = BOOM_SAMPLES - 1;
			play_sound((SAMPLE *)(melee[MELEE_BOOM + i].dat));
			}
		if (!other->isProtected()) other->damage += damage_factor;
		}
	other->handle_damage(this);
	return;
	}

int AndrosynthGuardian::activate_weapon() {
  
  if(specialActive)
	  return FALSE;

  game->addItem(new AndrosynthBubble(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon, 10, 50));
  return(TRUE);
}

int AndrosynthGuardian::activate_special() {
  if(specialActive)
    return(FALSE);

  if (batt < 1) return(FALSE);

  sprite = specialSprite;
  w      = specialSprite->width();
  h      = specialSprite->height();

  damage_factor = specialDamage;
  specialActive = TRUE;
  turn_rate     = specialTurnRate;
  
  recharge_step = recharge_rate;
	recharge_amount = -1;
	mass = specialMass;

  return(TRUE);
}

AndrosynthBubble::AndrosynthBubble(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(oship, ox, oy, oangle, ov, odamage, orange, oarmour, oship, 
    osprite, ofcount, ofsize),
  courseFrames(0)
{
  explosionSprite     = data->spriteWeaponExplosion;
  explosionFrameCount = 6;
  explosionFrameSize  = 50;
}

void AndrosynthBubble::calculate()
{
  double newAngle;

  AnimatedShot::calculate();
  courseFrames += frame_time;
  if(courseFrames >= 150) {
    courseFrames -= 150;
    
    newAngle = (double)(random() % 360);
    vx = v * cos(newAngle * ANGLE_RATIO);
    vy = v * sin(newAngle * ANGLE_RATIO);

    int i;
		double r = 99999;
		newAngle = 99999;
		if (ship) {
			for (i = 0; i < game->num_targets; i += 1) {
				SpaceObject *s = game->target[i];
				if (ship->control->valid_target(s) && (distance(s) < r) && !s->isInvisible()) {
					r = distance(s);
					newAngle = trajectory_angle(s);
					}
				}
			}
		else {
			for (i = 0; i < game->num_targets; i += 1) {
				SpaceObject *s = game->target[i];
				if (s->sameTeam(this) && (distance(s) < r) && !s->isInvisible()) {
					r = distance(s);
					newAngle = trajectory_angle(s);
					}
				}
			}
    if(newAngle < 720) {
      vx += v * cos(newAngle * ANGLE_RATIO) / 2;
      vy += v * sin(newAngle * ANGLE_RATIO) / 2;
      }      
		}
	return;
	}


REGISTER_SHIP(AndrosynthGuardian)

