#include "../ship.h"
REGISTER_FILE

class ArilouSkiff : public Ship {
  int    weaponColor;
  double weaponRange;
  int    weaponFrames;
  int    weaponDamage;

	int just_teleported;

  SpaceSprite *specialSprite;
  double       specialFrames;

  public:
  ArilouSkiff(double ox, double oy, double angle, ShipData *data, unsigned int code);

  protected:
	virtual void inflict_damage(SpaceObject *other);
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void calculate_gravity();
  virtual int accelerate(SpaceLocation *source, double angle, double velocity, 
			double max_speed);
  virtual int accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, 
			double max_speed);
	};

class ArilouLaser : public SpaceLine {
	int frame;
	int frame_count;

	SpaceLocation *pos;
	double d;

	public:
	ArilouLaser(double langle, int lcolor, double lrange, int ldamage,
		int lfcount, SpaceLocation *opos, double oposdist);

	void calculate();
	};

ArilouSkiff::ArilouSkiff(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
	{
	specialSprite = data->spriteSpecial;

	weaponColor  = get_config_int("Weapon", "Color", 0);
	weaponDamage = get_config_int("Weapon", "Damage", 0);
	weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
	weaponFrames = get_config_int("Weapon", "Frames", 0);

	specialFrames = get_config_float("Special", "Frames", 0);

	just_teleported = 0;
	}

void ArilouSkiff::inflict_damage(SpaceObject *other) {
	if (just_teleported && other->mass) {
		this->directDamage = 999;
		this->handle_damage(other);
		}
	Ship::inflict_damage(other);
	return;
	}

int ArilouSkiff::activate_weapon() {
  SpaceObject *o = NULL;

  double r = 99999;  

	Query a;
	for (a.begin(this, bit(LAYER_SHIPS), weaponRange + 200); a.current; a.next()) {
		if ((distance(a.current) < r) && !a.current->isInvisible()) {
			o = a.currento;
			r = distance(o);
			}
		}

  if (o) r = trajectory_angle(o); else r = angle;
// ArilouLaser redundant ??
//  spaceFrame->addSpaceLine(new ArilouLaser(r, pallete_color[weaponColor], 
//    weaponRange, weaponDamage, weaponFrames, this, w / 2.0));
  game->addItem(new Laser(this, r, pallete_color[weaponColor], 
    weaponRange, weaponDamage, weaponFrames, this, 0, 0));
  return TRUE;
	}

int ArilouSkiff::activate_special() {
	double dx, dy;
	dx = (double)(random() % (int)(3000)) - 1500;
	dy = (double)(random() % (int)(3000)) - 1500;
	translate(dx, dy);
	just_teleported = 1;

	game->addItem(new Animation(this, x, y,
			specialSprite, 0, 40, specialFrames/40, LAYER_HOTSPOTS-0.1));
	return(TRUE);
	}

void ArilouSkiff::calculate() {
	just_teleported = 0;
	Ship::calculate();

	if(!thrust) {
		vx *= 1 - frame_time * 0.02;
		vy *= 1 - frame_time * 0.02;
		}
	}

void ArilouSkiff::calculate_gravity() {}

int ArilouSkiff::accelerate(SpaceLocation *source, double angle, double velocity, 
		double max_speed) {
	if (source == this) 
		return Ship::accelerate(source, angle, velocity, max_speed);
	return false;
	}

int ArilouSkiff::accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, 
		double max_speed) {
	if (source == this) 
		return Ship::accelerate(source, angle, velocity, max_speed);
	return false;
	}

ArilouLaser::ArilouLaser(double langle, int lcolor, double lrange,
  int ldamage, int lfcount, SpaceLocation *opos, double oposdist) :
  SpaceLine(opos, 0.0, 0.0, langle, lrange, lcolor),
  frame(0),
  frame_count(lfcount),
  pos(opos),
  d(oposdist)
{
  damage_factor = ldamage;
  x = pos->normal_x() + (d * cos(angle * ANGLE_RATIO));
  y = pos->normal_y() + (d * sin(angle * ANGLE_RATIO));

  if(!pos->exists())
    state = 0;
}

void ArilouLaser::calculate()
{
  if((frame < frame_count) && (pos->exists())) {
    x = pos->normal_x() + (d * cos(angle * ANGLE_RATIO));
    y = pos->normal_y() + (d * sin(angle * ANGLE_RATIO));
    SpaceLine::calculate();
    frame++;
  } else
    state = 0;
}

REGISTER_SHIP(ArilouSkiff)
