#include "../ship.h"
REGISTER_FILE

#define MAX_DOGIS 4

class ChenjesuDOGI ;
class ChenjesuBroodhome : public Ship {
  double       weaponVelocity;
  double       shardRange;
  int          shardDamage;
  int          shardArmour;
  int          weaponDamage;
  int          weaponArmour;
  int          weaponFired;
  Shot        *weaponObject;

  double       specialVelocity;
  int          specialFuelSap;
  int          specialArmour;
  double       specialAccelRate;
  double       specialMass;
  int          specialNumDOGIs;

  public:
  ChenjesuBroodhome(double ox, double oy, double angle, ShipData *data, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
};

class ChenjesuShot : public AnimatedShot {
  public:
  ChenjesuShot(double ox, double oy, double oangle, double ov, int odamage,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void inflict_damage(SpaceObject *other);
	virtual void animateExplosion();
};

class ChenjesuDOGI : public AnimatedShot {
  int    sap_factor;
  double accel_rate;
  int   *num_dogis;

  public:
  ChenjesuDOGI(double ox, double oy, double oangle, double ov, int fuel_sap,
    int oarmour, double oaccel, double omass, Ship *oship,
    SpaceSprite *osprite, int *onum_dogis);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage(SpaceLocation *source);
  virtual void animateExplosion();
  virtual void soundExplosion();
};

ChenjesuBroodhome::ChenjesuBroodhome(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  shardRange     = scale_range(get_config_int("Weapon", "ShardRange", 0));
  shardDamage    = get_config_int("Weapon", "ShardDamage", 0);
  shardArmour    = get_config_int("Weapon", "ShardArmour", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponFired    = FALSE;
  weaponObject   = NULL;

  specialVelocity    = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialFuelSap     = get_config_int("Special", "FuelSap", 0);
  specialArmour      = get_config_int("Special", "Armour", 0);
  specialAccelRate   = scale_acceleration(get_config_float("Special", "AccelRate", 0), 1);
  specialMass        = get_config_float("Special", "Mass", 0);
  specialNumDOGIs    = 0;
}

int ChenjesuBroodhome::activate_weapon()
{
  if(weaponFired)
    return(FALSE);
  weaponObject = new ChenjesuShot(0.0, (height() / 2.0), angle,
    weaponVelocity, weaponDamage, weaponArmour, this,
    data->spriteWeapon, 64, 50);
  game->addItem(weaponObject);
  weaponFired = TRUE;
  return(TRUE);
}

int ChenjesuBroodhome::activate_special()
{
  if(specialNumDOGIs >= MAX_DOGIS)
    return(FALSE);

  game->addItem(new ChenjesuDOGI(0.0, -height() / 1.5,
    angle + 180.0, specialVelocity, specialFuelSap, specialArmour,
    specialAccelRate, specialMass, this, data->spriteSpecial,
    &specialNumDOGIs));
  return(TRUE);
}

void ChenjesuBroodhome::calculate()
{
  int i;

  Ship::calculate();

  if((weaponObject != NULL) && (!weaponObject->exists()))
    weaponObject = NULL;
  if(weaponFired && (!fire_weapon))
    weaponFired = FALSE;
  if((weaponObject != NULL) && (!fire_weapon)) {
    weaponObject->stop();
    for(i = 0; i < 8; i++) {
//			AnimatedShot::AnimatedShot(
			Shot *shot = new AnimatedShot(weaponObject, 0.0, 0.0,
        45.0 * (double)(i), weaponVelocity, shardDamage, shardRange,
        shardArmour, weaponObject, data->spriteExtra, 64, 50);
			shot->collide_flag_sameship = LAYER_SPECIAL;
      game->addItem(shot);
			}
    weaponObject->destroy();
    weaponObject = NULL;
    play_sound2(data->sampleExtra[0]);
  }
}

ChenjesuShot::ChenjesuShot(double ox, double oy, double oangle, double ov,
  int odamage, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  AnimatedShot(oship, ox, oy, oangle, ov, odamage, -1.0, oarmour, oship,
    osprite, ofcount, ofsize)
{
	collide_flag_sameship = bit(LAYER_SPECIAL);
  explosionSprite     = data->spriteWeaponExplosion;
  explosionFrameCount = 31;
  explosionFrameSize  = 50;
}

void ChenjesuShot::inflict_damage(SpaceObject *other)
{
  Shot::inflict_damage(other);
	if (other->mass <= 0) return;
  if(other->isShip())
    game->addItem(new FixedAnimation(this, other,
      explosionSprite, 0, explosionFrameCount,
			explosionFrameSize, LAYER_EXPLOSIONS));
  else
    game->addItem(new Animation(this, x, y,
      explosionSprite, 0, explosionFrameCount,
			explosionFrameSize, LAYER_EXPLOSIONS));
}
void ChenjesuShot::animateExplosion() {
	return;
	}
ChenjesuDOGI::ChenjesuDOGI(double ox, double oy, double ov, double oangle,
  int fuel_sap, int oarmour, double accel, double omass, Ship *oship,
  SpaceSprite *osprite, int *onum_dogis) :
  AnimatedShot(oship, ox, oy, ov, oangle, 0, -1.0, oarmour, oship,
    osprite, 64, 50),
  sap_factor(fuel_sap),
  accel_rate(accel),
  num_dogis(onum_dogis)
{
	layer = LAYER_SPECIAL;
  mass = omass;
  collide_flag_sameship = ALL_LAYERS;
  (*num_dogis)++;
}

void ChenjesuDOGI::calculate()
{
  AnimatedShot::calculate();

	if (ship) {
		target = ship->target;
		}
  if (target && !target->isInvisible()) {
    angle = trajectory_angle(target);

    if(((fabs(target->get_angle() - (trajectory_angle(target) + 180.0)) < 22.5) ||
      (fabs(target->get_angle() - (trajectory_angle(target) - 180.0)) < 22.5)) &&
			(distance(target) < 1000) )
      angle += 180.0;
    while(angle > 360.0)
      angle -= 360.0;
    while(angle < 0.0)
      angle += 360.0;
  }

	accelerate (this, angle, accel_rate * frame_time, v);

  if(state == 0)
    (*num_dogis)--;
}

void ChenjesuDOGI::inflict_damage(SpaceObject *other)
{
  play_sound(data->sampleExtra[1]);

  if(sameTeam(other))
    return;

  other->fuel_sap += sap_factor;
  other->handle_damage(this);
	double a;
	a = trajectory_angle(other);
	accelerate(this, a, v*2, GLOBAL_MAXSPEED);
}

void ChenjesuDOGI::handle_damage(SpaceLocation *source)
{
  AnimatedShot::handle_damage(source);
  if(state == 0)
    (*num_dogis)--;
}

void ChenjesuDOGI::animateExplosion()
{
  game->addItem(new Animation(this, x, y,
    data->spriteSpecialExplosion, 0, 16, 50, LAYER_EXPLOSIONS));
}

void ChenjesuDOGI::soundExplosion() {
	play_sound2(data->sampleExtra[2]);
	return;
	}


REGISTER_SHIP(ChenjesuBroodhome)
