#include "../ship.h"
REGISTER_FILE

class IlwrathAvenger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int cloak;
  int cloak_frame;

  public:
	static int cloak_color[3];
  IlwrathAvenger(double ox, double oy, double angle, ShipData *data, unsigned int code);

	virtual int isInvisible();
  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();
  virtual void calculate();
  virtual void animate(Frame *space);
};
int IlwrathAvenger::cloak_color[3] = { 15, 11, 9 };

IlwrathAvenger::IlwrathAvenger(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  cloak = FALSE;
  cloak_frame = 0;
}

int IlwrathAvenger::isInvisible() {
	if(cloak_frame >= 300)
		return(true);
	return false;
	}

int IlwrathAvenger::activate_weapon()
{
  if (cloak && (target != NULL)) angle = trajectory_angle(target);
  cloak = FALSE;
  game->addItem(new AnimatedShot(this, 0.0, (h / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour, 
    this, data->spriteWeapon, 50, 12, 1.0));
  return(TRUE);
}

void IlwrathAvenger::calculate_fire_special()
{
  special_low = FALSE;

  if(fire_special) {
    if((batt < special_drain) && (!cloak)) {
      special_low = TRUE;
      return;
    }

    if(special_recharge != 0)
      return;

    if(cloak) {
      cloak = FALSE;

      play_sound2(data->sampleSpecial[1]);
    } else {
      cloak = TRUE;
      play_sound2(data->sampleSpecial[0]);
      batt -= special_drain;
    }

    special_recharge = special_rate;
  }
}

void IlwrathAvenger::calculate_hotspots()
{
  if(!cloak)
    Ship::calculate_hotspots();
}

void IlwrathAvenger::calculate()
{
  if((cloak) && (cloak_frame < 300))
    cloak_frame += frame_time;
  if((!cloak) && (cloak_frame > 0))
    cloak_frame -= frame_time;

  Ship::calculate();
}

void IlwrathAvenger::animate(Frame *space) {
	if((cloak_frame > 0) && (cloak_frame < 300))
		sprite->animate_character( x, y, sprite_index, 
				pallete_color[cloak_color[cloak_frame / 100]], space);
	else if ((cloak_frame >= 300)) {
		sprite->animate_character( x, y, sprite_index, pallete_color[0], space);
		}
  else Ship::animate(space);		
	return;
	}

REGISTER_SHIP(IlwrathAvenger)
