#include "../ship.h"
REGISTER_FILE

#define T_FORM 0
#define Y_FORM 1

class MmrnmhrmXForm : public Ship {
  struct {
    public:
    double       speed_max;
    double       accel_rate;
    double       turn_rate;
    int          recharge_amount;
    int          recharge_rate;
    int          weapon_drain;
    int          weapon_rate;
    int          hotspot_rate;
    SpaceSprite *sprite;
  } form_data[2];

  int form;

  int        laserColor;
  double     laserRange;
	double     laserAngle;
  int        laserDamage;
  SpaceLine *laser1;
  SpaceLine *laser2;
  int        laserSpark;

  double missileRange;
  double missileVelocity;
  int    missileDamage;
  int    missileArmour;
  double missileTurnRate;

  public:
  MmrnmhrmXForm(double ox, double oy, double shipAngle, 
    ShipData *shipData, unsigned int code);

  void calculate();
  void handle_damage(SpaceLocation *source);

  int activate_weapon();
  int activate_special();
};

MmrnmhrmXForm::MmrnmhrmXForm(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code) 
	:
	Ship(ox, oy, shipAngle, shipData, code),
	laser1(NULL),
	laser2(NULL),
	laserSpark(0)
	{
	form_data[T_FORM].speed_max       = speed_max;
	form_data[T_FORM].accel_rate      = accel_rate;
	form_data[T_FORM].turn_rate       = turn_rate;
	form_data[T_FORM].recharge_amount = recharge_amount;
	form_data[T_FORM].recharge_rate   = recharge_rate;
	form_data[T_FORM].weapon_drain    = weapon_drain;
	form_data[T_FORM].weapon_rate     = weapon_rate;
	form_data[T_FORM].hotspot_rate    = hotspot_rate;
	form_data[T_FORM].sprite          = sprite;

	laserColor  = get_config_int("Weapon1", "Color", 0);
	laserDamage = get_config_int("Weapon1", "Damage", 0);
	laserRange  = scale_range(get_config_float("Weapon1", "Range", 0));

	missileRange    = scale_range(get_config_float("Weapon2", "Range", 0));
	missileVelocity = scale_velocity(get_config_float("Weapon2", "Velocity", 0));
	missileDamage   = get_config_int("Weapon2", "Damage", 0);
	missileArmour   = get_config_int("Weapon2", "Armour", 0);
	missileTurnRate = scale_turning(get_config_float("Weapon2", "TurnRate", 0));

	weapon_drain    = get_config_int("Special", "WeaponDrain", 0);
	weapon_rate     = scale_frames(get_config_float("Special", "WeaponRate", 0));
	double raw_hotspot_rate = get_config_float("Special", "HotspotRate", 0);
	hotspot_rate    = scale_frames(raw_hotspot_rate);
	recharge_amount = get_config_int("Special", "RechargeAmount", 0);
	recharge_rate   = scale_frames(get_config_float("Special", "RechargeRate", 0));
	turn_rate       = scale_turning(get_config_float("Special", "TurnRate", 0));
	speed_max       = scale_velocity(get_config_float("Special", "SpeedMax", 0));
	accel_rate      = scale_acceleration(get_config_float("Special", "AccelRate", 0), raw_hotspot_rate);

	form_data[Y_FORM].speed_max       = speed_max;
	form_data[Y_FORM].accel_rate      = accel_rate;
	form_data[Y_FORM].turn_rate       = turn_rate;
	form_data[Y_FORM].recharge_amount = recharge_amount;
	form_data[Y_FORM].recharge_rate   = recharge_rate;
	form_data[Y_FORM].weapon_drain    = weapon_drain;
	form_data[Y_FORM].weapon_rate     = weapon_rate;
	form_data[Y_FORM].hotspot_rate    = hotspot_rate;
	form_data[Y_FORM].sprite = sprite = data->spriteSpecial;

	laserAngle = asin(28 / laserRange) * 180 / PI;

	form = Y_FORM;
	activate_special();
	}

void MmrnmhrmXForm::calculate()
{
  if(laserSpark > 0) {
    laserSpark-= frame_time;
    if((laserSpark <= 0) && (laserSpark > -frame_time)) {
      if ((laser1->damage_factor > 0) && (laser2->damage_factor > 0))
        game->addItem(new Animation(this, 
          laser1->normal_x() + laser1->edge_x(),
          laser1->normal_y() + laser1->edge_y(),
          game->sparkSprite, 0, SPARK_FRAMES, 50, LAYER_EXPLOSIONS));
      laser1 = NULL;
      laser2 = NULL;
    }
  }

  Ship::calculate();
}

int MmrnmhrmXForm::activate_weapon()
{
  if(form == T_FORM) {
    game->addItem(laser1 = new Laser(this, angle + laserAngle,
      pallete_color[laserColor], laserRange, laserDamage, weapon_rate, this,
      -28.0, 2.0, true));
    game->addItem(laser2 = new Laser(this, angle - laserAngle,
      pallete_color[laserColor], laserRange, laserDamage, weapon_rate, this,
      28.0, 2.0, true));
    laserSpark = weapon_rate;
  }
  else if(form == Y_FORM) {
    game->addItem(new HomingMissile(this, -13.0, 2.0,
      angle - 25.0, missileVelocity, missileDamage, missileRange,
      missileArmour, missileTurnRate, this, data->spriteWeapon, target));
    game->addItem(new HomingMissile(this, 13.0, 2.0,
      angle + 25.0, missileVelocity, missileDamage, missileRange,
      missileArmour, missileTurnRate, this, data->spriteWeapon, target));
  }
  return(TRUE);
}

int MmrnmhrmXForm::activate_special()
{
	form_data[form].speed_max       = speed_max;
	form_data[form].accel_rate      = accel_rate;
	form_data[form].turn_rate       = turn_rate;
	form_data[form].recharge_amount = recharge_amount;
	form_data[form].recharge_rate   = recharge_rate;
	form_data[form].weapon_drain    = weapon_drain;
	form_data[form].weapon_rate     = weapon_rate;
	form_data[form].hotspot_rate    = hotspot_rate;
	form_data[form].sprite          = sprite;
	form = ((!form) & 1);
	speed_max       = form_data[form].speed_max;
	accel_rate      = form_data[form].accel_rate;
	turn_rate       = form_data[form].turn_rate;
	recharge_amount = form_data[form].recharge_amount;
	recharge_rate   = form_data[form].recharge_rate;
	weapon_drain    = form_data[form].weapon_drain;
	weapon_rate     = form_data[form].weapon_rate;
	hotspot_rate    = form_data[form].hotspot_rate;
	sprite          = form_data[form].sprite;
	weapon_sample   = form;
	return(TRUE);
}

void MmrnmhrmXForm::handle_damage(SpaceLocation *source) {
	double sl = speed_loss;
	Ship::handle_damage(source);
	if (sl) {
		activate_special();
		speed_loss = sl;
		Ship::handle_damage(source);
		activate_special();
		}
	}



REGISTER_SHIP(MmrnmhrmXForm)
