#include "../ship.h"
REGISTER_FILE

class MyconPodship : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  double       weaponHome;
  int          specialRepair;

  public:
  MyconPodship(double ox, double oy, double angle, ShipData *data, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
};

class MyconPlasma : public HomingMissile {
  double v;

	static SpaceSprite *spriteWeaponExplosion;
  int frame_count;
  int max_damage;

  public:
  MyconPlasma(double ox, double oy, double oangle, double ov, int odamage,
    double orange, double otrate, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();

  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage(SpaceLocation *source);
};


MyconPodship::MyconPodship(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponHome     = scale_turning(get_config_float("Weapon", "Homing", 0));
  specialRepair  = get_config_int("Special", "Repair", 0);
}

int MyconPodship::activate_weapon()
{
  game->addItem(new MyconPlasma(0.0, height(),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponHome, this,
    data->spriteWeapon, 64));
  return(TRUE);
}

int MyconPodship::activate_special()
{
  if(crew >= crew_max)
    return(FALSE);
  repair += specialRepair;
  handle_damage(this);
  return(TRUE);
}

SpaceSprite *MyconPlasma::spriteWeaponExplosion = NULL;
MyconPlasma::MyconPlasma(double ox, double oy, double oangle, double ov,
    int odamage, double orange, double otrate, Ship *oship,
    SpaceSprite *osprite, int ofcount) :
  HomingMissile( oship,ox, oy, oangle, ov, odamage, orange, 0, otrate, oship, 
      osprite, oship->target),
  v(ov),
  frame_count(ofcount),
  max_damage(odamage)
{
	spriteWeaponExplosion = data->spriteWeaponExplosion;
  collide_flag_sameship |= bit(LAYER_SHIPS);
  collide_flag_sameteam |= bit(LAYER_SHIPS);
}

void MyconPlasma::calculate()
{
  HomingMissile::calculate();

  sprite_index = (int)((d / range) * (double)(frame_count - 1));
  damage_factor = max_damage - (int)((d / range) * (double)(max_damage));
}

void MyconPlasma::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other);
  if (!other->isShot()) {
    game->addItem(new FixedAnimation(this, other,
      spriteWeaponExplosion, 0, 20, 50, LAYER_EXPLOSIONS));
    state = 0;
  }
}

void MyconPlasma::handle_damage(SpaceLocation *source) {
  int total_damage = 0;

  if((damage > 0) || (directDamage > 0)) {
    total_damage += damage;
    total_damage += directDamage;
    d += ((double)(total_damage) / (double)(max_damage) * range);
    damage = 0;
    directDamage = 0;
    if(d >= range)
      state = 0;
  }
}


REGISTER_SHIP(MyconPodship)
