#include "../ship.h"
REGISTER_FILE

class SyreenPenetrator : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialRange;
  double       specialVelocity;
  int          specialDamage;
  int          specialFrames;

  public:
  SyreenPenetrator(double ox, double oy, double angle, ShipData *data, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
};

class CrewPod : public SpaceObject {
	int frame_count;
	int frame_size;
	int frame_step;

	double velocity;
	int    life;
	int    lifetime;

	public:
	CrewPod(double ox, double oy, double oVelocity, int oLifeTime,
		Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

	virtual void calculate();
	virtual int isAlly(SpaceLocation *other);

	virtual void inflict_damage(SpaceObject *other);
	virtual void handle_damage(SpaceLocation *source);
	};

SyreenPenetrator::SyreenPenetrator(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialFrames   = get_config_int("Special", "Frames", 0);
}

int SyreenPenetrator::activate_weapon()
{
  game->addItem(new Missile(this, 0.0, (height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, data->spriteWeapon));
  return(TRUE);
}

int SyreenPenetrator::activate_special() {
	double minDist;
	minDist = specialRange + (width() / 2.0);
	int j;
	for (j = 0; j < game->num_targets; j += 1) {
		Ship *target = (Ship*) game->target[j];
		if (!target->isShip()) continue;
		if (!control->valid_target(target)) continue;
		int callDamage;
		double r;
		if (target) r = minDist - distance(target) - target->width() / 2.0;
		else r = -1;
		if ( (r > 0) && (target->crewPanelColor() == palette_color[2]) ) {
			callDamage = (int)(r * (double)(specialDamage) / specialRange);
			if (callDamage > specialDamage) callDamage = specialDamage;
			callDamage += (random() % specialDamage);
			if ((target->getCrew() - callDamage) < 1) callDamage = target->getCrew() - 1;
			target->directDamage += callDamage;
                        callDamage = target->getCrew();
			target->handle_damage(this);
                        callDamage -= target->getCrew();
			for(int i = 0; i < callDamage; i++) {
				game->addItem(new CrewPod(
						target->normal_x() + (cos((trajectory_angle(target) - 180.0) *
						ANGLE_RATIO) * target->width()) + (double)(random() % 50) - 25.0,
						target->normal_y() + (sin((trajectory_angle(target) - 180.0) *
						ANGLE_RATIO) * target->height()) + (double)(random() % 50) - 25.0,
						specialVelocity, specialFrames, this, data->spriteSpecial, 32, 
						specialFrames));
				}
			}
		}
	return(TRUE);
	}

CrewPod::CrewPod(double ox, double oy, double oVelocity, int oLifeTime,
  Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize) :
  SpaceObject(oship, ox, oy, 0.0, osprite),
  frame_count(ofcount),
  frame_size(ofsize),
  frame_step(0),
  velocity(oVelocity),
  life(0),
  lifetime(oLifeTime)
{
	collide_flag_sameship = ALL_LAYERS;
	collide_flag_sameteam = ALL_LAYERS;
	layer = LAYER_SPECIAL;
}

int CrewPod::isAlly(SpaceLocation *other) {
	return true;
	}

void CrewPod::calculate() {
	frame_step += frame_time;
	while (frame_step >= frame_size) {
		frame_step -= frame_size;
		sprite_index++;
		if(sprite_index == frame_count)	sprite_index = 0;
		}

	life += frame_time;
	if(life >= lifetime) {
		state = 0;
		return;
		}

  if (ship && ship->exists()) {
		vx = cos(trajectory_angle(ship) * ANGLE_RATIO) * velocity;
		vy = sin(trajectory_angle(ship) * ANGLE_RATIO) * velocity;
		}
	else ship = NULL;

	SpaceObject::calculate();
	}

void CrewPod::inflict_damage(SpaceObject *other)
{
  if (other->isShip()) {
    sound.stop(data->sampleExtra[0]);
    sound.play(data->sampleExtra[0]);
    ((Ship *)(other))->repair++;
    other->handle_damage(this);
		state = 0;
  }
}

void CrewPod::handle_damage(SpaceLocation *source)
{
  state = 0;
  damage = 0;
  directDamage = 0;
}



REGISTER_SHIP(SyreenPenetrator)
