#include "../ship.h"
REGISTER_FILE

class UmgahDrone : public Ship {
  int          weaponDamage;
  int          weaponCone;
  int          specialRate;

  public:

  int          firing;
  int          damage_type;

  UmgahDrone(double ox, double oy, double angle, ShipData *data, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate_thrust();
  virtual void calculate();
};

class UmgahCone : public SpaceObject {
  double dist;

  int cone_index;
  double damage;
  double residual_damage;
  int damage_type;

  public:
  UmgahCone(double odist, int odamage, UmgahDrone *oship, SpaceSprite *osprite);

	virtual bool change_owner(SpaceLocation *new_owner);
  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void UmgahCone::animate(Frame* space);
  virtual int canCollide(SpaceLocation* other);

};

UmgahDrone::UmgahDrone(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{

  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  specialRate    = special_rate;
  damage_type    = get_config_int("Weapon", "DamageType", 0);

  firing = false;
  game->addItem(new UmgahCone(81, weaponDamage, this, data->spriteWeapon));

}

int UmgahDrone::activate_weapon() {
  firing = true;
  return(TRUE);
}

int UmgahDrone::activate_special()
{
  special_rate = specialRate;
  vx = 0.0;
  vy = 0.0;
  x -= (cos(angle * ANGLE_RATIO) * width() * 2.0);
  y -= (sin(angle * ANGLE_RATIO) * height() * 2.0);
  if (!thrust) special_rate = 50;
  return(TRUE);
}

void UmgahDrone::calculate_thrust()
{
  if(special_recharge == 0)
    Ship::calculate_thrust();
}

void UmgahDrone::calculate()
{
  if (fire_weapon)
    recharge_step = recharge_rate;
  else
    firing = false;

  Ship::calculate();
}

UmgahCone::UmgahCone(double odist, int odamage, UmgahDrone *oship,
  SpaceSprite *osprite) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), 0.0, osprite),
  dist(odist)
{
	layer = LAYER_SHOTS;
	set_depth(LAYER_EXPLOSIONS);

  id = SPACE_SHOT;
  damage_factor = 1;

  damage_type = oship->damage_type;
  damage = (odamage/1000.0);
  residual_damage = 1e-5;

  cone_index = 0;

  x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
  y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
  sprite_index &= 63; 

	collide_flag_sameship = 0;
}

bool UmgahCone::change_owner(SpaceLocation *new_owner) {
	return false;
	}

void UmgahCone::calculate()
{
//  calc_base();

  if (ship==NULL) {
    state = 0; return; }
  if(!ship->exists()) {
    state = 0; return; }
//  if (((UmgahDrone*)ship)->firing) {
    x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
    y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
    sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
    sprite_index &= 63; 
//	}//sprite_index += (cone_index << 6); }
//  else state = 0;
//    residual_damage = 0;


  if (damage_type == 2) {
    residual_damage = fabs(residual_damage);
    if (residual_damage <= 0) residual_damage = 1e-5; }
}

void UmgahCone::inflict_damage(SpaceObject *other)
{
  switch (damage_type) {
    case 0: {
            residual_damage = damage * frame_time / 2.0;
            while (residual_damage >= 1) {
              residual_damage -= 1;
              SpaceObject::inflict_damage(other); };
            residual_damage *= 1e4;
            if (residual_damage > (random() % 10000))
              SpaceObject::inflict_damage(other);
            }; break;
    case 1: {
            residual_damage += damage * frame_time / 2.0;
            while (residual_damage >= 1) {
              residual_damage -= 1;
              SpaceObject::inflict_damage(other); };
            }; break;
    case 2: if (residual_damage > 0) {
              residual_damage += damage * frame_time;
              while (residual_damage >= 1) {
                residual_damage -= 1;
                SpaceObject::inflict_damage(other); };
              residual_damage *= -1;
            }; break;
  }

}

int UmgahCone::canCollide(SpaceLocation* other) {
//  calc_base();
  return (((UmgahDrone*)ship)->firing & !other->isPlanet());
}

void UmgahCone::animate(Frame* space) {
//  calc_base();
  if (!((UmgahDrone*)ship)->firing) return;
  if ((cone_index += 1) > 5) cone_index = 0;
  int si = sprite_index;
  sprite_index += (cone_index << 6);
  SpaceObject::animate(space);
  sprite_index = si;
}



REGISTER_SHIP(UmgahDrone)
