#include "../ship.h"
REGISTER_FILE

class VuxIntruder : public Ship {
	int    weaponColor;
	double weaponRange;
	int    weaponDamage;

	double specialRange;
	double specialVelocity;
	double specialSlowdown;
	int    specialArmour;

	public:
	VuxIntruder(double ox, double oy, double angle, ShipData *data, unsigned int code);

	virtual int activate_weapon();
	virtual int activate_special();
	virtual SpaceLocation *get_ship_phaser() ;
	void relocate() ;
  virtual void animate(Frame *space);
};

class VuxLimpet : public AnimatedShot {
  double slowdown_factor;

  public:
  VuxLimpet(double ox, double oy, double ov, double slowdown, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

VuxIntruder::VuxIntruder(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)

{
  weaponColor  = get_config_int("Weapon", "Color", 0);
  weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialSlowdown = get_config_float("Special", "Slowdown", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);

}

int VuxIntruder::activate_weapon()
{
  game->addItem(new Laser(this, angle,
    pallete_color[weaponColor], weaponRange, weaponDamage, weapon_rate,
    this, width()/11, (h / 2.07), true));
  return(TRUE);
}

int VuxIntruder::activate_special()
{
  game->addItem(new VuxLimpet(0, -height() / 2.8,
    specialVelocity, specialSlowdown, specialRange, specialArmour, this,
    data->spriteSpecial, 100, 5));
  return(TRUE);
}

VuxLimpet::VuxLimpet(double ox, double oy, double ov, double slowdown,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofsize, int ofcount) :
  AnimatedShot(oship, ox, oy, 0.0, ov, 0, orange, oarmour, oship, osprite,
    ofcount, ofsize),
  slowdown_factor(slowdown)
{
  if((ship->target) && (!ship->target->isInvisible()))
    angle = trajectory_angle(ship->target);
  else
    angle = ship->get_angle() - 180.0;

  vx    = v * cos(angle * ANGLE_RATIO);
  vy    = v * sin(angle * ANGLE_RATIO);

  if(!ship->exists())
    state = 0;
}

void VuxLimpet::calculate()
{
	if(!ship->exists()) {
		state = 0;
		return;
		}

	if(ship && (ship->target) && (!ship->target->isInvisible())) {
		angle = trajectory_angle(ship->target);
		vx = v * cos(angle * ANGLE_RATIO);
		vy = v * sin(angle * ANGLE_RATIO);
		}
	AnimatedShot::calculate();
	}

void VuxLimpet::inflict_damage(SpaceObject *other) {
	if(!other->isShip()) {
		if (other->damage_factor || other->mass) state = 0;
		return;
		}

	Ship *target = (Ship *) other;

	play_sound(data->sampleSpecial[1]);

	//MYCODE begin
	int hx,hy,tries, col;

	BITMAP *bmp = target->spritePanel->get_bitmap(0);
	//find a random spot on the target ship where it "exist" 
	tries = 0;
	while (tries < 30) {
		hx = 18 + (rand() % 27);  //NOT part of physics ; not synchronized in network games ; varies with graphics mode
		hy = 15 + (rand() % 36);
		tries++;
		col = getpixel(bmp,hx,hy);
		if (col == bitmap_mask_color(bmp)) continue;
		if (col == 0) continue;
		if (col == -1) continue;
		if (col == palette_color[8]) continue;
		break;
		} 

	// draw the Limplet on the ship panel
	BITMAP *s = sprite->get_bitmap(sprite_index);
	stretch_sprite(bmp, s, hx - s->w/4, hy - s->h/4,	s->w/2, s->h/2 );
	target->update_panel = TRUE;
	target->speed_loss += slowdown_factor;
	target->handle_damage(this);
	state = 0;
	}

void VuxIntruder::relocate() {
	if (target && (distance(target) > 500)) {
		x = target->normal_x() + (cos(angle * ANGLE_RATIO) * 125.0);
		y = target->normal_y() + (sin(angle * ANGLE_RATIO) * 125.0);
		angle = trajectory_angle(target);
		if (angle > 360) angle -= 360;
		if (angle < 0) angle += 360;
		}
        angle = ((int)(angle / 5.625)) * 5.625;
	sprite_index = int(16+angle * 64 / 360.) & 63;
	return;
	}


void VuxIntruder::animate(Frame *space){

        double back_x=w/3.60, back_y=-h/2.33,
               frnt_x=w/5.55, frnt_y=+h/17.01,
               back_y_1=-h/2.06;
  
        double tx = cos(angle*ANGLE_RATIO);
        double ty = sin(angle*ANGLE_RATIO);
        int s_index = ((int)(angle/ 5.625) + 16) & 63;

        if (turn_right)
                data->spriteWeapon->animate(x + frnt_y * tx - frnt_x * ty,
                     y + frnt_y * ty + frnt_x * tx, s_index + ((rand()%3) << 6), space);
        if (turn_left)
                data->spriteWeapon->animate(x + frnt_y * tx + frnt_x * ty,
                     y + frnt_y * ty - frnt_x * tx, s_index + ((rand()%3) << 6), space);

        s_index += 32; s_index &= 63;

        if (thrust) {
                data->spriteExtra->animate(x + back_y_1 * tx - back_x * ty,
                     y + back_y_1 * ty + back_x * tx, s_index + ((rand()%3) << 6), space);
                data->spriteExtra->animate(x + back_y_1 * tx + back_x * ty,
                     y + back_y_1 * ty - back_x * tx, s_index + ((rand()%3) << 6), space); }
        else {
                if (turn_left)
                        data->spriteWeapon->animate(x + back_y * tx - back_x * ty,
                             y + back_y * ty + back_x * tx, s_index + ((rand()%3) << 6), space);
                if (turn_right)
                        data->spriteWeapon->animate(x + back_y * tx + back_x * ty,
                             y + back_y * ty - back_x * tx, s_index + ((rand()%3) << 6), space); }

        Ship::animate(space);
};


class VuxPhaser : public Phaser {
	public:
	VuxPhaser(double ox, double oy, double nx, double ny, VuxIntruder *ship, 
			SpaceSprite *sprite, int osprite_index, int *ocolors, 
			int onum_colors, int ofsize, int steps, int step_time) ;

	virtual void calculate();
	};



VuxPhaser::VuxPhaser(double ox, double oy, double _nx, double _ny, VuxIntruder *ship, 
							 SpaceSprite *sprite, int osprite_index, int *ocolors, 
							 int onum_colors, int ofsize, int steps, int step_size) :
		Phaser(ship, ox, oy, _nx, _ny, ship, sprite, osprite_index, ocolors, onum_colors, ofsize, 
			steps, step_size) {}
void VuxPhaser::calculate() {
	Phaser::calculate();
	if (!ship) return;
	double bx = ship->normal_x(), by = ship->normal_y();
	((VuxIntruder*)ship)->relocate();
	translate(ship->normal_x() - bx, ship->normal_y() - by);
	return;
	}

SpaceLocation *VuxIntruder::get_ship_phaser() {
	return new VuxPhaser(
			x - cos(angle * PI/180) * PHASE_MAX * w,
			y - sin(angle * PI/180) * PHASE_MAX * h,
			cos(angle * PI/180) * PHASE_MAX * w,
			sin(angle * PI/180) * PHASE_MAX * h,
			this, sprite, sprite_index, hot_color, HOT_COLORS,
			PHASE_DELAY, PHASE_MAX, PHASE_DELAY);
	}




REGISTER_SHIP(VuxIntruder)
