#include "../ship.h"
REGISTER_FILE

#include "../frame.h"

class YehatTerminator : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int          specialFrames;
  int          shieldFrames;

  public:
  YehatTerminator(double ox, double oy, double angle, ShipData *data, unsigned int code);
  virtual int isProtected();

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void animate(Frame *space);
  virtual void handle_damage(SpaceLocation *source);
};

YehatTerminator::YehatTerminator(double ox, double oy, double angle, ShipData *data, unsigned int code) 
	:
	Ship(ox, oy, angle, data, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialFrames = get_config_int("Special", "Frames", 0);
  shieldFrames  = 0;
}

int YehatTerminator::activate_weapon()
{
  game->addItem(new Missile(this, -24.0, 14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, data->spriteWeapon));
  game->addItem(new Missile(this, 24.0, 14,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, data->spriteWeapon));
  return(TRUE);
}

int YehatTerminator::activate_special()
{
  shieldFrames = (shieldFrames % frame_time) + specialFrames;
  return(TRUE);
}

void YehatTerminator::calculate()
{
   if(shieldFrames > 0)
     shieldFrames-= frame_time;
   Ship::calculate();
}

void YehatTerminator::animate(Frame *space)
{
  if(shieldFrames > 0)
    data->spriteSpecial->animate( x, y, sprite_index, space);
  else
    sprite->animate( x, y, sprite_index, space);
}

void YehatTerminator::handle_damage(SpaceLocation *source)
{
  if(shieldFrames > 0) damage = 0;
  Ship::handle_damage(source);
}

int YehatTerminator::isProtected()
{
  return (shieldFrames > 0);
}


REGISTER_SHIP (YehatTerminator)
