#include "../ship.h"
REGISTER_FILE

class ChmmrAvatar : public Ship {
  double weaponRange;
  int    weaponDamage;

  double       specialForce;

  double extraRange;
  int    extraDamage;
  int    extraFrames;
  int    extraRechargeRate;
  int    extraColor;
  int    extraArmour;

  public:
  ChmmrAvatar(double ox, double oy, double shipAngle, 
    ShipData *shipData, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void materialize();
};

class ChmmrLaser : public Laser {
  public:
  ChmmrLaser(double langle, double lrange, int ldamage, int lfcount,
    SpaceLocation *opos, double rel_x, double rel_y);
};

class ChmmrBeam : public SpaceObject {
  int frame;
  int frame_count;
  Ship *ship;
  SpaceObject *target;

  public:
  ChmmrBeam(Ship *oship, int oframes);

  virtual void calculate();
  virtual void animate(Frame *space);
};

class ChmmrZapSat : public SpaceObject {
	int frame;

	double lRange;
	int    lDamage;
	int    lFrames;
	int    lRechargeRate;
	int    lRecharge;
	int    lColor;
	int    armour;

	public:
	ChmmrZapSat(double oangle, double orange, int odamage, int oframes,
			int orechargerate, int ocolor, int oarmour, Ship *oship,
			SpaceSprite *osprite);

	virtual void calculate();
	virtual void handle_damage(SpaceLocation *source);
	virtual int canCollide(SpaceLocation *other);
	};

ChmmrAvatar::ChmmrAvatar(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code) 
	:
	Ship(ox, oy, shipAngle, shipData, code) 
	{

	weaponRange  = scale_range(get_config_float("Weapon", "Range", 0));
	weaponDamage = get_config_int("Weapon", "Damage", 0);

	specialForce = scale_velocity(get_config_float("Special", "Force", 0));

	extraRange        = scale_range(get_config_float("Extra", "Range", 0));
	extraFrames       = get_config_int("Extra", "Frames", 0);
	extraDamage       = get_config_int("Extra", "Damage", 0);
	extraRechargeRate = get_config_int("Extra", "RechargeRate", 0);
	extraColor        = get_config_int("Extra", "Color", 0);
	extraArmour       = get_config_int("Extra", "Armour", 0);
	}

int ChmmrAvatar::activate_weapon() {
  game->addItem(new ChmmrLaser(angle, weaponRange,
    weaponDamage, weapon_rate, this, 0.0, 25.0));
	if ((random() % 200) < frame_time) {
		double f, o, c, s;
		f = (((double)random()) / MAXINT);
		o = ((((double)random()) / MAXINT) - 0.5) * (f+.5) / 2.5;
		c = cos(angle*PI/180);
		s = sin(angle*PI/180);
		game->addItem(new Animation(this, 
				x + (c * f + s * o) * weaponRange, 
				y + (s * f + c * o) * weaponRange, 
				data->spriteWeaponExplosion, (random()&1)*6, 6, 50, LAYER_HOTSPOTS));
		f = (((double)random()) / MAXINT);
		o = ((((double)random()) / MAXINT) - 0.5) * (f+.5) / 2.5;
		c = cos(angle*PI/180);
		s = sin(angle*PI/180);
		game->addItem(new Animation(this, 
				x + (c * f + s * o) * weaponRange, 
				y + (s * f + c * o) * weaponRange, 
				data->spriteWeaponExplosion, (random()&1)*6, 6, 50, LAYER_HOTSPOTS));
		f = (((double)random()) / MAXINT);
		o = ((((double)random()) / MAXINT) - 0.5) * (f+.5) / 2.5;
		c = cos(angle*PI/180);
		s = sin(angle*PI/180);
		game->addItem(new Animation(this, 
				x + (c * f + s * o) * weaponRange, 
				y + (s * f + c * o) * weaponRange, 
				data->spriteWeapon, (random()&1)*6, 6, 100, LAYER_HOTSPOTS));
		}
	return(TRUE);
	}

int ChmmrAvatar::activate_special()
{
	if ((target != NULL) && (!target->isInvisible()) && (target->mass > 0) && (distance(target) < 2000)) {
		game->addItem(new ChmmrBeam(this, special_rate));
		target->accelerate(this, target->trajectory_angle(this), specialForce / target->mass, GLOBAL_MAXSPEED);
		return (true);
		}
	return false;
}

void ChmmrAvatar::materialize()
{
	Ship::materialize();
  game->addItem(new ChmmrZapSat(0.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
  game->addItem(new ChmmrZapSat(120.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
  game->addItem(new ChmmrZapSat(240.0, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteExtra));
}

ChmmrLaser::ChmmrLaser(double langle, double lrange, int ldamage,
  int lfcount, SpaceLocation *opos, double rel_x, double rel_y) :
  Laser(opos, langle, pallete_color[hot_color[random() % HOT_COLORS]], lrange,
    ldamage, lfcount, opos, rel_x, rel_y, true)
{
}

ChmmrBeam::ChmmrBeam(Ship *oship, int oframes) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), oship->get_angle(), 
    game->sparkSprite),
  frame(0),
  frame_count(oframes),
  ship(oship),
  target(oship->target)
{
	collide_flag_anyone = 0;
  if(!ship->exists()) {
    state = 0;
    return;
  }

  target = ship->target;
  if((target == NULL) || (target->isInvisible())) {
    state = 0;
    return;
  }
}

void ChmmrBeam::calculate()
{
  if(!ship->exists()) {
    state = 0;
    return;
  }

  target = ship->target;
  if((!target) || (target->isInvisible())) {
    state = 0;
    return;
  }

  x = ship->normal_x();
  y = ship->normal_y();
  frame += frame_time;
  if(frame >= frame_count) state = 0;
}

void ChmmrBeam::animate(Frame *space)
{
  const int beam_color[5] = { 80, 81, 82, 83, 84 };
  int i;
  double length;

  length = sqrt((target->get_vx() * target->get_vx()) +
    (target->get_vy() * target->get_vy())) + (target->width() / 4.0);

	for(i = 3; i >= 0 ; i--)
		target->getSprite()->animate_character(
				target->normal_x() + (i + 1) * 
				cos((trajectory_angle(target) - 180.0) * ANGLE_RATIO) * length,
				target->normal_y() + (i + 1) *
				sin((trajectory_angle(target) - 180.0) * ANGLE_RATIO) * length,
				target->getSpriteIndex(), pallete_color[beam_color[i]], space);
}

ChmmrZapSat::ChmmrZapSat(double oangle, double orange, int odamage,
  int oframes, int orechargerate, int ocolor, int oarmour, Ship *oship,
  SpaceSprite *osprite) :
  SpaceObject(oship, 0.0, 0.0, 0.0, osprite),
  lRange(orange),
  lDamage(odamage),
  lFrames(oframes),
  lRechargeRate(orechargerate),
  lRecharge(0),
  lColor(ocolor),
  armour(oarmour)
{
	layer = LAYER_SPECIAL;
	collide_flag_anyone = ALL_LAYERS &~ bit(LAYER_CBODIES);
	id |= CHMMR_SPEC;
  angle = oangle;
  x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
  y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);

  if(!ship->exists()) state = 0;
}

void ChmmrZapSat::calculate() {

	SpaceObject::calculate();

	if (!ship) {
		state = 0;
		return;
		}

	x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
	y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);

	angle += 0.1 * frame_time;
	if(angle >= 360.0) angle -= 360.0;
	sprite_index = (int)(2*angle / 5.625) + 16;
	sprite_index &= 63;

	if(lRecharge > 0) {
		lRecharge -= frame_time;
		return;
		}

	Query q;
	for (q.begin(this, OBJECT_LAYERS &~ bit(LAYER_CBODIES), lRange); q.currento; q.next()) {
		if (!q.currento->isInvisible() && !q.currento->sameTeam(this) && (q.currento->collide_flag_anyone&bit(LAYER_LINES))) {
			game->addItem(new PointLaser(this, pallete_color[lColor], 1, lFrames,
                        		this, q.currento, 0.0, 0.0));
			sound.play((SAMPLE *)(melee[MELEE_BOOM + 0].dat));
			lRecharge += lRechargeRate;
			break;
			}
		}
	return;
	}
int ChmmrZapSat::canCollide(SpaceLocation *other) {
	if (!other->damage_factor) return false;
	return SpaceObject::canCollide(other);
	}

void ChmmrZapSat::handle_damage(SpaceLocation *source) {
	if ((damage > 0) || (directDamage > 0)) {
		armour -= damage;
		armour -= directDamage;
		damage = 0;
		directDamage = 0;
		if(armour <= 0) {
			armour = 0;
			state = 0;
			game->addItem(new Animation(this, x, y,
					game->kaboomSprite, 0, KABOOM_FRAMES, 50, LAYER_EXPLOSIONS));
			sound.stop(data->sampleExtra[0]);
			sound.play(data->sampleExtra[0]);
			}
		}
	}




REGISTER_SHIP(ChmmrAvatar)
