#include "../ship.h"
REGISTER_FILE

class MelnormeTrader : public Ship {
  double       weaponRange;
	double       weaponRangeUp;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  SpaceObject *weaponObject;

  double       specialRange;
  double       specialVelocity;
  int          specialFrames;
  int          specialArmour;

  public:
  MelnormeTrader(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual void calculate();

  virtual int activate_weapon();
  virtual int activate_special();
};

class MelnormeShot : public Shot {
  double v;
  int frame;
  int frame_step;
  int frame_size;
  int frame_count;
  int charge_frame;
  int charge_phase;
  int released;
	double RangeUp;

  public:
  MelnormeShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, double rangeup, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();
  virtual void handle_damage(SpaceLocation *source);
};

class MelnormeDisable : public SpaceObject {
  Ship *ship;
  int   disableframe;
  int   disableframe_count;
  int   frame_step;
  int   frame_size;
  int   frame_count;

  public:
  MelnormeDisable(Ship *oship, SpaceSprite *osprite, int ofcount,
    int ofsize, int disableFrames);

  virtual void calculate();
};

class MelnormeSpecial : public AnimatedShot {
  int disableFrames;
  SpaceSprite *disableSprite;

  public:
  MelnormeSpecial(double ox, double oy, double oangle, double ov,
    int oframes, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofsize, int ofcount);

  void inflict_damage(SpaceObject *other);
  void animateExplosion();
};

MelnormeTrader::MelnormeTrader(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponRangeUp  = scale_range(get_config_float("Weapon", "RangeUp", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponObject   = NULL;

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialFrames   = get_config_int("Special", "Frames", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

void MelnormeTrader::calculate()
{
  Ship::calculate();
  if (weaponObject) {
    if (!weaponObject->exists()) weaponObject = NULL;
    if (fire_weapon) recharge_step += frame_time; else weaponObject = NULL;
    }
}

int MelnormeTrader::activate_weapon()
{
  if(weaponObject)
    return(FALSE);
  game->addItem(weaponObject = new MelnormeShot(
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange, weaponRangeUp, 
    weaponArmour, this, data->spriteWeapon, 10, 50));
  return(TRUE);
}

int MelnormeTrader::activate_special() {
  game->addItem( new MelnormeSpecial(
    0.0, (height() / 2.0), angle, specialVelocity, specialFrames,
    specialRange, specialArmour, this, data->spriteSpecial, 20, 50));
  return(TRUE);
}

MelnormeShot::MelnormeShot(double ox, double oy, double oangle, double ov,
  int odamage, double orange, double rangeup, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  Shot(oship, ox, oy, oangle, ov, odamage, orange, oarmour, oship, osprite),
  v(ov),
  frame(0),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount),
  charge_frame(0),
  charge_phase(0),
  released(FALSE),
	RangeUp(rangeup)
{  
  vx = ship->get_vx();
  vy = ship->get_vy();
}

void MelnormeShot::calculate() {
	if(released) {
		Shot::calculate();
		}
	else {
		SpaceObject::calculate();
		}
	if (!exists()) return;
	frame_step += frame_time;
	if(frame_step >= frame_size) {
		frame_step -= frame_size;
		frame++;
		if(frame == frame_count) {
			frame = 0;
			if((!released) && (charge_phase < 3)) {
				charge_frame++;
				if(charge_frame == 5) {
					charge_frame = 0;
					charge_phase++;
					damage_factor *= 2;
					armour *= 2;
					range += RangeUp;
					play_sound(data->sampleWeapon[0]);
					}
				}
			}
		}
	sprite_index = (charge_phase * 10) + frame;
	if(!released) {
		if (!ship || !ship->fire_weapon) {
			vx = cos(angle * ANGLE_RATIO) * v;
			vy = sin(angle * ANGLE_RATIO) * v;
			released = TRUE;
			play_sound(data->sampleSpecial[0]);
			}
		else {
			angle = ship->get_angle();
			x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * (ship->width() / 2.0));
			y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * (ship->height() / 2.0));
			vx = ship->get_vx();
			vy = ship->get_vy();
			}
		}
	return;
	}

void MelnormeShot::animateExplosion()
{
  game->addItem( new Animation(this, x, y,
    data->spriteWeaponExplosion, (charge_phase * 20), 20, 25, 
    LAYER_EXPLOSIONS));
}

void MelnormeShot::handle_damage(SpaceLocation *source)
{
  int old = armour;
  Shot::handle_damage(source);
  if (!released && (armour > 0)) armour = old;
}

MelnormeDisable::MelnormeDisable(Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize, int disableFrames) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), 0.0, osprite),
  ship(oship),
  disableframe(0),
  disableframe_count(disableFrames),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount)
{
	collide_flag_anyone = 0;
}

void MelnormeDisable::calculate() {
	frame_step+= frame_time;
	while (frame_step >= frame_size) {
		frame_step -= frame_size;
		sprite_index++;
		if(sprite_index == frame_count)
			sprite_index = 0;
		}
	if(!ship->exists()) {
		state = 0;
		return;
		}
	x = ship->normal_x();
	y = ship->normal_y();
	ship->nextkeys &= ~(keyflag::left | keyflag::right | keyflag::special);
	ship->nextkeys |= keyflag::right;
	disableframe += frame_time;
	if (disableframe >= disableframe_count) state = 0;
	SpaceObject::calculate();
	}

MelnormeSpecial::MelnormeSpecial(double ox, double oy, double oangle,
  double ov, int oframes, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  AnimatedShot(oship, ox, oy, oangle, ov, 0, orange, oarmour, oship, osprite,
    ofcount, ofsize),
  disableFrames(oframes),
  disableSprite(data->spriteExtra)
{
	collide_flag_anyone = bit(LAYER_SHIPS);
}

void MelnormeSpecial::animateExplosion() {}

void MelnormeSpecial::inflict_damage(SpaceObject *other) {
	if (other->isShip()) game->addItem(new MelnormeDisable(
			(Ship *)(other), disableSprite, 20, 50, disableFrames));
	state = 0;
	return;
	}



REGISTER_SHIP(MelnormeTrader)
