#include "../ship.h"
REGISTER_FILE
#include "../melee/mcbodies.h"

class PkunkFury : public Ship {
  int reborn;

  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  PkunkFury(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual void handle_damage(SpaceLocation *source);
  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

PkunkFury::PkunkFury(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

	reborn = 0;
}

void PkunkFury::handle_damage(SpaceLocation *source) {
  Ship::handle_damage(source);

  if (exists()) return;
  if (random() % 2) return;

  state = 1;

	x = (double)(random() % (int)(3001)) - 1500;
	y = (double)(random() % (int)(3001)) - 1500;
	SpaceLocation *spacePlanet = nearest_planet();
	if (spacePlanet && (distance(spacePlanet) < 1000.0)) {
		x += cos(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
		y += sin(trajectory_angle(spacePlanet) * ANGLE_RATIO) * 1000.0;
		}
	angle = random() % 360;
	sprite_index = int(16 + angle*64/360.) & 63;
	vx = vy = 0.0;
	crew = crew_max;
	batt = batt_max;
	reborn = TRUE;
	update_panel = TRUE;
	play_sound(data->sampleExtra[0]);
	game->removeItem(this);
	game->addItem(new Phaser (this, 
			x - cos((angle+0) * PI/180) * PHASE_MAX * w, 
			y - sin((angle+0) * PI/180) * PHASE_MAX * h, 
			cos((angle+0) * PI/180) * PHASE_MAX * w, 
			sin((angle+0) * PI/180) * PHASE_MAX * h, 
			this, sprite, sprite_index, hot_color, HOT_COLORS, 
			PHASE_DELAY, PHASE_MAX, PHASE_DELAY) );
	game->addItem(new Phaser (this, 
			x - cos((angle+90) * PI/180) * PHASE_MAX * w, 
			y - sin((angle+90) * PI/180) * PHASE_MAX * h, 
			cos((angle+90) * PI/180) * PHASE_MAX * w, 
			sin((angle+90) * PI/180) * PHASE_MAX * h, 
			NULL, sprite, (sprite_index+0)&63, hot_color, HOT_COLORS, 
			PHASE_DELAY, PHASE_MAX, PHASE_DELAY) );
	game->addItem(new Phaser (this, 
			x - cos((angle-90) * PI/180) * PHASE_MAX * w, 
			y - sin((angle-90) * PI/180) * PHASE_MAX * h, 
			cos((angle-90) * PI/180) * PHASE_MAX * w, 
			sin((angle-90) * PI/180) * PHASE_MAX * h, 
			NULL, sprite, (sprite_index-0)&63, hot_color, HOT_COLORS, 
			PHASE_DELAY, PHASE_MAX, PHASE_DELAY) );
  }

int PkunkFury::activate_weapon()
{
  game->addItem(new AnimatedShot(this, 
    0.0, (height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon, 10, 1, 1.0));
  game->addItem(new AnimatedShot(this, 
    -(width() / 2.0), 0.0, angle - 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, data->spriteWeapon, 10, 1, 1.0));
  game->addItem(new AnimatedShot(this, 
    width() / 2.0, 0.0, angle + 90.0, weaponVelocity, weaponDamage,
    weaponRange, weaponArmour, this, data->spriteWeapon, 10, 1, 1.0));

  return(TRUE);
}

void PkunkFury::calculate_fire_special()
{
  int i;

  if(fire_special) {
    if((special_recharge != 0) || (batt >= batt_max))
      return;

    batt += special_drain;
    if(batt > batt_max)
      batt = batt_max;

    special_recharge = special_rate;

    //sound.stop(data->sampleSpecial[special_sample]);

    do {
      i = random() % 6;
    } while(i == special_sample);
    special_sample = i;

    play_sound(data->sampleSpecial[special_sample]);
  }
}



REGISTER_SHIP(PkunkFury)
