
#include "../ship.h"

class SlylandroProbe : public Ship {
  int frame;
  int thrustActive;
  int thrustForward;
  int segments;
  int segment_length, segment_dispersion;
  int rnd_angle, aiming, dispersion;

  int sprite_index2;

  public:
  SlylandroProbe(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  int crewPanelColor();

  virtual void calculate_thrust();
  virtual void calculate_fire_special();
  virtual void calculate_hotspots();
  virtual int activate_weapon();
  virtual void calculate();
	virtual void inflict_damage(SpaceObject *other);
	virtual int accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, double max_speed);
};

int SlylandroProbe::accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, 
		double max_speed) {
	if (source == this) 
		return Ship::accelerate(source, angle, velocity, max_speed);
	return false;
	}

class SlylandroLaser : public Laser {
	int level;
	SpaceLocation *target;
	Ship *ship;
	double base_length, aiming, dispersion;

  public:
  SlylandroLaser(Ship *lship, SpaceLocation *lroot, SpaceLocation *ltarget, 
			int llevel, int b_length, int r_length,
                        int r_angle, int oaiming, int odispersion);

  virtual void calculate();
};

SlylandroProbe::SlylandroProbe(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code),
  frame(0),
  thrustActive(FALSE),
  thrustForward(TRUE)
{
	sprite_index2 = random() & 63;
        segment_length = get_config_int("Weapon", "SegmentLength",0);
        segment_dispersion = get_config_int("Weapon", "SegmentLengthDispersion",0);
        segments = get_config_int("Weapon", "Segments", 0);
        rnd_angle = get_config_int("Weapon", "RandomAngle", 0);
        aiming = get_config_int("Weapon", "Aiming", 0);
        dispersion = get_config_int("Weapon", "Dispersion", 0);
}

int SlylandroProbe::crewPanelColor()
{
  return pallete_color[8];
}

void SlylandroProbe::calculate()
{
	Ship::calculate();

  frame+= frame_time;
  if(frame >= 50) {
    frame -= 50;
    if(thrustForward) {
      sprite_index2++;
      if(sprite_index2 == 64)
        sprite_index2 = 0;
    } else {
      sprite_index2--;
      if(sprite_index2 == -1)
        sprite_index2 = 63;
    }
  }
	sprite_index = sprite_index2;

}

void SlylandroProbe::calculate_hotspots() {
	}
void SlylandroProbe::calculate_fire_special() {
	return;
	}
void SlylandroProbe::calculate_thrust() {
	if (thrust && !thrustActive) {
		angle = angle + 180.0;
		if (angle > 360.0) angle -= 360.0;
		}
	thrustActive = thrust;
	thrust = true;
	Ship::calculate_thrust();
	thrust = thrustActive;
	return;
	}

int SlylandroProbe::activate_weapon()
{

	SpaceLocation *t = NULL;
	double r = 99999;
	int i;
	for (i = 0; i < game->num_targets; i += 1) {
		SpaceObject *s = game->target[i];
		if ((distance(s) < r) && control->valid_target(s)) {
			t = s;
			r = distance(t);
			}
		}
	game->addItem(new SlylandroLaser( this, this, t, segments, segment_length,
                                         segment_dispersion, rnd_angle, aiming, dispersion));

  return(TRUE);
}

void SlylandroProbe::inflict_damage(SpaceObject *other) {
	if (!fire_special || !other->isAsteroid()) {
		Ship::inflict_damage(other);
		return;
		}
	damage_factor = 1;
	Ship::inflict_damage(other);
	damage_factor = 0;
	batt = batt_max;
	play_sound2(data->sampleSpecial[0]);
	return;
	}

SlylandroLaser::SlylandroLaser(Ship *lship, SpaceLocation *lroot, 
		SpaceLocation *ltarget, int llevel, int b_length, int r_length,
                int r_angle, int oaiming, int odispersion) :
		Laser(lship, 0, pallete_color[15], 0.0, 1, 300, lroot, 0.0, 0.0),
		level(llevel), target(ltarget),	ship(lship),
                aiming(oaiming), dispersion(odispersion) {

	base_length = (random()%(2*r_length+1) - r_length) + b_length;
	x = pos->normal_x();
	y = pos->normal_y();
	if (target) angle = trajectory_angle(target);
	else angle = random() % 360;
	if (ship != pos) {
		x += ((SlylandroLaser *)(pos))->edge_x();
		y += ((SlylandroLaser *)(pos))->edge_y();
		}
	angle += (random() % (r_angle*2+1)) - r_angle;
	color = pallete_color[79 - level * 2 - (random()%8)];
	if (level) game->addItem(new SlylandroLaser(ship, this, target, level-1, b_length,
                                         r_length, r_angle, aiming, dispersion));
	}

void SlylandroLaser::calculate() {
	Laser::calculate();
	if (frame < (frame_count/2)) length = base_length * (frame) / frame_count;
	else length = base_length * (frame_count-frame) / frame_count;
	if (!pos->exists()) return;
	x = pos->normal_x();
	y = pos->normal_y();
	if (ship != pos) {
		x += ((SlylandroLaser *)(pos))->edge_x();
		y += ((SlylandroLaser *)(pos))->edge_y();
		}
	if (!target) return;
        double d_a = normalize(trajectory_angle(target) - angle, 360);
        if (d_a > 180) d_a -= 360;
        if (d_a > 0)
                angle += (aiming + dispersion*(-50+random()%101) / sqrt(frame_time)) * frame_time / 100.0;
        else
                angle -= (aiming + dispersion*(-50+random()%101) / sqrt(frame_time)) * frame_time / 100.0;
	if (!target->exists()) target = NULL;
	return;
	}


REGISTER_SHIP(SlylandroProbe)
