#include "../ship.h"
REGISTER_FILE

class ThraddashTorch : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialThrust;
  int          specialDamage;
  int          specialArmour;

  public:
  ThraddashTorch(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate_thrust();
};

class ThraddashFlame : public Animation {
  int armour;

  public:
  ThraddashFlame(double ox, double oy, int odamage, int oarmour, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void handle_damage(SpaceLocation *source);
};

ThraddashTorch::ThraddashTorch(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialThrust = scale_velocity(get_config_float("Special", "Thrust", 0));
  specialDamage = get_config_int("Special", "Damage", 0);
  specialArmour = get_config_int("Special", "Armour", 0);
}

int ThraddashTorch::activate_weapon() {
  game->addItem(new Missile(this, 0.0, (height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, data->spriteWeapon));
  return(TRUE);
}

int ThraddashTorch::activate_special() {
	accelerate(this, angle, specialThrust, GLOBAL_MAXSPEED);
  game->addItem(new ThraddashFlame(
    x ,//- cos(angle * ANGLE_RATIO) * (width() / 2.0),
    y ,//- sin(angle * ANGLE_RATIO) * (height() / 2.0),
    specialDamage, specialArmour, this, data->spriteSpecial, 40, 100));
  return(TRUE);
}

void ThraddashTorch::calculate_thrust()
{
	if(special_recharge <= 0) Ship::calculate_thrust();
}

ThraddashFlame::ThraddashFlame(double ox, double oy, int odamage, int oarmour,
  Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize) :
  Animation(oship, ox, oy, osprite, 0, ofcount, ofsize, LAYER_SHOTS),
  armour(oarmour)
{
	collide_flag_anyone = ALL_LAYERS;
	collide_flag_sameteam = ALL_LAYERS;
  damage_factor = odamage;

  if(!ship->exists())
    state = 0;
}

void ThraddashFlame::calculate()
{
	Animation::calculate();
}

void ThraddashFlame::handle_damage(SpaceLocation *source) {
	if ((damage > 0) || (directDamage > 0))
		armour -= (damage + directDamage);

	if ((armour <= 0) || (source->isObject() && ((SpaceObject*)source)->mass)) {
		state = 0;
		game->addItem(new Animation(this, normal_x(), normal_y(), 
				game->sparkSprite, 0, SPARK_FRAMES, 50, LAYER_EXPLOSIONS));
		}
	}



REGISTER_SHIP(ThraddashTorch)
