#include "../ship.h"
REGISTER_FILE

#include "../frame.h"

class UtwigJugger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  UtwigJugger(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);
  virtual void calculate();
  virtual int isProtected();

  protected:
  virtual void calculate_fire_weapon();
  virtual void animate(Frame *space);
  virtual void handle_damage(SpaceLocation *source);
};

UtwigJugger::UtwigJugger(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
}

void UtwigJugger::calculate()
{
        Ship::calculate();
        if (!fire_special)
                special_recharge = 0;
}

void UtwigJugger::calculate_fire_weapon()
{
  weapon_low = FALSE;

  if(fire_weapon) {
    if((weapon_recharge > 0) || (special_recharge > 0))
      return;
    if (fire_special && batt) return;
    
    game->addItem(new Missile(this, 34.0, 11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));
    game->addItem(new Missile(this, -34.0, 11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));
    game->addItem(new Missile(this, 18.0, 20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));
    game->addItem(new Missile(this, -18.0, 20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));
    game->addItem(new Missile(this, 6.0, 27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));
    game->addItem(new Missile(this, -6.0, 27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, data->spriteWeapon));

    weapon_recharge = weapon_rate;

    play_sound2(data->sampleWeapon[0]);
  }
}

void UtwigJugger::animate(Frame *space) {
  if (special_recharge > 0) {
		sprite->animate_character(x, y, sprite_index, pallete_color[hot_color[((int)(special_recharge/10)) % HOT_COLORS]], space);
		}
  else sprite->animate( x, y, sprite_index, space);
	return;
	}

void UtwigJugger::handle_damage(SpaceLocation *source) {
	if(special_recharge > 0) {
		batt += damage;
		if (batt > batt_max) batt = batt_max;
		damage = 0;
		}
	Ship::handle_damage(source);
	return;
	}

int UtwigJugger::isProtected()
{
        return (special_recharge > 0);
}



REGISTER_SHIP(UtwigJugger)
