#include "../ship.h"
REGISTER_FILE

#include <math.h>

class ZoqFotPikStinger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  int          specialDamage;

  public:
  ZoqFotPikStinger(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
};

class ZoqFotPikShot : public Shot {
  int frame_count;

  public:
  ZoqFotPikShot(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int num_frames);

  virtual void calculate();
};

class ZoqFotPikTongue : public SpaceObject {
  double dist;
  Ship  *ship;
  int    lick_factor;

  int frame;
  int frame_size;
  int frame_count;
  int frame_step;

  public:
  ZoqFotPikTongue(double odist, int odamage, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

ZoqFotPikStinger::ZoqFotPikStinger(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  specialDamage = get_config_int("Special", "Damage", 0);
  if (get_config_int("Special", "Licking", 0) == 0) specialDamage *= -1;
}

int ZoqFotPikStinger::activate_weapon() {
  game->addItem(new ZoqFotPikShot(
    0.0, (height() / 2.0), angle - 10.0 + ((double)(random() % 21) / 1.0),
    weaponVelocity, weaponDamage, weaponRange, weaponArmour, this,
    data->spriteWeapon, 6));
  return(TRUE);
}

int ZoqFotPikStinger::activate_special()
{
  game->addItem(new ZoqFotPikTongue(
    39, specialDamage, this, data->spriteSpecial, 6, 50));
  return(TRUE);
}

ZoqFotPikShot::ZoqFotPikShot(double ox, double oy, double oangle,
  double ov, int odamage, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite, int num_frames) :
  Shot(oship, ox, oy, oangle, ov, odamage, orange, oarmour, oship, osprite),
  frame_count(num_frames)
{
}

void ZoqFotPikShot::calculate()
{
  Shot::calculate();
  sprite_index = (int)((d / range) * (double)(frame_count - 1));
}

ZoqFotPikTongue::ZoqFotPikTongue(double odist, int odamage, Ship *oship,
  SpaceSprite *osprite, int ofcount, int ofsize) :
  SpaceObject(oship, oship->normal_x(), oship->normal_y(), 0.0,
    osprite),
  dist(odist),
  ship(oship),
  lick_factor(odamage),
  frame(0),
  frame_size(ofsize),
  frame_count(ofcount),
  frame_step(0)
{
  layer = LAYER_SHOTS;
  damage_factor = abs(lick_factor);
  x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
  y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
  vx = ship->get_vx(); vy = ship->get_vy();
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
	sprite_index &= 63;
  sprite_index += (64 * frame);
}

void ZoqFotPikTongue::calculate()
{
  int current_frame = frame;

  if (!ship)
    {state = 0; return;}
  if(!ship->exists())
    {state = 0; return;}

  frame_step+= frame_time;
  while(frame_step >= frame_size) {
    frame_step -= frame_size;
    frame++;
    if(frame == frame_count)
      state = 0;
  }
  if((current_frame != frame) && (lick_factor > 0))
    damage_factor = lick_factor;

  x = ship->normal_x() + (cos(ship->get_angle() * ANGLE_RATIO) * dist);
  y = ship->normal_y() + (sin(ship->get_angle() * ANGLE_RATIO) * dist);
  vx = ship->get_vx(); vy = ship->get_vy();
  sprite_index = (int)((ship->get_angle() + 90.0) / 5.625);
	sprite_index &= 63;
  sprite_index += (64 * frame);

  SpaceObject::calculate();
}

void ZoqFotPikTongue::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other); 
  damage_factor = 0;
}



REGISTER_SHIP(ZoqFotPikStinger)
