#include "../ship.h"
REGISTER_FILE

#include "../melee/mmain.h"

#define OWADISABLE_SPEC 0x36

class OwaVoyager : public Ship {

  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponHoming;
  int          weaponArming;
  double       weaponLaunch;

  double       shrapnelRange;
  double       shrapnelVelocity;
  int          shrapnelDamage;
  int          shrapnelArmour;
  double       shrapnelHoming;
  int          shrapnelArming;
  int          shrapnelHotspot;

  double       specialRange;
  double       specialVelocity;
  int          specialArmour;
  int          specialFrames;

public:

  OwaVoyager(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual int activate_weapon();
  virtual int activate_special();
};

class OwaMissile : public HomingMissile {

  int          missileArming;
  int          missileActive;
  double       launch;

  double       shrapnelRange;
  double       shrapnelTurn;
  double       shrapnelVelocity;
  int          shrapnelDamage;
  int          shrapnelArmour;
  double       shrapnelHoming;
  double       shrapnelArming;
  int          shrapnelHotspot;
  SpaceSprite  *shrapnelSprite;

  public:
  OwaMissile(double ox,double oy,double oangle, double ov,
    int odamage, double orange, int oarmour,double otrate , int oarming,
    double olaunch, Ship *oship, SpaceSprite *osprite, double sv, int sdamage,
    double srange, int sarmour, double strate, int sarming,
    SpaceSprite *ssprite,int shotspot);
  virtual void calculate();
};

class OwaShrapnel : public HomingMissile {

  int    missileArming;
  int    missileActive;
  int    hotspot_frame;
  int    hotspot_rate;
  double turnrate;
  public:
OwaShrapnel(double ox,double oy,double oangle, double ov,
  int odamage, double orange, int oarmour, double otrate, int oarming,
  OwaMissile *opos, SpaceSprite *osprite, int ohotspot);
  virtual void calculate();
};

class OwaSpecial : public Missile {
  int disableFrames;

  public:
  OwaSpecial(double ox, double oy, double oangle, double ov,
    int oframes, double orange, int oarmour, Ship *oship,
    SpaceSprite *osprite);

  void inflict_damage(SpaceObject *other);
  void animateExplosion();
};

class OwaDisable : public SpaceObject {
  int   disableframe;
  int   disableframe_count;
  int   frame_step;
  int   frame_size;
  int   frame_count;
  int   lowerindex;

  public:
  OwaDisable(Ship *otarget, OwaSpecial *ocreator, SpaceSprite *osprite, int ofcount,
    int ofsize, int disableFrames, int lowerFrames);

	virtual void target_died();
  virtual void calculate();
  void reset_time();
  Ship *return_disable();
};

OwaVoyager::OwaVoyager(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{
  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponHoming   = get_config_float("Weapon","Homing",0);
  weaponArming   = scale_frames(get_config_int("Weapon","Arming",0));
  weaponLaunch   = scale_range(get_config_float("Weapon", "Launch",0));

  shrapnelRange    = scale_range(get_config_float("Shrapnel", "Range", 0));
  shrapnelVelocity = scale_velocity(get_config_float("Shrapnel", "Velocity", 0));
  shrapnelDamage   = get_config_int("Shrapnel", "Damage", 0);
  shrapnelArmour   = get_config_int("Shrapnel", "Armour", 0);
  shrapnelHoming   = get_config_float("Shrapnel","Homing",0);
  shrapnelArming   = scale_frames(get_config_int("Shrapnel","Arming",0));
  shrapnelHotspot  = scale_frames(get_config_int("Shrapnel","Hotspot",0));

  specialRange    = scale_range(get_config_float("Special", "Range", 0));
  specialVelocity = scale_velocity(get_config_float("Special", "Velocity", 0));
  specialArmour   = get_config_int("Special", "Armour", 0);
  specialFrames   = scale_frames(get_config_int("Special", "Frames", 0));
}

int OwaVoyager::activate_weapon()
{
  game->addItem(new OwaMissile(0,height()/2,angle,weaponVelocity,
    weaponDamage,weaponRange,weaponArmour,weaponHoming,weaponArming,
    weaponLaunch,this, data->spriteWeapon, shrapnelVelocity,shrapnelDamage,
    shrapnelRange,shrapnelArmour,shrapnelHoming,shrapnelArming,data->spriteExtra,
    shrapnelHotspot));
  return(TRUE);
}

int OwaVoyager::activate_special()
{
  game->addItem( new OwaSpecial(0.0, -(height() / 2.0), angle-180,
    specialVelocity, specialFrames,specialRange, specialArmour, this,
    data->spriteSpecial));
  return(TRUE);
}

OwaMissile::OwaMissile(double ox,double oy,double oangle, double ov,
    int odamage, double orange, int oarmour,double otrate , int oarming,
    double olaunch, Ship *oship, SpaceSprite *osprite, double sv, int sdamage,
    double srange, int sarmour, double strate, int sarming,
    SpaceSprite *ssprite,int shotspot) : HomingMissile(oship, ox, oy, oangle, ov,
    odamage, orange, oarmour, otrate, oship, osprite, oship->target)

{
  missileArming = oarming;
  missileActive = FALSE;
  launch = olaunch;
  shrapnelRange    = srange;
  shrapnelVelocity = sv;
  shrapnelDamage   = sdamage;
  shrapnelArmour   = sarmour;
  shrapnelHoming   = strate;
  shrapnelArming   = sarming;
  shrapnelSprite   = ssprite;
  shrapnelHotspot  = shotspot;
}

void OwaMissile::calculate() {
	HomingMissile::calculate();
	if (!missileActive) {
		missileArming -= frame_time;
		if (missileArming <= 0)
			missileActive = TRUE;
		} 
	else if (target) if ((distance(target) < launch) || (d >= range)) {
		int deg = 35;
		for (int i=0;i <8;i++)  {
			OwaShrapnel *shrap;
			shrap = (new OwaShrapnel(0,0,angle+deg ,shrapnelVelocity,
					shrapnelDamage, shrapnelRange, shrapnelArmour, shrapnelHoming,
					shrapnelArming, this, shrapnelSprite, shrapnelHotspot));
			deg -= 10;
			game->addItem(shrap);
			}
		play_sound(data->sampleWeapon[1]);
		state = 0;
		}
	return;
	}

OwaShrapnel::OwaShrapnel(double ox,double oy,double oangle, double ov,
    int odamage, double orange, int oarmour, double otrate, int oarming,
    OwaMissile *opos, SpaceSprite *osprite, int ohotspot) :
    HomingMissile(opos, ox, oy, oangle, ov, odamage, orange, oarmour, 0,
    opos, osprite, opos->target)
{
   missileArming = oarming;
   missileActive = FALSE;
   turnrate = otrate;
   hotspot_rate = ohotspot;
   hotspot_frame = 0;
}

void OwaShrapnel::calculate()
{
  if (!missileActive) {
    missileArming -= frame_time;
    if (missileArming <= 0) {
      missileActive = TRUE;
      turn_rate = turnrate; }}
  if (hotspot_frame <= 0) {
    game->addItem(new Animation(this, 
      normal_x() - (cos(angle * ANGLE_RATIO) * w / 2.5),
      normal_y() - (sin(angle * ANGLE_RATIO) * h / 2.5),
      game->hotspotSprite, 0, HOTSPOT_FRAMES, time_ratio, LAYER_HOTSPOTS));
    hotspot_frame += hotspot_rate;
  }
  if (hotspot_frame > 0) hotspot_frame -= frame_time;
  HomingMissile::calculate();
  return;
}

OwaDisable::OwaDisable(Ship *otarget, OwaSpecial *ocreator, SpaceSprite *osprite,
  int ofcount, int ofsize, int disableFrames,int lowerFrames) :
  SpaceObject(ocreator, otarget->normal_x(), otarget->normal_y(), 0.0, osprite),
  disableframe(lowerFrames),
  disableframe_count(disableFrames),
  frame_step(0),
  frame_size(ofsize),
  frame_count(ofcount),
  lowerindex(lowerFrames)
{
	target = otarget;
	id = OWADISABLE_SPEC;
	sprite_index = lowerFrames;
	collide_flag_anyone = 0;
	layer = LAYER_SPECIAL;
}
void OwaDisable::target_died() {
	state = 0;
	return;
	}
void OwaDisable::calculate() {
	frame_step+= frame_time;
	while (frame_step >= frame_size) {
		frame_step -= frame_size;
		sprite_index++;
		if(sprite_index == (lowerindex + frame_count))
			sprite_index = lowerindex;
		}
	if (!lowerindex) {
		x = target->normal_x();
		y = target->normal_y();
		((Ship*)target)->nextkeys &= ~(keyflag::left | keyflag::right | keyflag::thrust);
		}
	else {
		x = target->normal_x();
		y = target->normal_y();
		}
	disableframe += frame_time;
	if (disableframe >= disableframe_count) state = 0;
	SpaceObject::calculate();
	return;
	}

Ship *OwaDisable::return_disable()
{
  return (ship);
}

void OwaDisable::reset_time()
{
  disableframe = 0;
}

OwaSpecial::OwaSpecial(double ox, double oy, double oangle,
  double ov, int oframes, double orange, int oarmour, Ship *oship,
  SpaceSprite *osprite) : Missile (oship, ox, oy, oangle, ov, 0, orange, oarmour, oship, osprite),
  disableFrames(oframes)
{}

void OwaSpecial::animateExplosion() {}

void OwaSpecial::inflict_damage(SpaceObject *other) {
          if (other->isShip()) {
            play_sound(data->sampleExtra[0]);
            SpaceObject *o = NULL;
            Query a;
            int found = FALSE;
            for (a.begin(this, ALL_LAYERS ,distance(other)+ 10);
              a.current; a.next()) {
		o = a.currento;
		if ((o->getID()) == OWADISABLE_SPEC)
		  if ((((OwaDisable *)o)->return_disable()) == other) {
			((OwaDisable *)o)->reset_time();
                        found = TRUE;
	  }
          if (!found) {
          game->addItem(new OwaDisable(
            (Ship *)(other), this, data->spriteExtraExplosion, 32, 40, disableFrames, 0));
          game->addItem(new OwaDisable(
            (Ship *)(other), this, data->spriteExtraExplosion, 32, 40, disableFrames, 32));
          }  }   }
        state = 0;
	return;
}




REGISTER_SHIP(OwaVoyager)
