#include "../ship.h"
REGISTER_FILE

#include "../frame.h"

class PloxisPlunderer : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponTurn;

  int          specialFrames;
  int          shieldFrames;

  double abss(double value);

  public:
  PloxisPlunderer(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  protected:
  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
  virtual void handle_damage(SpaceLocation *source);
};

PloxisPlunderer::PloxisPlunderer(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{

  weaponRange    = scale_range(get_config_float("Weapon", "Range", 0));
  weaponVelocity = scale_velocity(get_config_float("Weapon", "Velocity", 0));
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponTurn     = scale_turning(get_config_float("Weapon", "Turning", 0));

  specialFrames = get_config_int("Special", "Frames", 0);
  shieldFrames  = 0;
}

double PloxisPlunderer::abss(double value)
{
  if (value < 0)
    return (value * -1);
  else return (value);
}

int PloxisPlunderer::activate_weapon()
{
  game->addItem(new HomingMissile(this, 0, 35,
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    weaponTurn, this, data->spriteWeapon, target));
  return(TRUE);
}

int PloxisPlunderer::activate_special()
{
  shieldFrames = (shieldFrames % frame_time) + specialFrames;
  sprite = data->spriteSpecial;
  return(TRUE);
}

void PloxisPlunderer::calculate()
{
   if(shieldFrames > 0) {
     recharge_amount = 0;
     shieldFrames-= frame_time;
     Query a;
     for (a.begin(this, bit(LAYER_SHOTS),250); a.current; a.next()) {
       SpaceObject *o = a.currento;
       if ((!o->sameShip(this)) && (o->isShot()) && ((distance(o) 
				 - abss(cos(trajectory_angle(o) * ANGLE_RATIO) * (height() /2))
         - abss(sin(trajectory_angle(o) * ANGLE_RATIO) * (width()/3))
         - abss(sin(o->trajectory_angle(this) * ANGLE_RATIO) * (o->height()/2))
         - abss(cos(o->trajectory_angle(this) * ANGLE_RATIO) * (o->width()/2))
         - sqrt(((o->get_vy()+vy)*(o->get_vy()+vy))+
         ((o->get_vx()+vx)*(o->get_vx()+vx)))) < 0 )) {
          Shot *other= (Shot*)o;
          other->change_owner(this);
          other->changeDirection((other->get_angle()-
            trajectory_angle(other))*-1+(trajectory_angle(other)+180));
            if ((other)->isHomingMissile())
          ((HomingMissile*) other)->target = this->target;
          play_sound(data->sampleSpecial[1]);
          }
        }
   if (shieldFrames <= 0)
     sprite = data->spriteShip;
     }
   else recharge_amount = 1;
   Ship::calculate();
}

void PloxisPlunderer::handle_damage(SpaceLocation *source)
{
  if(shieldFrames > 0)
      damage = 0;
  Ship::handle_damage(source);
}




REGISTER_SHIP(PloxisPlunderer)
