#include "../ship.h"
REGISTER_FILE

struct hitlist {
  SpaceObject *item;
  int          endframe;
  struct hitlist *next;
  };
void hitlist_clear(hitlist **list) {
	while (true) {
		if (!*list) break;
		hitlist *tmp = *list;
		*list = (*list)->next;
		delete tmp;
		}
	return;
	}
hitlist *hitlist_check(hitlist **list, SpaceObject *item) {
	while (true) {
		if (!*list) break;
		if ((*list)->endframe > game->game_time ) {
			if ((*list)->item == item) return *list;
			list = &(*list)->next;
			}
		else {
			hitlist *tmp = *list;
			*list = (*list)->next;
			delete tmp;
			}
		}
	return NULL;
	}
bool hitlist_hit(hitlist **list, SpaceObject *item, int duration) {
	if (hitlist_check(list, item)) return false;
	hitlist *old = *list;
	*list = new hitlist;
	(*list)->item = item;
	(*list)->endframe = game->game_time + duration;
	(*list)->next = old;
	return true;
	}

class VyroIngoSheild;

class VyroIngoInvader : public Ship {

  public:

  VyroIngoSheild *sheild;
  int    weaponDamage;
  int    weaponFrames;
  int    specialDamage;
  int    specialFrames;

  public:
  VyroIngoInvader(double ox, double oy, double shipAngle,
    ShipData *shipData, unsigned int code);

  virtual void calculate();
  virtual int activate_weapon();
  virtual int activate_special();
};

class VyroIngoSheild : public SpaceObject {

  int frames_left;
  int frames_total;
  hitlist *list;
  int dam;
  Ship *ship;

public:

VyroIngoSheild(double ox,double oy,int odamage,int oframes, Ship *oship,
  SpaceSprite *osprite);

void reset_time();
virtual void calculate();
virtual void inflict_damage(SpaceObject *other);
virtual void death();
};

class VyroIngoWake : public AnimatedShot {

  int frames_total;
  int frames_left;
  SpaceSprite *mid;
  SpaceSprite *end;
  int status;
  public:

  VyroIngoWake (double ox,double oy,int odamage,int oframes,Ship *oship,
    SpaceSprite *osprite,SpaceSprite *ostartsprite, SpaceSprite *oendsprite);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

VyroIngoInvader::VyroIngoInvader(double ox, double oy, double shipAngle,
	ShipData *shipData, unsigned int code)
	:
	Ship(ox, oy, shipAngle, shipData, code)
{
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialFrames   = scale_frames(get_config_int("Special", "Frames", 0));
  sheild = NULL;
}

int VyroIngoInvader::activate_weapon()
{
  if (sheild)
    if (sheild->exists())
      sheild->reset_time();
  if ((!sheild) || (!sheild->exists())) {
    sheild = new VyroIngoSheild(normal_x(), normal_y(),
      weaponDamage,weapon_rate,this,data->spriteWeapon);
    game->addItem(sheild);
    }
  return(TRUE);
}

int VyroIngoInvader::activate_special()
{
  game->addItem(new VyroIngoWake(0,-(height()/2+32),specialDamage,
    specialFrames,this,data->spriteSpecial,data->spriteExtra,
    data->spriteExtraExplosion));
  return(TRUE);
}
void VyroIngoInvader::calculate()

{
  if (sheild)
    if (sheild->exists()) {
      if (!fire_weapon)
        sheild->state = 0;
    } else sheild = NULL;
  Ship::calculate();
}

VyroIngoSheild::VyroIngoSheild (double ox,double oy, int odamage, int oframes,
  Ship *oship,SpaceSprite *osprite) : SpaceObject (oship,ox,oy,0,osprite),
  frames_left(0),
  frames_total(oframes),
  list(NULL),
  dam(odamage),
  ship(oship)
{
  layer = LAYER_SPECIAL;
  damage_factor = odamage;
  sprite_index = 1;
	vx = ship->get_vx();
	vy = ship->get_vy();
}

void VyroIngoSheild::calculate()
{
  if(!ship->exists())
    state = 0;

  if(frames_left >= frames_total)
    state = 0;

  frames_left += frame_time;

  sprite_index = 1;

  x = ship->normal_x();
  y = ship->normal_y();
	vx = ship->get_vx();
	vy = ship->get_vy();
  SpaceObject::calculate();
}

void VyroIngoSheild::inflict_damage(SpaceObject *other) {
	damage_factor = dam;
	if (!hitlist_hit(&list, other, frames_total)) {
		damage_factor = 0;
		}
	SpaceObject::inflict_damage(other);
	return;
	}

void VyroIngoSheild::death() {
	hitlist_clear(&list);
	SpaceObject::death();
	return;
	}

void VyroIngoSheild::reset_time()
{
  frames_left = 0;
}

VyroIngoWake::VyroIngoWake (double ox,double oy,int odamage,int oframes,
  Ship *oship, SpaceSprite *osprite, SpaceSprite *ostartsprite,
    SpaceSprite *oendsprite) : AnimatedShot(oship, ox,oy,0,0,odamage,-1.0,99,
    oship, ostartsprite,60,30),
  frames_total(oframes),
  frames_left(0),
  mid(osprite),
  end(oendsprite),
  status(1)
{
  mass = 1000;
}

void VyroIngoWake::calculate()
{
  vx = 0;
  vy = 0;
  AnimatedShot::calculate();
  if ((status == 1) && (sprite_index == 59))
    {
      status = 2;
      frame_count = 20;
      sprite_index = 0;
      sprite = mid;
    }
  frames_left+=frame_time;
  if (frames_left >= frames_total) {
    if ((sprite_index == 19) && (status == 2))
      {
      status = 3;
      frame_count = 64;
      sprite_index = 0;
      sprite = end;
      }
    if ((status == 3) && (sprite_index == 62))
      state = 0;
   }
  vx = 0;
  vy = 0;
}

void VyroIngoWake::inflict_damage(SpaceObject *other)

{
  if ((other->isShip()) || (other->isAsteroid()))
    SpaceObject::inflict_damage(other);
  else AnimatedShot::inflict_damage(other);
  vx=0;
  vy=0;
}




REGISTER_SHIP(VyroIngoInvader)
