/* modified by orz for Star Control: TimeWarp 
*/

/*
 * aastr.c --- anti-aliased stretching for Allegro
 *
 * This file is gift-ware.  This file is given to you freely
 * as a gift.  You may use, modify, redistribute, and generally hack
 * it about in any way you like, and you do not have to give anyone
 * anything in return.
 *
 * I do not accept any responsibility for any effects, adverse or
 * otherwise, that this code may have on just about anything that
 * you can think of.  Use it at your own risk.
 *
 * Copyright (C) 1998, 1999  Michael Bukin
 */

#include <allegro.h>
#ifdef __DJGPP__
#include <sys/farptr.h>
#endif

#include "aastr.h"
#include "aautil.h"

#include "../melee.h"
#include "math.h"

/*
 * modified Engine of anti-aliased stretching
Takes non-integer coordinates for x,y,w, & h.  
Easier to read (doesn't use those evil macros).  
 */
void _aa2_stretch_blit (BITMAP *src, BITMAP *dest, double _x, double _y, 
		double _w, double _h, int masked) {
	unsigned long addr;
	int x, y, w, h;
	void (*add) (BITMAP *_src, int _sx1, int _sx2, int _sy1, int _sy2, unsigned long _num);
	void (*put) (unsigned long _addr, int _x);

	x = (int)floor(ldexp(_x, aa_BITS));
	y = (int)floor(ldexp(_y, aa_BITS));
	w = (int)floor(ldexp(_w, aa_BITS));
	h = (int)floor(ldexp(_h, aa_BITS));
	if ((w < 1) || (h < 1)) throw "too narrow or too short";
	if (!is_memory_bitmap((BITMAP*)src) || is_planar_bitmap(dest) 
			|| is_system_bitmap(dest)) 
		throw "not the right kind of bitmaps";
	add = get_aa_add_function(bitmap_color_depth(src), 1);
	put = get_aa_put_function(bitmap_color_depth(dest), 1);
	int y1, y2, x1, x2;
	y1 = y       >> aa_BITS;
	y2 = (y + h) >> aa_BITS;
	x1 = x       >> aa_BITS;
	x2 = (x + w) >> aa_BITS;
	int cx, cy, cw, ch;
	cw = (src->w << (aa_BITS*2)) / w;
	ch = (src->h << (aa_BITS*2)) / h;
	int iy, ix;
	bmp_select(dest); //this helps if dest is a video bitmap
	int yscale = (((src->h) << (2 * aa_BITS))-1) / h;
	int ybase = (y * yscale) >> aa_BITS;
	int xscale = (((src->w) << (2 * aa_BITS))-1) / w;
	int xbase = (x * yscale) >> aa_BITS;
	int mw = src->w << aa_BITS;
	int mh = src->h << aa_BITS;
	if (cw < aa_SIZE) cw = aa_SIZE;
	if (ch < aa_SIZE) ch = aa_SIZE;
	int _area = cw * ch;
	if (dest->clip) {
		if (y1 < dest->ct) y1 = dest->ct;
		if (y2 >= dest->cb) y2 = dest->cb - 1;
		if (x1 < dest->cl) x1 = dest->cl;
		if (x2 >= dest->cr) x2 = dest->cr - 1;
		}
	for (iy = y1; iy <= y2; iy += 1) {
		cy = iy * yscale - ybase;
		addr = bmp_write_line(dest, iy); //this helps if dest is a video bitmap
		int th = ch;
		if (cy < 0) { //top edge of image
			th += cy;
			cy = 0;
			}
		if (cy + th > mh) { //bottom edge of image
			th = mh - cy;
			}
		if (th < aa_SIZE) { //either edge
			cy -= (aa_SIZE - th) / 2;
			th = aa_SIZE;
			if (cy < 0) cy = 0;
			if (cy > (int)(mh - aa_SIZE)) cy = mh - aa_SIZE;
			}
		for (ix = x1; ix <= x2; ix += 1) {
			cx = ix * xscale - xbase;
			int tw = cw;
			if (cx < 0) {//left edge of image
				tw += cx;
				cx = 0;
				}
			if (cx + tw > mw) {//right edge of image
				tw = mw - cx;
				}
			if (tw < aa_SIZE) {//either edge of image
				cx -= (aa_SIZE - tw) / 2;
				tw = aa_SIZE;
				if (cx < 0) cx = 0;
				if (cx > (int)(mw - aa_SIZE)) cx = mw - aa_SIZE;
				}
			add ((BITMAP *)src, cx, cx + tw, cy, cy + th, _area);
			put (addr, ix);
			}
		bmp_unwrite_line(dest); //this helps if dest is a video bitmap
		}
	return;
	}

/*
 * Engine of anti-aliased stretching.
 */
void _aa_stretch_blit (BITMAP *_src, BITMAP *_dst,
		int _sx, int _sy, int _sw, int _sh,
		int _dx, int _dy, int _dw, int _dh, int _masked)
{
  int sx, sy, dx, dy, ydx, ysx;
  int xinc, yinc, dsx, dsy;
  int xi1, xi2, xdd, yxdd;
  int yi1, yi2, ydd;
  int dxbeg, dxend, dybeg, dyend;
  unsigned long num;
  void (*add) (BITMAP *_src, int _sx1, int _sx2, int _sy1, int _sy2, unsigned long _num);
  void (*put) (unsigned long _addr, int _x);

  if ((_dw <= 0) || (_dh <= 0) || (_sw <= 0) || (_sh <= 0))
    return;

  if (_dst->clip)
    {
      dybeg = ((_dy > _dst->ct) ? _dy : _dst->ct);
      dyend = (((_dy + _dh) < _dst->cb) ? (_dy + _dh) : _dst->cb);
      if (dybeg >= dyend)
	return;

      dxbeg = ((_dx > _dst->cl) ? _dx : _dst->cl);
      dxend = (((_dx + _dw) < _dst->cr) ? (_dx + _dw) : _dst->cr);
      if (dxbeg >= dxend)
	return;
    }
  else
    {
      dxbeg = _dx;
      dybeg = _dy;
      dxend = _dx + _dw;
      dyend = _dy + _dh;
    }

  _sx <<= aa_BITS;
  _sw <<= aa_BITS;
  dsx = _sw / _dw;

  if (dsx < aa_SIZE)
    {
      /* Exploding by x.  */
      _dw--;
      _sw -= aa_SIZE;
      dsx = aa_SIZE;
    }

  _sy <<= aa_BITS;
  _sh <<= aa_BITS;
  dsy = _sh / _dh;

  if (dsy < aa_SIZE)
    {
      /* Exploding by y.  */
      _dh--;
      _sh -= aa_SIZE;
      dsy = aa_SIZE;
    }

  num = dsx * dsy;

  if (num > aa_MAX_NUM)
    {
      if (dsx > aa_MAX_SIZE)
	dsx = aa_MAX_SIZE;
      if (dsy > aa_MAX_SIZE)
	dsy = aa_MAX_SIZE;
      num = dsx * dsy;
    }

  /* Walk in x direction up to dxbeg and save Bresenham state there.
   * Later, it will be used to restart at any line.  */
  aa_PREPARE (xinc, yxdd, xi1, xi2, _sw, _dw);
  for (ydx = _dx, ysx = _sx; ydx < dxbeg; ydx++)
    {
      aa_ADVANCE (ysx, xinc, yxdd, xi1, xi2);
    }

  /* Color manipulation routines.  */
  if (is_screen_bitmap ((BITMAP *)_src))
    return;
  else
    {
      add = get_aa_add_function(bitmap_color_depth (_src), _masked);
    }

  if (is_planar_bitmap (_dst))
    return;
  else
    {
      put = get_aa_put_function(bitmap_color_depth (_dst), _masked);
    }

  /* Walk in y until we reach first non-clipped line.  */
  aa_PREPARE (yinc, ydd, yi1, yi2, _sh, _dh);
  for (dy = _dy, sy = _sy; dy < dybeg; dy++)
    {
      aa_ADVANCE (sy, yinc, ydd, yi1, yi2);
    }

  _farsetsel (_dst->seg);

  /* Stretch all non-clipped lines.  */
  for (; dy < dyend; dy++)
    {
      unsigned long daddr = bmp_write_line (_dst, dy);

      for (dx = ydx, sx = ysx, xdd = yxdd; dx < dxend; dx++)
	{
	  (*add) (_src, sx, sx + dsx, sy, sy + dsy, num);
	  (*put) (daddr, dx);

	  aa_ADVANCE (sx, xinc, xdd, xi1, xi2);
	}

      aa_ADVANCE (sy, yinc, ydd, yi1, yi2);
    }
}

/*
 * Anti-aliased bitmap stretching with blit.
 */
void
aa_stretch_blit (BITMAP *_src, BITMAP *_dst,
		 int _sx, int _sy, int _sw, int _sh,
		 int _dx, int _dy, int _dw, int _dh)
{
  _aa_stretch_blit (_src, _dst, _sx, _sy, _sw, _sh, _dx, _dy, _dw, _dh, 0);
}

/*
 * Anti-aliased bitmap stretching with blit (masked).
 */
void
aa_stretch_sprite (BITMAP *_dst, BITMAP *_src, int _dx, int _dy, int _dw, int _dh)
{
  _aa_stretch_blit (_src, _dst, 0, 0, _src->w, _src->h, _dx, _dy, _dw, _dh, 1);
}

/*
 * aastr.c ends here
 */
